/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <string>
#include <net/ethernet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>

using namespace std;

class Packet_Client
/** zur Verbindung eines externen Programms mit dem Paketserver und zum Senden
 *  von Paketen an ihn */
{
	private:
		/** Statusvariable, ob gerade eine Verbindung zum Server
		 *  besteht */
		bool			is_connected;
		/** die eigentliche Verbindung zum Server */
		int			client_socket;
		/** Portnummer, &uuml;ber die der Server erreichbar ist */
		in_port_t		server_port;
		/** IP-Adresse des Servers */
		in_addr_t		server_ip;
		/** Name des Dienstes, den die geschickten Pakete simulieren
		 *  sollen, wird zum Server geschickt, der Sie auswerten kann
		 *  und bei Bedarf (siehe TcpQueue /
 		 *  UdpQueue::findServicePort())
		 *  in Portnummern umwandelt (wenn Paket-Quellport == 0) */
		string			service_name;

		/** Ethernet-Kopf des zu schickenden Paketes */
		struct ether_header	eth_header;
		/** IPv4-Kopf des zu schickenden Paketes */
		struct ip		ip_header;
		/** TCP-Kopf des zu schickenden Paketes */
		struct tcphdr		tcp_header;
		/** UDP-Kopf des zu schickenden Paketes */
		struct udphdr		udp_header;

		int		sendServiceName(void) const;
		void		updateIpChecksum(void);
		void		updateTcpChecksum(void);
		void		updateUdpChecksum(void);
	public:
		Packet_Client(void);
		~Packet_Client(void);
		int	init(const char new_server_ip[],
				unsigned short new_server_port);
		int	connect(void);
		int	close(void);

		//@{ @name Senden
		int	sendRawPacket(const char *packet_buffer,
					size_t packet_size) const;
		int	sendRawWithService(const char *packet_buffer,
					size_t packet_size) const;
		int	sendTcpPacket(void);
		int	sendUdpPacket(void);
		//@}

		//@{ @name IP-, TCP-, UDP- und Dienst-Einstellungen
		int	setSourceIpAddress(const char new_source_ip[]);
		int	setDestinationIpAddress(
					const char new_destination_ip[]);
			// type of service OR DSCP, Packet id, time to live 
		void	setIpHeader(u_int8_t tos, u_int16_t id, u_int8_t ttl);

		void	setSourcePort(in_port_t new_source_port);
		void	setDestinationPort(in_port_t new_destination_port);
			// Sequence Nr., Acknowledgement Number,
			// Window size, urgent pointer
		void	setTcpHeader(u_int32_t seq, u_int32_t ack,
					u_int16_t window, u_int16_t urgent_p);
		void	setTcpFlags(bool fin, bool sync, bool reset, bool push,
					bool ack, bool urgent);

		void	setServiceName(const char new_service_name[]);
		//@}
};

