/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <list>
#include <vector>

using namespace std;

#include "ipv4_packet.h"

class PacketServer
/** wartet auf Verbindungen von Klienten (z.B. mit Packet_Client), welche
 *  Pakete in die Simulation eines Netzwerkes einbauen wollen */
{
	private:
		/** Portnummer, &uuml;ber die sich Klienten verbinden
		 *  k&ouml;nnen */
		unsigned short	server_port;
		/** Socket, mit dem auf Klienten-Verbindungen gewartet wird */
		int		server_socket;
		/** der zuletzt vom Klient geschickte Dienstname, zu dem das
		 *  Paket geh&ouml;ren soll, wird im Paket eingetragen,
		 *  angezeigt und bei Quellport == 0 in Portnummer
		 *  umgewandelt */
		string		current_client_service_name;
		/** alle gerade offenen Verbindungen mit Klienten */
		vector<int>	client_sockets;
		/** Liste aller geschickten Pakete, die noch nicht in der
		 *  Simulation eingebaut wurden (m&uuml;ssen noch von der
		 *  Simulationsschleife in Network::executeNet() geholt
		 *  werden) */
		list<Packet*>	incoming_packets;

		int	startServer(void);
		int	serveClient(int &current_client_socket);
		int	stopServer(void);
		void	delAllPackets(void);

		//@{ @name Pakete einlesen
		bool	findServiceNameForPort(unsigned short port,
						const char protocol[],
						string &service_name);
		void	readCommandPacket(unsigned char* &packet_buffer,
						size_t &buffer_length);
		void	readEthernetPacket(unsigned char* &packet_buffer,
						size_t &buffer_length,
						Packet* &cable_packet);
		void	readIpPacket(unsigned char* &packet_buffer,
						size_t &buffer_length,
						Ethernet_Packet* &eth_packet);
		void	readTcpPacket(unsigned char* packet_buffer,
						size_t packet_length,
						IPv4_Packet* &ip_packet);
		void	readUdpPacket(unsigned char* packet_buffer,
						size_t packet_length,
						IPv4_Packet* &ip_packet);
		//@}

	public:
		PacketServer(void);
		~PacketServer(void);
		void	init(unsigned short new_server_port);
		void	serverStep(void);
		void	getPacket(Packet* &cable_packet);
		size_t	numPackets(void) const;
};

