/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef CONSTANTS_H
#define CONSTANTS_H 1

#include <GL/gl.h>
#include <libxml/tree.h>

// XML names case insensitive here
/** bei XML-Namen ist es hier egal, ob Gro- oder Kleinschreibung */

//@{ @name IP-Adresse, Ethernet-Adresse, Kabelattribute
// ipqueue
/** Name des XML-Attributs im Tag 'address' */
const xmlChar xml_syntax_ipaddress[] = "ipaddr";
// net_cable, ethernetqueue
/** Name des XML-Attributs in den Tags 'address', 'begin' und 'end'   */
const xmlChar xml_syntax_macaddress[] = "macaddr";
// net_cable
/** Name des XML-Tags innerhalb des Tags 'cable' (Kabel von wo nach wo) */
const xmlChar xml_syntax_begin[] = "begin";
/** Name des XML-Tags innerhalb des Tags 'cable' (Kabel von wo nach wo) */
const xmlChar xml_syntax_end[] = "end";
//@}

//@{ @name Haupttag, Netzwerk-Bestandteile
// network
/** Name des globalen Tags, in dem alle anderen verwendet werden */
const xmlChar  xml_syntax_netview[] = "netview";
/** Name des Tags unter 'netview' (Kabel) */
const xmlChar  xml_syntax_cable[] = "cable";
/** Name des Tags unter 'netview' (Computer / Ger&auml;t) */
const xmlChar  xml_syntax_computer[] = "computer";
/** Name des Tags unter 'netview' (Router / Ger&auml;t) */
const xmlChar  xml_syntax_router[] = "router";
//@}

//@{ @name Position, Adresse und Namen
// point_3d
/** Name des Attributs von 'pos' 
 * (Position von Ger&auml;ten / Kabeln / Paketen) */
const xmlChar xml_syntax_point[] = "point";
// stack
/** Name des Tags innerhalb der Defintion zu den Ger&auml;ten / Kabeln /
 *  Paketen (Position) */
const xmlChar xml_syntax_pos[] = "pos";
/** Name des Tags innerhalb der Ger&auml;te-/Paketdefinition (IP-Adresse) */
const xmlChar xml_syntax_address[] = "address";
/** Name des Tags f&uuml;r 'Namen' von Interfaces, Host, Router, 
 *  Route::Interface, Menu, Menu Entry, Action */
const xmlChar xml_syntax_name[] = "name";
//@}

//@{ @name Warteschlangen
/** Name des Tags f&uuml;r TcpQueue */
const xmlChar xml_syntax_tcp_queue[] = "tcp_queue";
/** Name des Tags f&uuml;r UdpQueue */
const xmlChar xml_syntax_udp_queue[] = "udp_queue";
/** Name des Tags f&uuml;r IpQueue */
const xmlChar xml_syntax_ip_queue[] = "ip_queue";
/** Name des Tags f&uuml;r EthernetQueue */
const xmlChar xml_syntax_ethernet_queue[] = "ethernet_queue";
/** Name des Tags f&uuml;r Up_Queue */
const xmlChar xml_syntax_up_queue[] = "up_queue";
/** Name des Tags f&uuml;r Down_Queue */
const xmlChar xml_syntax_down_queue[] = "down_queue";
//@}

//@{ @name Netzwerkkarte (Interface)
/** Name des Tags f&uuml;r Interface (Netzwerkkarte) */
const xmlChar xml_syntax_interface[] = "interface";
/** Name des Attributes f&uuml;r Interface::sending_arp_packet */
const xmlChar xml_syntax_interface_sending_arp_packet[] = "sending_arp_packet";
//@}

//@{ @name NAT
/** Name des Tags f&uuml;r NAT */
const xmlChar xml_syntax_nat[] = "nat";
/** Name des Attributes zum Beginn des Portbereiches in NAT */
const xmlChar xml_syntax_nat_minport[] = "minport";
/** Name des Attributes zum Ende des Portbereiches in NAT */
const xmlChar xml_syntax_nat_maxport[] = "maxport";
/** Name des Tags zum Quellnetzwerk in NAT (Attribute ipaddr) */
const xmlChar xml_syntax_nat_source_network[] = "source_network";
/** Name des Tags zur Quellnetzmaske in NAT (Attribute ipaddr) */
const xmlChar xml_syntax_nat_source_netmask[] = "source_netmask";
/** Name des Tags zum Zielnetzwerk in NAT (Attribute ipaddr) */
const xmlChar xml_syntax_nat_target_network[] = "target_network";
/** Name des Tags zur Zielnetzmaske in NAT (Attribute ipaddr) */
const xmlChar xml_syntax_nat_target_netmask[] = "target_netmask";
//@}

//@{ @name ARP-Cache und Routentabelle
/** Name des Tags f&uuml;r ArpCache */
const xmlChar xml_syntax_arp_cache[] = "arp_cache";
/** Name des Tags f&uuml;r Route_Table */
const xmlChar xml_syntax_route_table[] = "route_table";
/** Name des Tags f&uuml;r Route */
const xmlChar xml_syntax_route[] = "route";
/** Name des Tags f&uuml;r das Zielnetzwerk in Routen */ 
const xmlChar xml_syntax_route_destination[] = "destination";
/** Name des Tags f&uuml;r die Gateway-Adresse in Routen */ 
const xmlChar xml_syntax_route_gateway[] = "gateway";
/** Name des Tags f&uuml;r die Zielnetzmaske in Routen */ 
const xmlChar xml_syntax_route_genmask[] = "genmask";
/** Name des Tags f&uuml;r die Netzwerkkarte in Routen */ 
const xmlChar xml_syntax_route_interface[] = "interface_device";
//@}

//@{ @name Men&uuml;
/** Name des Tags f&uuml;r Men&uuml; (Menu) */
const xmlChar xml_syntax_menu[] = "menu";
/** Name des Tags f&uuml;r einen Men&uuml;eintrag */
const xmlChar xml_syntax_menu_entry[] = "entry";
/** Name des Tags f&uuml;r die Aktion eines Men&uuml;eintrags */
const xmlChar xml_syntax_menu_action[] = "action";
//@}

//@{ @name Paket (Packet)
/** Name des Tags f&uuml;r Pakete (Packet) */
const xmlChar xml_syntax_packet[] = "packet";
/** Name des Tags f&uuml;r die Anfangsposition des Paketes */
const xmlChar xml_syntax_packet_first_pos[] = "first_pos";
/** Name des Attributes f&uuml;r den Rotationswinkel des Paketes */
const xmlChar xml_syntax_packet_angle[] = "angle";
/** Name des Attributes f&uuml;r den zur&uuml;ckgelegten Weg des Paketes */
const xmlChar xml_syntax_packet_moved_way[] = "moved_way";
/** Name des Tags f&uuml;r die Position eines Kopfes des Paketes */
const xmlChar xml_syntax_packet_header_pos[] = "header_pos";
/** Name des Tags f&uuml;r die Position der Darstellung des Inhaltes des
 *  Paketes */
const xmlChar xml_syntax_packet_content_pos[] = "content_pos";
//@}

//@{ @name Ethernet-Paket, ARP-Paket
/** Name des Tags f&uuml;r Ethernet-Pakete (Ethernet_Packet) */
const xmlChar xml_syntax_ethernet_packet[] = "ethernet_packet";
/** Name des Tags f&uuml;r ARP-Pakete (ARP_Packet) */
const xmlChar xml_syntax_arp_packet[] = "arp_packet";
/** Name des Attributes f&uuml;r den Operationskode eines ARP-Paketes */
const xmlChar xml_syntax_arp_packet_opcode[] = "opcode";
/** Name des Tags f&uuml;r die Quell-Ethernetadresse eines
 *  ARP- / Ethernet-Paketes */
const xmlChar xml_syntax_source_mac[] = "source_mac";
/** Name des Tags f&uuml;r die Ziel-Ethernetadresse eines
 *  ARP- / Ethernet-Paketes */
const xmlChar xml_syntax_destination_mac[] = "destination_mac";
//@}

//@{ @name IP-Paket
/** Name des Tags f&uuml;r ein IP-Paket (IPv4_Packet) */
const xmlChar xml_syntax_ipv4_packet[] = "ipv4_packet";
/** Name des Tags f&uuml;r die Quell-IP-Adresse eines IP-Paketes */
const xmlChar xml_syntax_source_ip[] = "source_ip";
/** Name des Tags f&uuml;r die Ziel-IP-Adresse eines IP-Paketes */
const xmlChar xml_syntax_destination_ip[] = "destination_ip";
/** Name des Tags f&uuml;r die Next-Hop-IP-Adresse eines IP-Paketes */
const xmlChar xml_syntax_next_hop_ip[] = "next_hop_ip";
//@}

//@{ @name TCP-Paket, UDP-Paket, Dienstpakete
/** Name des Tags f&uuml;r TCP-Pakete (TCP_Packet) */
const xmlChar xml_syntax_tcp_packet[] = "tcp_packet";
/** Name des Tags f&uuml;r UDP-Pakete (UDP_Packet) */
const xmlChar xml_syntax_udp_packet[] = "udp_packet";
/** Name des Tags f&uuml;r den Quellport eines TCP- / UDP-Paketes */
const xmlChar xml_syntax_source_port[] = "source_port";
/** Name des Tags f&uuml;r den Zielport eines TCP- / UDP-Paketes */
const xmlChar xml_syntax_destination_port[] = "destination_port";
/** Name des Tags f&uuml;r Dienstpakete (Service_Packet)*/
const xmlChar xml_syntax_service_packet[] = "service_packet";
/** Name des Attributes f&uuml;r den Dienstnamen eines
 *  (TCP-/UDP-)Dienstpaketes */
const xmlChar xml_syntax_service_packet_service_name[] = "service_name";
/** Name des Tags f&uuml;r TCP-Dienstpakete (TCP_Service_Packet)*/
const xmlChar xml_syntax_tcp_service_packet[] = "tcp_service_packet";
/** Name des Tags f&uuml;r UDP-Dienstpakete (UDP_Service_Packet)*/
const xmlChar xml_syntax_udp_service_packet[] = "udp_service_packet";
//@}

//@{ @name Farbwerte und Karte
// point_3d
/** Materialdefintion fr spekulare Lichtreflektion, 
 * wird nicht berall verwendet */
const GLfloat material_specular_std[4] = {0.3f, 0.3f, 0.3f, 1.0f};
			// Standard specular reflection (not for text)
/** Konstante f&uuml;r den Glanz eines Materials (GL_SHININESS in Point_3D und
 *  Rectangle) */
const GLfloat material_shininess_std = 5.0;

// glutmain, mouse
/** Gr&ouml;&szlig;e des Weltausschnittes f&uuml;r die dargestellte Karte
 *  im 3D-Fenster: linkes Ende (X), rechtes Ende (X), unteres Ende (Y),
 *  oberes Ende (Y), vorderes Ende (Z) und hinteres Ende (Z) */
const double map_sizes[6] =	{-20.0, 20.0, -20.0, 20.0, -10.0, 10.0};
				// left, right, bottom, top, near, far
//@}

//@{ @name Kommandopaket
/** Text am Anfang eines Kommandopaketes f&uuml;r den Dienstnamen eines
 *  folgenden Paketes (PacketServer, Packet_Client) */
const char 		service_name_format_string[] = 
					"command: service name: %s";
			// can't be an real packet: 
			// vi is an unknown ethernet type and
			// ce second 4 bits is wrong header length for IP 
/** L&auml;nge eines Dienstnamenkommandos mit Name, also des gesamten
 *  Kommandopaketes */
const unsigned short	service_name_length = 40; // with format name
//@}

#endif

