/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <vector>
#include <string>

#include "variable_tree.h"

using namespace std;

struct callback_struct
/** Struktur zur Zusammenfassung von Funktionen, die als Callback f&uuml;r den
 *  Dialog dienen */
{
	/** Callback-Funktion f&uuml;r das Neuzeichnen */
	void (* display_callback)(void);
	/** Callback-Funktion f&uuml;r Tastatureingaben (ohne Sondertasten) */
	void (* keyboard_callback)(unsigned char, int, int);
	/** Callback-Funktion f&uuml;r die Sondertasteneingabe (Cursor-Tasten,
	 *  Funktionstasten, Ins, Pos1, Ende, Bild auf/ab) */
	void (* specialkey_callback)(int, int, int);
};

class Dialog
/** zur Darstellung und &Auml;nderung der Werte eines Variablenbaums
 *  (Variable_Tree) */
{
	private:
		/** Breite des Randes um den Text zum Dialogfensterrahmen */
		const int		border;
		/** Zustandsvariablen: Einf&uuml;gemodus (insert_mode) und
		 *  Inhalt des Dialogs ge&auml;ndert (content_changed) */
		bool			insert_mode, content_changed;
		/** Identifikation des Dialogfensters innerhalb von GLUT */
		int			window_id;

		//@{ @name Breite, H&ouml;he und Cursorposition
		/** Anzahl der Zeichen je Zeile (num_columns) und Anzahl der
		 *  Zeilen insgesamt (num_rows) */
		unsigned int		num_columns, num_rows;
		/** Spalte (cursor_column) und Zeile (cursor_row), in der
		 *  der Cursor zur Zeit steht */
		unsigned int		cursor_column, cursor_row;
		/** Breite (current_width) und H&ouml;he (current_height)
		 *  des Dialogfensters in Pixeln */
		int			current_width, current_height;
		//@}

		/** Feld aller Textzeilen des Dialogs */
		vector<string>		text;

		// most left/right column, wich is editable, per line
		/** Felder: geben f&uuml;r jede Zeile den Anfang
		 *  (left_editable) und das Ende (right_editable) des
		 *  editierbaren Bereiches an */
		vector<unsigned int>	left_editable, right_editable;
		/** Zeiger auf den Zeichensatz zur Textdarstellung */
		void			*font;
		/** der darzustellende Variablenbaum */
		Variable_Tree		*var_tree;

		void check_cursor_column();
	public:
		Dialog(void);
		~Dialog(void);
		bool isChanged(void);
		bool isHidden(void);
		void init(struct callback_struct &callback_functions);
		void setSize(unsigned int new_num_columns, 
				unsigned int new_num_rows);
		void getText(Variable_Tree* &current_var_tree);
		void setText(Variable_Tree* new_var_tree);
		
		void destroy(void);
		void draw(void);
		void redraw(void);	// forces redraw
		bool key(unsigned char k, int x, int y);
		void specialKey(int k, int x, int y);
};

