/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "actions.h"
#include "capture.h"
#include "key_control.h"
#include "lights.h"
#include "menu.h"
#include "mouse.h"

class GlutMain
/** Hauptklasse als Verbindung von Grafik- und Netzwerkobjekten */
{
	private:
		//@{ @name Einfache Variablen
		/** Zusammenfassung mehrerer Statusvariablen */		
		unsigned int	main_states;
		/** Identifikator des Hauptfensters in GLUT */
		int		mainwindow;
		/** Fensterbreite (current_width) und 
		 *  Fensterh&ouml;he (current_height) */
                int		current_width, current_height;
		/** Verschiebung der grafischen Darstellung auf der X-, Y- und
		 *  Z-Achse vom Koordinatenursprung */
		double		x_translation, y_translation, z_translation;
		/** Geschwindigkeitsfaktor f&uuml;r die Animation,
		 *  beeinflu&szlig;t Pakettransport und Rotation von
		 *  Netzwerkknoten */
		double		animation_speed_factor;
		/** Zeitpunkt der letzten Neuzeichnung des Fensters, f&uuml;r
		 *  Aktualisierung in unterschiedlichen Fensterzust&auml;nden
		 *  (versteckt, verdeckt, im Vordergrund) */
		time_t		last_drawn;
		/** Name des Netzwerkknoten oder des Kabels, welches gerade im
		 *  Kamerfokus liegt ("wo gerade etwas passiert") */
		string		current_name_on_network;
		/** Displayliste der Darstellung des Namens, zur
		 *  Grafikbeschleunigung */
		GLuint		display_list_name_text;
		//@}

		//@{ @name Kamera und Viewport (eigene Klassen)
		/** die zwei Kameras f&uuml;r die linke Fensterh&auml;lfte
		 *  (mainwindow_cam) und die rechte -h&auml;lfte
		 *  (rightview_cam, nur wenn dort Weltansicht und keine
		 *  Kartenansicht) */
		Camera		mainwindow_cam, rightview_cam;
		/** Viewports der verschiedenen dargestellten Bereiche:
		 *  - scene_viewport: 3D-Ansicht des Netzwerkes mit Navigation
		 *  - map_viewport: Draufsicht auf das Netzwerk (Kartenansicht)
		 *  - name_viewport: Darstellung des Namens des fokussierten
		 *    Netzwerkelementes (Kabel/Knoten)
		 *  - second_scene_viewport: zweite 3D-Ansicht der Welt, aber
		 *    ohne Navigation */
		Viewport	scene_viewport, map_viewport, name_viewport,
				second_scene_viewport;
		//@}

		//@{ @name andere Objekte eigener Klassen
		/** zum Aufnehmen von Screenshots und Videos (Bildfolgen) */
		Capture		image_capturer;
		/** Dialogfenster zum Ver&auml;ndern von Einstellungen */
		Dialog		dialog_window;
		/** die Lichter in der 3D-Welt */
		Lights		mainwindow_lights;
		/** Verarbeitung der Tastatureingaben im Hauptfenster */
		Key_Control	mainwindow_keys;
		/** Aufbau des Men&uuml;s im Hauptfenster und Verarbeitung
		 *  der ausgew&auml;hlten Eintr&auml;ge */
		Menu		mainwindow_menu;
		/** Verarbeitung der Mauseingaben im Hauptfenster */
		Mouse		mainwindow_mouse;
		/** das dargestellte und simulierte Netzwerk */
		Network		network_web;
		/** Schriftart, Farbe und Position der Ausgabe des Namens des
		 *  fokussierten Kabels oder Netzwerkknotens */
		Stroke_Font	name_output;
		/** Variablenbaum zur Darstellung der Copyright-Hinweise im
		 *  Dialogfenster */
		Variable_Tree	copyright_infos;
		//@}

		void	drawCurrentName(void);
		void	updateDisplaylistNameText(void);
		void	drawGrid(void);
		void	drawWindow(void);
		void	initLoadNet(void);
		void	initSaveNet(void);
		void	getMaxMinSize(int &max_size, int &min_size);
		void	setViewportSizes(void);
		void	executeNet(void);
		void	toggleState(int state);
		void	showCopyright(void);
	public:
		GlutMain(void);
		~GlutMain(void);
		void	createWindow(void);
		void	display(void);
		void	dialog_display(void);

		//@{ @name Tastatureingaben
		void	dialog_key(unsigned char k, int x, int y);
		void	dialog_specialKey(int k, int x, int y);
		void	key(unsigned char k, int x, int y);
		void	specialKey(int k, int x, int y);
		//@}

		//@{ @name Maus und Men&uuml;
		void	mouse_func(int button, int state, int x, int y);
		void	motion(int x, int y);
		void	passiveMotion(int x, int y);
		void	menu_func(int entry);
		void	menuState(int state);
		//@}

		void	idle(void);
		void	init(void (* menu_callback)(int entry),
				struct callback_struct &dialog_callbacks);
		void	checkArgs(int argc, char *argv[]);
		void	reshape(int width, int height);
		void 	startAction(Actions number);
		void	windowStatus(int status);
};

