/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "cylinder.h"

class Camera
/** Eine Kamera zur Darstellung eines Ausschnittes der 3D-Welt aus
    einem bestimmten Blickwinkel. */
{
	private:
		/** Aktuelle, maximale und minimale Geschwindigkeit der 
		 * Kamerabewegung */
		double		speed, max_speed, min_speed;
		/** Aktuelle Ausrichtung als Winkel um die X-Achse
		 *  (waagerechte Achse, Kippen), um die Y-Achse 
		 *  (senkrechte Achse, Richtung) und Zielwinkel der Y-Achse
		 *  am Ende einer schrittweisen Rotation (y_turn_to_angle) */
		double		x_angle, y_angle, y_turn_to_angle;

		//@{ @name Position, Blickrichtung und Up-Vektor
		/**  Aktuelle Kamera-Position */
		double		x_position, y_position, z_position;
		/** Mittelpunkt des Fensterbereiches, Zentrum des 
		 *  Kamera-Sichtfeldes, berechnet aus Winkel und Position */
		double		center_x, center_y, center_z;
		/** Vektor, der oben aus der Kamera herauszeigt (gluLookAt),
		 *  berechnet aus den Kamera-Winkeln */
		double		up_x, up_y, up_z;
		//@}

		/** Darstellung der Kamera in der Kartenansicht, zur Markierung
		 *  der Kamera-Position */
		Cylinder	camera_figure;
	public:
		Camera(void);
		~Camera(void);
		void	draw(void) const;
		void	gotoPos(void) const;
		void	init(void);

		//@{ @name aktuelle Kameraeinstellungen abfragen
		double	getXAngle(void) const;
		double	getYAngle(void) const;
		void	getPos(double &x_pos, double &y_pos,
				double &z_pos) const;
		void	getPos(Point_3D &camera_pos) const;
		bool	is_moving(void) const;
		bool	needsTurn(void) const;
		//@}

		//@{ @name Position und Blickrichtung setzen
		void	movePos(double add_x_pos, double add_y_pos,
					double add_z_pos);
		void	move(double speed_step, double speed_factor);
		void	setAngle(double new_x_angle, double new_y_angle);
		void	setPos(double x_pos, double y_pos, double z_pos);
		void	turn(double x_angle_step, double y_angle_step);

		void	turnToAngle(double new_y_angle);
		void	turnToAngleStep(void);
		//@}

		void	updateCam(Point_3D &new_camera_pos,
					double new_camera_y_angle,
					bool follow_angle);
		Camera&	operator = (const Camera &new_values);
};

