/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <vector>
#include <GL/gl.h>

using namespace std;

struct LightValues
/** Abspeicherung der Werte einer Lichtquelle */
{
	/** Position des Lichtes (X, Y, Z) und Art 
	 * (paralleles oder Punktlicht) */
	GLfloat	position[4];
	/** Lichtfarbe (RGBA) im ambienten Bereich, Wertebereich jeweils 
	 * 0.0 - 1.0 */
	GLfloat	ambient[4];
	/** Lichtfarbe (RGBA) im diffusen Bereich, Wertebereich jeweils 
	 * 0.0 - 1.0 */
	GLfloat	diffuse[4];
	/** Lichtfarbe (RGBA) im spiegelnden/spekul&auml;ren Bereich,
	 * Wertebereich jeweils 0.0 - 1.0 */
	GLfloat	specular[4];
	/** Richtung des Lichtes (Vektor aus X, Y und Z) */
	GLfloat	direction[3];
};

class Lights
/** Zusammenfassung aller Lichtquellen und deren Umsetzung in der 
 *  Darstellung */
{
	private:
		/** Grundhelligkeit (Farbe) der Szene in RGBA, 
		 *  Wertebereich je 0.0 - 1.0 */
		GLfloat				light_amb[4];
		/** Feld aller genutzten Lichtquellen */
		vector<struct LightValues>	light;
	public:
		Lights(void);
		~Lights(void);
		void	init(void);
		bool	setValues(size_t light_number, GLenum pname,
					const GLfloat new_values[]);
		/* transfer current intern values to the GL environment */
		void	update(void);
};

