/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <png.h>

/** Farbtiefe je Farbkanal in PNG-Datei */
const int	png_image_writer_bit_depth = 8;
/** Farbtyp in PNG-Datei */
const int	png_image_writer_color_type = PNG_COLOR_TYPE_RGB;
/** Kompressionsrate der PNG-Daten */
const int	png_image_writer_compression_level = Z_BEST_SPEED;

// taken from "man libpng"

class PNG_Image_Writer
/** Eine Klasse zum Abspeichern eines Bildes im PNG-Format,
 *  Ablauf: init(), setImage(), setSize(), writePNG() und am Ende
 *  clear() / init() (wenn noch ein Bild folgt) / 
 *  Destruktor ~PNG_Image_Writer() */
{
	private:
		/** Zeiger auf die Bilddaten im Hauptspeicher */
		char*		image_data;
		/** Bildbreite (width) und Bildh&ouml;he (height) */
		size_t		image_width, image_height;
		/** Farbtiefe je Farbkanal (depth) und Farbtyp (type) */ 
		int		image_bit_depth, image_color_type;
		/** Datenstruktur zum Schreiben der Datei */
		png_structp	png_write_pointer;
		/** Datenstruktur mit Infos zur PNG-Datei */
		png_infop	png_info_pointer;
		/** Feld von Zeigern auf die Zeilen des Bildes */
		png_byte**	row_pointers;

	public:
		PNG_Image_Writer(void);
		~PNG_Image_Writer(void);

		void	clear(void);
		int	init(int new_bit_depth = png_image_writer_bit_depth,
				int new_color_type =
					png_image_writer_color_type,
				int image_compression_level = 
					png_image_writer_compression_level);
		int	setImage(char* new_data, bool flip_lines = false);
		int	setSize(size_t new_width, size_t new_height,
				bool flip_lines = false);
		int	writePNG(const char* const filename) const;
};

