/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <zlib.h>

/** Kompressionsrate der Datei (gepacktes PNM / .pnm.gz) */
const int	pnm_image_writer_compression_level = Z_BEST_SPEED;

class PNM_Image_Writer
/** Eine Klasse zum Abspeichern eines Bildes im PNM-Format
 *  Ablauf: init(), setImage(), setSize(), writePNM() / writePNM_GZ()
 *  und am Ende clear() / init() (wenn noch ein Bild folgt) / 
 *  Destruktor ~PNM_Image_Writer(); <br>
 *  wenn sich nur der Bildinhalt &auml;ndert und nicht der Zeiger / die
 *  Ausma&szlig;e kann mehrmals writePNM() / writePNM_GZ() aufgerufen werden
 *  (was bei PNG_Image_Writer nicht geht) */
{
	private:
		/** Statusvariable, zeigt an, ob Bildzeilen umzudrehen sind */
		bool		flip_image_lines;
		/** Zeiger auf die Bilddaten im Hauptspeicher */
		char*		image_data;
		/** Kompressionsrate mit der Bilddatei gespeichert
		 *  werden soll */
		int		image_compression_level;
		/** Bildbreite (width) und Bildh&ouml;he (height) */
		size_t		image_width, image_height;

	public:
		PNM_Image_Writer(void);
		~PNM_Image_Writer(void);

		void	clear(void);
		int	init(int new_compression_level = 
					pnm_image_writer_compression_level);
		int	setImage(char* new_data, bool flip_lines = false);
		int	setSize(size_t new_width, size_t new_height,
					bool flip_lines = false);
		int	writePNM(const char* const filename) const;
		int	writePNM_GZ(const char* const filename) const;
};

