/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef POINT_3D_H
#define POINT_3D_H 1

#include <iostream>
#include <libxml/parser.h>
#include <libxml/xmlreader.h>
#include <libxml/xmlwriter.h>

using namespace std;

class Point_3D
/** ein drei-dimensionaler Punkt inklusive Farbe */
{
	private:
		/** Position des Punktes auf der X-, Y- bzw. Z-Achse */
		double posx, posy, posz;
		/** Farbe des Punktes in RGBA (Rot, Gr&uuml;n, Blau, Alpha), 
		 * Wertebereich 0.0 - 1.0 */
		double red, green, blue, alpha;
	public:
		Point_3D(void);
		~Point_3D(void);

		//@{ @name Abfragen zu Position, Farbe, Distanz und Winkel
		double	getX(void) const;
		double	getY(void) const;
		double	getZ(void) const;
		double	getR(void) const;
		double	getG(void) const;
		double	getB(void) const;
		double	getA(void) const;
		
		double  getDistance(const Point_3D &end_point, 
					bool with_y = true) const;
		double  getAngleXZ(const Point_3D &end_point) const;
		//@}
	
		void	parseXML(xmlNode * const parse_child);
		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;

		void	setPos(double new_posx, double new_posy, 
				double new_posz);
                void	setColor(double new_red, double new_green, 
				double new_blue, double new_alpha);
		void	translate(double mult) const; 
		// mult = -1 -> opposite direction, mult scales the translation
		void	colorForDraw() const;	//glColor/glMaterial
		void	vertexDraw(	double add_x, double add_y, 
					double add_z) const;
		Point_3D&	operator += (const Point_3D &point);
		friend ostream& operator << (ostream &output,
						const Point_3D &point);
};

#endif

