/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef STROKE_FONT_H
#define STROKE_FONT_H 1

#include <GL/gl.h>

#include "point_3d.h"

/** Art des Punktes in der Zeichenbeschreibung (Keine, Punkt, Strich, Ende) */
enum Font_Point_Type {NO_POINT_TYPE, PT, STROKE, END};

struct CharPoint
/** ein Punkt der Zeichenbeschreibung,
 *  ein Zeichen ist ein Feld von CharPoint */
{
	/** Position des Punktes auf der X- und Y-Achse */
	GLfloat		x, y;
	/** Art des Punktes */
	Font_Point_Type	type;
};

class Stroke_Font
/** eine Klasse zur Darstellung von Text in einer Schriftart,
 *  enth&auml;lt die Beschreibung des Fonts */
{
	private:
		/** Feld von Zeigern (Felder) mit den Beschreibungen der
		 *  Buchstaben */
		CharPoint*	letter_data[256];
		/** Position der Textausgabe (Ecke links unten) */
		Point_3D	pos;
		/** Breite (width) und H&ouml;he (height) je Buchstabe
		 *  (Skalierung zu Standardbreite 5 und -h&ouml;he 10) */
		double		width, height;

		void	drawLetter(CharPoint *letter) const;
	public:
		Stroke_Font(void);
		~Stroke_Font(void);
		void	init(void);
		void	print(const char text[]) const;
                void	setColor(double new_red, double new_green,
					double new_blue, double new_alpha);
		void	setPos(double new_centerx, double new_centery, 
				double new_centerz);
                void	setSize(double new_width, double new_height);
};

#endif
