/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef VIEWPORT_H
#define VIEWPORT_H 1

#include "rectangle.h"

class Viewport : public Rectangle
/** Beschreibung und Darstellung eines rechteckigen Fensterbereiches mit
 * eigener Zeichenfunktion */
{
	private:
		/** Darstellungsmatrizen dieses Fensterbereiches */
		double	model_matrix[16], projection_matrix[16];
		/** Darstellungsfunktion des Fensterbereiches */
		int	(*drawContent)(void);
	public:
		Viewport(void);
		~Viewport(void);

		//@{ @name Position und Skalierung
		void scale(double mult_width, double mult_height);
		void scalePoint(double window_height, double size[2], 
				double point[2], double scaled_point[2]);
		void setPos(double new_left, double new_top, 
			    	double new_right, double new_bottom);
		void setPos(const double new_pos[4], const double scaling[4]);
		//@}

		//@{ @name Matrizen und Projektionen
		void setModelMatrix(void);
		void setProjectionMatrix(void);

		void setglOrtho(const double ortho_values[6],
				const double scaling[6]);
		void setgluPerspective(double fovy, double aspect,
					double zNear, double zFar);
		//@}

		bool isInViewport(int window_height, int x, int y) const;
		void draw(void) const;
		void setDrawContent(int (*new_drawContent)(void));
};

#endif

