/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "actions.h"
#include "camera.h"
#include "dialog.h"
#include "network.h"
#include "viewport.h"

class Mouse
/** Klasse zur Verarbeitung von Mauseingaben */
{
		/** Aktionsmodus im Karten-Fensterbereich: "Was l&ouml;st ein
		 *  Mausklick in der Karte aus?": <br>
		 *  Einf&uuml;gen, Kamera positionieren, L&ouml;schen,
		 *  Verschieben, (Einstellungen) anzeigen/bearbeiten */
	enum	mode_type	{insert_mode, cam_mode, remove_mode, move_mode,
					info_mode };
	private:
		//@{ @name Einstellungen, Status und Gr&ouml;&szlig;en
		/** die Mausposition in 'first_posxz' ist g&uuml;ltig */
		bool		first_is_valid;	// first_posxz is valid
		/** Status der Modifikationstasten (Shift, Alt, Strg):
		 *  gedr&uuml;ckt oder nicht */
		// Shift, Alt, Ctrl pressed?
		int		current_active_modifiers;
		/** aktuelle Fensterbreite (current_window_with) und
		 *  Fensterh&ouml;he (current_window_height) */
		int		current_window_width, current_window_height;
		/** Aktionsmodus im Karten-Fensterbereich, siehe mode_type */
		mode_type	map_mode;
		/** Farbe der dargestellten Linie vom Anfang der Mausbewegung
		 *  mit gedr&uuml;ckter (linker/mittlerer) Maustaste bis zur
		 *  aktuellen Position */
		Point_3D	mouse_line_color;
		//@}

		//@{ @name Mauspositionen
		/** aktuelle Mausposition (mouse_pos_x, mouse_pos_y) und
		 *  letzte benutzte Maustaste (mouse_button) */
		int		mouse_pos_x, mouse_pos_y, mouse_button;
		/** Position der Maus, als der Maustastendruck begann,
		 *  f&uuml;r Verschiebungen bei gedr&uuml;ckter Maustaste */
		// positions when button press started
		int		pos_x_onenterbutton, pos_y_onenterbutton;
		/** auf OpenGL-Weltkoordinaten umgerechnete Mauspositionen
		 *  (X- und Z-Achse, y = 0) im Karten-Fensterbereich:
		 *  Mausposition bei Beginn des Tastendrucks (first_posxz)
		 *  und aktuelle Mausposition (scaled_pos) */
		// mouse positions in opengl coordinate system in map viewport
		double		first_posxz[2], scaled_pos[2];
		/** auf OpenGL-Weltkoordinaten umgerechnete Mausposition
		 *  im Info-Modus: zum Objekt an dieser Stelle wurde der Dialog
		 *  (mit den Informationen des Objektes) aufgerufen */
		double		last_info_posxz[2];
		//@}

		//@{ @name Zeiger auf Variablen in GlutMain
		// following are all pointers to variables in GlutMain
		/** Zeiger auf die Kamera des Hauptfensters,
		 *  von GlutMain::mainwindow_cam */
		Camera		*mainwindow_cam;
		/** Zeiger auf das Dialogfenster des Programms,
		 *  von GlutMain::dialog_window */
		Dialog		*dialog_window;
		/** Zeiger auf das simulierte Netzwerk,
		 *  von GlutMain::network_web */
		Network		*network_web;
		/** Zeiger auf den Viewport (Fensterbereich) der Kartenansicht,
		 *  von GlutMain::map_viewport */
		Viewport	*map_viewport;
		//@}

		void	do_cam_mode(int state);
		void	do_info_mode(int state);
		void	do_insert_mode(int state);
		void	do_move_mode(int state);
		void	do_scaling(void);
		void	do_motion_shift(bool direction_changed);
	public:
		Mouse(void);
		~Mouse(void);
		void	draw(void);
		void	getLastInfoPos(double return_last_info_posxz[2]);
		void	init(Camera *new_camera, Dialog *new_dialog,
				Network *new_network, Viewport *new_map_view);
		void	mouse_func(int button, int state, int x, int y,
					bool level_of_details_on,
					bool map_is_shown);
		void	motion(int x, int y, bool level_of_details_on);
		void	passiveMotion(int x, int y);
		void	reshape(int width, int height);
		void	startAction(Actions number);
};

