/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <map>

#include "ipaddress.h"
#include "macaddress.h"
#include "rectangle.h"
#include "stroke_font.h"
#include "variable_tree.h"

using namespace std;

class ArpCache
/** Darstellung und Umsetzung eines ARP-Caches zur &Uuml;bersetzung von
 *  IP-Adressen in Ethernet-Adressen */
{
	private:
		/** &Uuml;bersetzungstabelle von IP- nach Ethernet-Adressen */
		map<IpAddress,MacAddress>	ip_mac_map;
		/** letzte zu &uuml;bersetzende Adresse, zur Darstellung
		 *  des Caches im 3D-Fenster */
		IpAddress			last_searched_ip;
		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree			current_infos;

		/** Textposition (text_pos) und Textgr&ouml;&szlig;e
		 *  (text_size), zur Berechnung und Darstellung n&ouml;tig */
		Point_3D			text_pos, text_size;
		/** Farbe, Textgr&ouml;&szlig;e und Schriftart zur
		 *  Darstellung */
		Stroke_Font			cache_text_output;

		void	drawColorBackground(bool is_same_ip,
					class Rectangle &background) const;
	public:
		ArpCache(void);
		~ArpCache(void);
		void	draw(bool level_of_details_on,
				Point_3D camera_pos) const;
		void	init(void);

		void	setColor(double new_red, double new_green,
				double new_blue, double new_alpha);
		void	setPos(double new_centerx, double new_centery,
				double new_centerz);
		void	setSize(double new_width, double new_height,
				double new_depth);

		bool	findMac(const IpAddress &ipaddress,
					MacAddress &macaddress);
		void	insertIpMac(IpAddress &ipaddress, 
					MacAddress &macaddress);

		//@{ @name XML- und Dialog-Datenaustausch	
		void	getInfos(Variable_Tree* &infos);
		void	setInfos(Variable_Tree* infos);
		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;
		//@}
};

