/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef INTERFACE_H
#define INTERFACE_H 1

#include "interface_executenet.h"

class Interface : private Interface_ExecuteNet
/** Klasse zur Simulation und Darstellung einer Netzwerkkarte in Routern und
 *  Hosts */
{
	private:
		/** Position (position) und Gr&ouml;&szlig;e (graphic_size) der
		 *  Darstellung der Netzwerkkarte */
		Point_3D	position, graphic_size;

		/** Name der Netzwerkkarte, f&uuml;r das Routing */
		string		name;
		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree	current_infos;

		// defined in Interface_ExecuteNet:
		// double		direction_angle, angle_step; 
		// double		needed_angle, last_needed_angle;
		// bool			sending_arp_packet;
		// ArpCache		arp_cache;
		// IpQueue		ip_queue;
		// EthernetQueue	eth_queue;
		// Net_Cable		*attached_cable;

		void	updatePos(void);

	public:
		Interface();
		~Interface();
		//@{ @name Position, Gr&ouml;&szlig;e, Zeichnen und Init
		void	setPos(double posx, double posy, double posz);
		void	setSize(double new_width, double new_height, 
				double new_depth);
		void	drawArpCache(bool level_of_details_on,
					Point_3D camera_pos) const;
		void	drawPackets(bool level_of_details_on,
					Point_3D camera_pos) const;
		void	drawQueue(void) const;
		void	init(void);
		//@}

		void	attachCable(Net_Cable *new_cable);
		double	getAngle(void) const;
		void	setAngle(double new_angle);
		bool	needsTurn(void) const;
		size_t	numPackets(void) const;
		size_t	numPacketsOnCable(void) const;
		void	executeNet(double speed_factor);

		//@{ @name Adressen und Name
		char*	getIpAddress(void) const;
		void	getIpAddress(IpAddress &current_ip) const;
		bool	setIpAddress(const char ip_string[]);
		string	getMacAddress(void) const;
		void	getMacAddress(MacAddress &current_mac) const;
		bool	setMacAddress(const char mac_string[]);
		string	getName(void) const;
		void	setName(string new_name);
		//@}

		//@{ @name XML- und Dialog-Datenaustausch
		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;
		void	getInfos(Variable_Tree* &infos);
		void	setInfos(Variable_Tree* infos);
		//@}

		//@{ @name Up, Down, Erstes Packet
		void	down_in(IPv4_Packet* incoming_packet);
		void	down_out(Ethernet_Packet* &outgoing_packet);
		void	up_in(Ethernet_Packet* incoming_packet);
		void	up_out(IPv4_Packet* &outgoing_packet);
		// returns true if there is a packet in one of the queues
		bool	getFirstPacketPos(Point_3D &packet_pos) const;
		//@}
};

#endif
