/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "arpcache.h"
#include "arp_packet.h"
#include "ethernetqueue.h"
#include "ipqueue.h"
#include "net_cable.h"

class Interface_ExecuteNet
/** Erweiterungsklasse zu Interface zur Netzwerksimulation */
{
	protected:
		/** aktuelle Rotationsrichtung (direction_angle) und
		 *  Schrittweite der Drehung (angle_step) */
		double		direction_angle, angle_step;
		/** aktueller Zielwinkel zur Rotation (needed_angle) und
		 *  letzter erreichter Zielwinkel (last_needed_angle) zur
		 *  Berechnung der Drehgeschwindigkeit */
		double		needed_angle, last_needed_angle;
		/** Geschwindigkeitsfaktor in der Simulation f&uuml;r
		 *  Pakettransport und Drehung, zwischengespeichert */
		double		animation_speed_factor;

		/** ist gerade ein ARP-Paket von diesem Interface gesendet
		 *  worden (und noch nicht beantwortet) ? */
		bool		sending_arp_packet;
		/** Cache zur &Uuml;bersetzung von IP- in Ethernet-Adressen */
		ArpCache	arp_cache;
		/** Warteschlangen f&uuml;r IP-Pakete zum Senden und
		 *  Empfangen */
		IpQueue		ip_queue;
		/** Warteschlangen f&uuml;r Ethernet-Pakete zum Senden und
		 *  Empfangen */
		EthernetQueue	eth_queue;
		/** an die Netzwerkkarte angeschlossenes Kabel, damit werden
		 *  die Pakete verschickt und empfangen */
		Net_Cable	*attached_cable;

	private:
		// returns if interface were turned
		bool	turnInterface(void);

		bool	ipSend(void);
		void	arpSend(IPv4_Packet* ip_packet);
		void	intoEthQueue(Ethernet_Packet *eth_packet);
		bool	ethernetSend(void);
		bool	ethernetReceive(void);
		bool	ipReceive(void); // with ARP
		void	arpReply(ARP_Packet* arp_packet);
		void	arpReceive(ARP_Packet* arp_packet);

	public:
		Interface_ExecuteNet();
		~Interface_ExecuteNet();

		void	executeNet(double speed_factor);

		size_t	numPackets(void) const;
		size_t	numPacketsOnCable(void) const;
};

