/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <map>

#include "ipv4_packet.h"
#include "variable_tree.h"

using namespace std;

struct ipaddr_port
/** Struktur zur Abspeicherung einer Kombination "IP-Adresse und Portnummer" */
{
	/** eine IP-Adresse */
	IpAddress	address;
	/** eine Portnummer */
	unsigned short	port;
};

class NAT
/** Klasse zur Simulation einer Netzwerk-Adress&uuml;bersetzung (NAT) von einem
 *  Quellnetzwerk (mit &uuml;blicherweise privaten Adressen) in ein
 *  Zielnetzwerk (mit &uuml;blicherweise &ouml;ffentliche(r/n) Adresse(n)) und
 *  R&uuml;ck&uuml;bersetzung f&uuml;r Antwortpakete ans Quellnetzwerk; <br>
 *  Pakete aus dem Zielnetzwerk ans Quellnetzwerk ohne passenden Eintrag in den
 *  Tabellen werden gel&ouml;scht; <br>
 *  PAT (Port and Address Translation) wird nur gew&auml;hlt, wenn nur eine
 *  Adresse im Zielnetzwerk zur Verf&uuml;gung steht (Netzmaske
 *  255.255.255.255), dann werden auch die Portnummern &uuml;bersetzt, <br>
 *  bei normalem NAT bleiben die Portnummern unver&auml;ndert, nur die Adressen
 *  werden getauscht */
{
	private:
		/** Adresse/Adressbereich des Quellnetzwerkes */
		IpAddress				source_network;
		/** Netzmaske des Quellnetzwerkes */
		IpAddress				source_netmask;
		/** Adressse/Adressbereich des Zielnetzwerkes */
		IpAddress				target_network;
		/** Netzmaske des Zielnetzwerkes */
		IpAddress				target_netmask;
		/** Portbereich (min/max) f&uuml;r PAT, in dem Pakete aus dem
		 *  Quellnetzwerk &uuml;bersetzt werden und Anwortpakete ans
		 *  Quellnetzwerk erkannt werden */
		unsigned short				min_port, max_port;
		// PAT: port address translation, only one IpAddress is usable
		/** &Uuml;bersetzungstabelle f&uuml;r PAT: Portnummer auf
		 *  IP-Adresse und Port des Quellnetzwerkes */
		map<unsigned short, ipaddr_port>	pat_translations;
		/** &Uuml;bersetzungstabelle f&uuml;r NAT: IP-Adresse aus
		 *  Zielnetzwerk auf IP-Adresse im Quellnetzwerk */
		map<IpAddress, IpAddress>		nat_translations;

		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree				current_infos;

		bool	isInNetwork(IpAddress test_address, IpAddress network,
					IpAddress netmask);
		void	incIpAddress(IpAddress &address);
		void	findAndSetSourcePAT(IPv4_Packet* ip_packet);
		void	findAndSetSourceNAT(IPv4_Packet* ip_packet);
		void	findAndSetDestinationPAT(IPv4_Packet* &ip_packet);
		void	findAndSetDestinationNAT(IPv4_Packet* &ip_packet);
	public:
		NAT(void);
		~NAT(void);
		void	init(void);
		
		void	getInfos(Variable_Tree* &infos);
		void	setInfos(Variable_Tree* infos);
		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;
		void	translate(IPv4_Packet* &ip_packet);
};

