/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef NET_CABLE_H
#define NET_CABLE_H 1

#include <vector>

#include "macaddress.h"
#include "net_cable_packets.h"
#include "packet.h"
#include "point_3d.h"
#include "variable_tree.h"

using namespace std;

class Net_Cable: public Net_Cable_Packets
/** Beschreibung und Darstellung eines (TP-)Kabels in einem Rechner-Netzwerk */
{
	private:
		/** Tiefe (depth) bzw. Hhe (height) des Kabels (Kabeldicke) */
		double			depth, height;		// thickness
		/** letzter bekannter Zustand des Level of Detail (an/aus),
		 *  muss zwischengespeichert werden */
		bool			last_level_of_details_on;
		/** letzte bekannte Kameraposition, auch f&uuml;r Level of
		 *  Detail, muss zwischengespeichert werden */
		Point_3D		last_camera_pos;
		/** Displaylisten zur Beschleunigung der Darstellung der Pakete
		 *  des Kabels */
		GLuint			display_list;

		/** Ethernet-Adressen der Karten am Anfang (begin_address) bzw.
		 *  Ende des Kabels (end_address) */
		MacAddress  		begin_address, end_address;
		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree		current_infos;
		
		void initGraphic(void);		
		void updateDisplaylist(void);
		
	public:
		Net_Cable(void);
		~Net_Cable(void);

		//@{ @name Netzwerksimulation
		void		insertPacket(MacAddress begin_or_end,
						Packet *new_packet);
		// returns true if the first packet is for your begin_or_end
		// if first_packet == NULL packet position isn't the position
		// needed to give the packet out. 
		bool		sniffPacket(MacAddress begin_or_end,
						Packet* &first_packet);
		void		executeNet(double speed_factor);
		//@}

		//@{ @name XML- und Dialog-Datenaustausch
                void    	getInfos(Variable_Tree* &infos);
                void    	setInfos(Variable_Tree* infos);
		void 		loadXML(xmlTextReaderPtr infile);
		void 		saveXML(xmlTextWriterPtr outfile) const;
		//@}

		//@{ @name Abfragen der Einstellungen und Zust&auml;nde
		void 		getAddresses(MacAddress &begin, 
						MacAddress &end);
		double		getAngle(MacAddress begin_or_end) const;
		void		getName(string &current_name);
		// return true if point is on the cable
		bool		isOnCable(double x, double y, double z);
		bool		testExchangeAddresses(Point_3D test_pos,
						MacAddress test_address);
		//@}

		void 		draw(bool with_control_points = false) const;
		void		drawSetCameraPos(bool level_of_details_on,
							Point_3D camera_pos);
		void 		init(void);
		void 		setColor(double new_red,  double new_green, 
					 double new_blue, double new_alpha);
		void 		setSize(double new_depth, double new_height);

		//@{ @name Anfang und Ende
		void 		setBegin(double x, double y, double z);
		void 		setEnd(double x, double y, double z);
		/* should be set on Node::attachCable */
		void		setBeginNodeRadius(double new_begin);
		void		setEndNodeRadius(double new_end);
		//@}

		//@{ @name Hilfspunkte (Corners)
		void 		addCorner(double x, double y, double z);
		void		moveCorner(double old_posxz[2], 
						double new_posxz[2]);
		// returns number of corners left
		unsigned int	removeCorner(double x, double y, double z);
		//@}

};

#endif

