/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef NET_CABLE_CORNERS_H
#define NET_CABLE_CORNERS_H 1

#include <vector>
#include <GL/gl.h>

#include "point_3d.h"

using namespace std;

class Net_Cable_Corners
/** Beschreibung und Darstellung des grafischen Teiles eines (TP-)Kabels in
 *  einem Rechner-Netzwerk, Erweiterungsklasse zu Net_Cable */
{
	private:
		/** Beginn, Ecken bzw. Ende des Kabels, als Punkt, dies
		 *  sind die Hilfspunkte zur Berechnung der Kurve */
		vector<Point_3D>	corners;
		/** Kurvenpunkte zum Zeichnen des Kabels, das Ergebnis der
		 *  Berechnung der Bezier-Kurve */
		vector<Point_3D>	curve_points;
		/** Farbe der schr&auml;gen Teile des Kabels */
		Point_3D		shade_color;
		/** Verk&uuml;rzung des Kabels am Beginn bzw. Ende, damit sie
		 *  genau an den Knoten 'andocken' und nicht bis zur Mitte
		 *  dieser reichen */
		double			begin_node_radius, end_node_radius;
		/** Displayliste zur Beschleunigung der Darstellung des Kabels
		 */
		GLuint			display_list_corners;

		bool isInArea(const Point_3D &first_corner,
					const Point_3D &test_point,
					const Point_3D &second_corner) const;
		bool findNearestByAngle(const Point_3D &before,
					const Point_3D &current,
					const Point_3D &after,
					const Point_3D &test_point) const;
		void findNearestByDistance(const Point_3D &test_point,
				vector<Point_3D>::const_iterator &nearest,
				const vector<Point_3D> &point_array) const;
		void findNearest(const Point_3D &test_point,
				vector<Point_3D>::const_iterator &nearest,
				const vector<Point_3D> &point_array) const;
	protected:
		void findNearestCorner(const Point_3D &test_point, 
					vector<Point_3D>::const_iterator
					&nearest) const;
		void findNearestCurvePoint(const Point_3D &test_point,
					vector<Point_3D>::const_iterator
					&nearest) const;
		void curveLength(vector<Point_3D>::const_iterator &before,
				vector<Point_3D>::const_iterator &after,
				double &before_length, double &after_length,
				double &full_length) const;
		void getCurve(vector<Point_3D>::const_iterator &first,
				vector<Point_3D>::const_iterator &last) const;
		void		initGraphic(void);

		void 		loadXML(xmlTextReaderPtr infile,
					int num_corner_read);
		void 		saveXML(xmlTextWriterPtr outfile) const;

		void 		setColor(double new_red,  double new_green, 
					 double new_blue, double new_alpha);
		// return true if point is on the cable
		bool		isOnCable(double x, double y, double z,
						double depth) const;

		//@{ @name Hilfspunkte (Corners)
		Point_3D	getCorner(size_t number) const;
		size_t		numCorners(void) const;
		void 		addCorner(double x, double y, double z);
		void		moveCorner(double old_posxz[2], 
						double new_posxz[2]);
		// returns number of corners left
		size_t		removeCorner(double x, double y, double z);
		//@}

		//@{ @name Anfang und Ende
		void 		setBegin(double x, double y, double z);
		void 		setEnd(double x, double y, double z);
		// should be set on Node::attachCable
		void		setBeginNodeRadius(double new_begin);
		void		setEndNodeRadius(double new_end);
		//@}

		//@{ @name Zeichenmethoden (draw)
		void 		draw(void) const;
		void		drawControlPoints(double depth,
							double height) const;
		void 		drawCorner(Point_3D pos, double depth,
					double height) const;
		void 		drawSegment(Point_3D begin, Point_3D end,
					double begin_angle, double end_angle,
					double begin_translate,
					double end_translate,
					double depth, double height,
					double ground_depth_factor) const;
		void		updateDisplaylistCorners(
					double depth, double height,
					double ground_depth_factor);
		//@}

	public:
		Net_Cable_Corners(void);
		~Net_Cable_Corners(void);

		void		getBeginEnd(Point_3D &begin,
						Point_3D &end) const;
};

#endif

