/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef NET_CABLE_PACKETS_H
#define NET_CABLE_PACKETS_H 1

#include <list>

#include "macaddress.h"
#include "net_cable_corners.h"
#include "packet.h"

using namespace std;

class Net_Cable_Packets: public Net_Cable_Corners
/** Beschreibung und Darstellung der Pakete eines (TP-)Kabels in einem
 *  Rechner-Netzwerk, Erweiterungsklasse zu Net_Cable */
{
	private:
		/** Liste der Pakete des Kabels */
		list<Packet*>		traffic;	// array of packets

	protected:
		void 	loadXML(xmlTextReaderPtr infile);
		void 	saveXML(xmlTextWriterPtr outfile) const;

		void	findEndCorner(Packet* cable_packet,
					Point_3D &current_pos,
					Point_3D &end_corner) const;
		void	insertPacket(bool is_on_begin, Packet *new_packet,
					double depth,
					double ground_depth_factor,
					double angle_on_begin_or_end);
		// returns true if the first packet is for your begin_or_end
		// if first_packet == NULL packet position isn't the position
		// needed to give the packet out. 
		bool	sniffPacket(bool is_on_begin,
					Packet* &first_packet) const;
		double	calcSpeed(bool direction_beginend,
				Point_3D &current_pos,
				vector<Point_3D>::const_iterator &before,
				vector<Point_3D>::const_iterator &after) const;
		void 	movePacket(double speed_factor,
					Packet* cable_packet) const;

		void	executeNet(double speed_factor);

		void 	setSize(double depth, double ground_depth_factor);
		void 	draw(bool level_of_details_on,
					Point_3D last_camera_pos) const;
	public:
		Net_Cable_Packets(void);
		~Net_Cable_Packets(void);
		size_t	numPackets(void) const;
		void	deletePacket(Packet *first_packet);
		void 	delAllPackets(void);

		// returns true if there is a packet in the cable
		bool	getFirstPacketPosAngle(Point_3D &packet_pos,
					double &packet_angle) const;
};

#endif

