/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <list>
#include <vector>

#include "node.h"
#include "net_cable.h"
#include "packet_server.h"

using namespace std;

class Network
/** Klasse zur Darstellung eines Computer-Netzwerkes mit Knoten (Node) und
 *  Kabeln (Net_Cable) */
{
	private:
		/** die Position der Knoten auf der senkrechten Achse
		 *  (Verschiebung nach oben, alle sind auf gleicher H&ouml;he)
		 */
		const double		node_posy;	// pos on y
		/** die Position der Netzkabel auf der senkrechten Achse
		 *  (Verschiebung nach oben, alle sind auf gleicher H&ouml;he)
		 */
		const double		cable_posy;	// pos on y

		/** Dateiname, in dem das aktuell Netzwerk gespeichert ist /
		 *  gesichert werden soll */
		string			filename;
		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree		current_infos;
		/** Feld aller Netzknoten (Net_Switch, L3_Router, Host) */
		vector<Node*>		nodes;
		/** Feld aller Netzkabel des Netzwerkes */
		vector<Net_Cable*>	cables;
		/** Server zum Paketaustausch mit externen Programmen
		 *  (momentan nur Richtung zu 'netview' realisiert) */
		PacketServer		packet_socket_io;

		void attachCablesToNodes(void);

		void executeNetImportPacket(Packet* cable_packet);
		bool executeNetCables(double speed_factor,
					Point_3D &camera_pos,
					double &camera_angle,
					bool nodes_need_turn,
					string &current_cable_name);
		bool executeNetNodes(double speed_factor, Point_3D &camera_pos,
					double &camera_angle,
					bool &needs_turn,
					string &current_node_name);
	public:
		Network(void);
		~Network(void);
		void clear(void);
		void init(unsigned short new_server_port = 0);

		//@{ @name Dialog-Datenaustausch
		void getInfos(Variable_Tree* &infos);
		void setInfos(Variable_Tree* infos);
		void getInfos(double posxz[2], Variable_Tree* &infos);
		void setInfos(double posxz[2], Variable_Tree* infos);
		//@}

		// returns if camera_pos, camera_angle has been changed
		bool executeNet(double speed_factor, Point_3D &camera_pos,
				double &camera_angle, string &current_name);
		//@{ @name Einf&uuml;gen, Verschieben, L&ouml;schen
		// y_pos is standard and set inside
		void insert_host(double new_xpos, double new_zpos);
		void insert_router(double new_xpos, double new_zpos);
		void insert_cable(double begin_posxz[2], double end_posxz[2]);
		void move(double old_posxz[2], double new_posxz[2],
				bool move_node_with_cable = false);
		void remove(double old_xpos, double old_zpos);
		//@}

		void load(const char filename_to_load[] = "");
		void save(void);
		void draw(bool with_cable_control_points = false);
		void drawSetCameraPos(bool level_of_details_on,
					Point_3D camera_pos);
};

