/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef ROUTE_H
#define ROUTE_H 1

#include <string>
#include <libxml/xmlreader.h>
#include <libxml/xmlwriter.h>
#include <net/route.h>

#include "ipaddress.h"
#include "ipv4_packet.h"

class IPv4_Packet;

using namespace std;

class Route
/** ein Routen-Eintrag innerhalb der Routentabelle Route_Table */
{
	private:
		/** Struktur, in der alle Informationen zur Route gespeichert
		 *  werden */
		struct rtentry	route_entry;

		void	genIpAddress(IpAddress &destination,
					IpAddress &gateway,
					IpAddress &genmask) const;
		bool	takeAddress(string &new_route,
					struct sockaddr &target);
		void	updateDeviceName(string new_device);

	public:
		Route(void);
		~Route(void);
		void	init(void);

		string	getRoute(void);
		string	getInterface(void);
		void	getGatewayAddress(IpAddress &gateway_address);
		bool	setRoute(string new_route);
		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;

		// returns number of matching bits in destination/genmask
		// or -1 if route don't match
		short	matchRoute(IPv4_Packet *ip_packet) const;
};

#endif
