/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef ROUTE_TABLE_H
#define ROUTE_TABLE_H 1

#include <vector>

#include "route.h"
#include "variable_tree.h"
#include "stroke_font.h"

class Route_Table
/** Klasse zur Darstellung und Simulation einer Routentabelle */
{
	private:
		/** zeigt an, ob die Suche einer Route f&uuml;r
		 *  packet_for_routing (passiert in mehreren Schritten)
		 *  abgeschlossen ist, siehe processPacket(), executeNet() */
		bool		search_finished;
		/** Index auf die letzte passende (last_found_route) und die
		 *  aktuell verglichene Route (current_route) */
		size_t		last_found_route, current_route;
		/** Paket, f&uuml;r das in mehreren Schritten eine Route
		 *  gesucht wird, siehe processPacket() */
		IPv4_Packet*	packet_for_routing;
		/** die eigentliche Tabelle als Feld von Routen */
		vector<Route*>	route_array;
		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree	current_infos;

		/** Position (text_pos) und Gr&ouml;&szlig;e (text_size) des
		 *  Textbereiches, mit dem die Tabelle dargestellt wird */
		Point_3D	text_pos, text_size;
		/** Farbe, Textgr&ouml;&szlig;e und Schriftart zur
		 *  Darstellung */
		Stroke_Font	route_text_output;

		void	drawColorBackground(size_t current_i,
					class Rectangle &background) const;
		void	resize(size_t new_number_of_routes);
	public:
		Route_Table();
		~Route_Table();
		void	clear(void);
		void	draw(bool level_of_details_on,
				Point_3D camera_pos) const;
		void	init(void);

		void	setColor(double new_red, double new_green,
				double new_blue, double new_alpha);
		void	setPos(double new_centerx, double new_centery,
				double new_centerz);
		void	setSize(double new_width, double new_height,
				double new_depth);

		//@{ @name XML- und Dialog-Datenaustausch
		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;
		void	getInfos(Variable_Tree* &infos);
		void	setInfos(Variable_Tree* infos);
		//@}

		//@{ @name Netzwerksimulation
		string	findInterface(IPv4_Packet *ip_packet,
					Route* &found_route);
		// for L3_Router:
		void	executeNet(double speed_factor);
		bool	foundRoute(IPv4_Packet* &ip_packet,
					Route* &found_route);
		bool	getPacketPos(Point_3D &packet_pos) const;
		bool	processPacket(IPv4_Packet* ip_packet);
		//@}
};

#endif
