/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "cuboid.h"
#include "cylinder.h"
#include "host_executenet.h"
#include "node.h"

class Host : public Cuboid, public Node, private Host_ExecuteNet
/** Klasse zur Darstellung und Simulation eines Hosts (Computer mit nur EINER
 *  Netzwerkkarte und Verarbeitung von Diensten) */
{
	private:
		/** aktueller Rotationswinkel um die Senkrechte */
		double		direction_angle;
		/** die Quader als Seitenw&auml;nde der Host */
		Cuboid		walls[2];
		/** Zylinder als Boden und 'Dach' des Hosts */
		Cylinder	top, bottom;
		/** Zylinder als Transportschienen (vertikal) der Pakete im
		 *  Host */
		Cylinder	poles[4];
		/** letzter bekannter Zustand des Level of Detail (an/aus),
		 *  muss zwischengespeichert werden */
		bool		last_level_of_details_on;
		/** letzte bekannte Kameraposition, auch f&uuml;r Level of
		 *  Detail, muss zwischengespeichert werden */ 
		Point_3D	last_camera_pos;
		/** Displaylisten zur Beschleunigung der Darstellung des Hosts
		 *  und seiner Pakete */
		GLuint		display_list_host, display_list_packets;

		/** der Name des Hosts */
		string		name;
		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree	current_infos;

		// defined in Host_ExecuteNet:
		// Interface		network_card;
		// Route_Table		route_table;
		// TcpQueue		tcp_queue;
		// UdpQueue		udp_queue;
		// list<Service*>	services;

		void		updatePos(void);
		void		updateDisplaylistHost(void);
		void		updateDisplaylistPackets(void);

	public:
		Host(void);
		~Host(void);
		void	draw(void) const;
		void	drawSetCameraPos(bool level_of_details_on,
						Point_3D camera_pos);
		void	init(void);

		void	getInfos(Variable_Tree* &infos);
		void	setInfos(Variable_Tree* infos);

		//@{ @name Knotenfunktionen
		void	attachCable(Net_Cable *new_cable);
		void	detachCables(void);
		// returns true if there is a packet in one of the queues/cards
		bool	getFirstPacketPos(Point_3D &packet_pos) const;
		void	executeNet(double speed_factor);
		void	executeNetImportPacket(Packet* cable_packet);
		void	getName(string &node_name) const;
		bool	hasIpAddress(IpAddress &searched_address) const;
		bool	hasMacAddress(MacAddress &searched_address) const;
		bool	needsTurn(void) const;
		char*	getAddress(void) const;
		void	setAddress(char *ip_string);
		//@}

		//@{ @name Position, Winkel, Farbe, Gr&ouml;&szlig;e
		void	getPos(double &current_centerx, double &current_centery,
				double &current_centerz) const;
		void	getPosAngle(Point_3D &current_pos,
				double &current_angle) const;
		void 	setColor(double new_red,  double new_green, 
					 double new_blue, double new_alpha);
		void	setPos(double new_centerx, double new_centery,
				double new_centerz);
		void	setSize(double new_width, double new_height, 
				double new_depth);
		//@}

		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;
};

