/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "interface.h"
#include "route_table.h"
#include "tcpqueue.h"
#include "service.h"
#include "udpqueue.h"

class Host_ExecuteNet
/** Erweiterungsklasse zur Simulation der Paketverarbeitung im Host */
{
	protected:
		/** die Netzwerkkarte des Hosts */
		Interface	network_card;
		/** Routentabelle des Hosts, vor allem f&uuml;r
		 *  Gateway-Adressen */
		Route_Table	route_table;
		/** die Schlange der TCP-Pakete im Host */
		TcpQueue	tcp_queue;
		/** die Schlange der UDP-Pakete im Host */
		UdpQueue	udp_queue;
		/** Liste aller Dienste, die im Host bekannt sind
		 *  (und meist auch laufen) */
		list<Service*>	services;

	private:
		bool	serviceSend(void);
		bool	tcpSend(void);
		bool	udpSend(void);
		void	intoInterface(IPv4_Packet* ip_packet);
		bool	ipReceive(void);
		bool	tcpReceive(void);
		bool	udpReceive(void);
		bool	serviceReceive(Service_Packet *data_packet);

	public:
		Host_ExecuteNet(void);
		~Host_ExecuteNet(void);

		void	executeNet(double speed_factor,
					double &direction_angle);
		void	importPacket(Packet* cable_packet);
};

