/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "cuboid.h"
#include "cylinder.h"
#include "l3_router_executenet.h"
#include "node.h"

class L3_Router : public Cuboid, public Node, private L3_Router_ExecuteNet
/** Klasse zur Simulation und Darstellung eines Routers (f&uuml;r IP-Pakete) */
{
	private:
		/** aktueller Rotationswinkel um die Senkrechte */
		double			direction_angle;
		/** die Quader als Seitenw&auml;nde des Routers */
		Cuboid			walls[2];
		/** die Zylinder als Basis und 'Dach' des Routers */
		Cylinder		top, bottom;
		/** die Zylinder als Transportschienen (vertikal) der Pakete
		 *  im Router */
		Cylinder		poles[4];
		/** der letzte bekannte Zustand des Level of Detail (an/aus),
		 *  muss zwischengespeichert werden */
		bool			last_level_of_details_on;
		/** die letzte bekannte Kameraposition relativ zur
		 *  Routerposition (Kamera im Router->pos=(0,0,0)),
		 *  auch f&uuml;r Level of Detail,
		 *  muss zwischengespeichert werden */
		Point_3D		last_camera_pos;
		/** die Displayliste zur Beschleunigung der Darstellung des
		 *  Routers */
		GLuint			display_list_l3_router,
		/** die Displayliste zur Beschleunigung der Darstellung der
		 *  Pakete im Router */
					display_list_packets;

		/** der Name des Routers */
		string			name;
		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree		current_infos;

		// defined in L3_Router_ExecuteNet
		// vector<Interface*>	network_cards;
		// Route_Table		route_table;
		// NAT			nat_packet_translator;

		void	resizeInterfaceArray(size_t new_size);
		void	updatePos(void);
		void	updateDisplaylistL3Router(void);
		void	updateDisplaylistPackets(void);

	public:
		L3_Router();
		~L3_Router();

		void	draw(void) const;
		void	drawSetCameraPos(bool level_of_details_on,
						Point_3D camera_pos);
		void	init(void);

		//@{ @name Knotenfunktionen
		void	attachCable(Net_Cable *new_cable);
		void	detachCables(void);
		// returns true if there is a packet in one of the cards
		bool	getFirstPacketPos(Point_3D &packet_pos) const;
		void	executeNet(double speed_factor);
		void	executeNetImportPacket(Packet* cable_packet);
		void	getName(string &node_name) const;
		bool	hasIpAddress(IpAddress &searched_address) const;
		bool	hasMacAddress(MacAddress &searched_address) const;
		bool	needsTurn(void) const;
		//@}

		void	getInfos(Variable_Tree* &infos);
		void	setInfos(Variable_Tree* infos);
		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;

		//@{ @name Position, Winkel, Farbe, Gr&ouml;&szlig;e
		void	getPos(double &current_centerx, double &current_centery,
				double &current_centerz) const;
		void	getPosAngle(Point_3D &current_pos,
				double &current_angle) const;
		void	setColor(double new_red,  double new_green, 
					 double new_blue, double new_alpha);
		void	setPos(double new_centerx, double new_centery,
				double new_centerz);
		void	setSize(double new_width, double new_height,
				double new_depth);
		//@}
};

