/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "cuboid.h"
#include "ethernetqueue.h"
#include "node.h"

class Net_Switch : public Cuboid, public Node
/** Klasse zur Simulation und Darstellung eines Switches
 *  (verarbeitet nur Ethernet-Pakete), nicht implementiert */
{
	private:
		/** Feld von Ethernet-Warteschlangen, je Anschluss eine,
		 *  zum Senden und Empfangen von Paketen */
		vector<EthernetQueue*>		connectors;
		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree			current_infos;
	public:
		Net_Switch(void);
		~Net_Switch(void);
		/** zur Zeit leer, <br> Initialisierung des Switches */
		void	init(void);

		void	getInfos(Variable_Tree* &infos);
		void	setInfos(Variable_Tree* infos);
		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;

		//@{ @name Knotenfunktionen
		void	attachCable(Net_Cable *new_cable);
		void	detachCables(void);
		// returns true if there is a packet in one of the cards/queues
		bool	getFirstPacketPos(Point_3D &packet_pos) const;
		void	executeNet(double speed_factor);
		void	executeNetImportPacket(Packet* cable_packet);
		void	getName(string &node_name) const;
		bool	hasIpAddress(IpAddress &searched_address) const;
		bool	hasMacAddress(MacAddress &searched_address) const;
		// return if (one) interface has to be turned
		bool	needsTurn(void) const;
		//@}

		//@{ @name Position, Winkel, Farbe, Gr&ouml;&szlig;e
		void	getPos(double &current_centerx, double &current_centery,
					double &current_centerz) const;
		void	getPosAngle(Point_3D &current_pos,
					double &current_angle) const;
		void	setColor(double new_red,  double new_green, 
					double new_blue, double new_alpha);
		void	setPos(double new_centerx, double new_centery,
					double new_centerz);
		void	setSize(double new_width, double new_height,
					double new_depth);
		//@}

		void	draw(void) const;
		void	drawSetCameraPos(bool level_of_details_on,
						Point_3D camera_pos);
};

