/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef IPV4_PACKET_H
#define IPV4_PACKET_H 1

#include <netinet/ip.h>

#include "ethernet_packet.h"
#include "ipaddress.h"
#include "route.h"

class Route;

class IPv4_Packet : public Ethernet_Packet
/** Klasse zur Beschreibung und Darstellung eines IPv4-Packetes */
{
	protected:
		/** Daten im Kopf eines IPv4-Packetes */
		struct ip	ipv4_header;

		// /usr/include/linux/skbuff.h
		/** Gateway-Adresse aus der Route bzw. n&auml;chste IP-Adresse
		 *  &uuml;ber die das Paket geschickt werden soll, kann auch
		 *  der Zieladresse entsprechen */
		IpAddress	next_hop_address;
		/** Zeiger auf die Route, &uuml;ber die das Paket verschickt
		 *  werden soll -> n&ouml;tig f&uuml;r next_hop_address */
		Route		*route_to_destination;

		void		initPacketHeaders(void);
		void		updateIpChecksum(void);
		void		updatePacketText(void);
	public:
		IPv4_Packet(void);
		virtual ~IPv4_Packet(void);
		virtual void	copy(Packet* &copy_of_packet);
		virtual void	loadXML(xmlTextReaderPtr infile);
		virtual void	saveXML(xmlTextWriterPtr outfile) const;

		void		importHeader(struct ip *new_ipv4_header);
		void		exportHeader(
					struct ip *current_ipv4_header) const;

		//@{ @name Ports, Adressen
		virtual unsigned short	getSourcePort(void) const;
		virtual unsigned short	getDestinationPort(void) const;
		virtual void		setDestinationPort(
						unsigned short dest_port);
		virtual void		setSourcePort(
						unsigned short source_port);


		bool		setDestinationIpAddress(const char *ip_string);
		bool		setSourceIpAddress(const char *ip_string);
		void		setDestinationIpAddress(IpAddress address);	
		void		setSourceIpAddress(IpAddress address);
		void		getDestinationIpAddress(
					IpAddress &address) const;
		void		getSourceIpAddress(IpAddress &address) const;
		void		getNextHopAddress(
					IpAddress &hop_address) const;
		//@}

		u_int8_t	getTTL(void) const;
		void		setTTL(u_int8_t new_ttl);
		void		decreaseTTL(void);
		void		setRoute(Route* new_route_to_destination);
};

#endif
