/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef PACKET_H
#define PACKET_H 1

#include <string>
#include <GL/glu.h>

#include "cuboid.h"
#include "point_3d.h"
#include "stroke_font.h"

/** beschreibt die mglichen Protokolle des Paketes */
typedef enum {NO_PROTOCOL, SERVICE_PACKET, TCP, UDP, ICMP, IGMP, IPv4, IPv6, 
		ARP, ETHERNET} Protocol;
	
class Packet
/** Beschreibung und Darstellung eines Netzwerk-Paketes */
{
	private:
		//@{ @name Protokoll
		/** Paket dieses Netzwerk-Protokolls */
		Protocol 	l1_type, l2_type, l3_type, l4_type;
		//@}

		//@{ @name Positionen, Farbe und Winkel
		/** Drehwinkel, mit dem Paket gezeichnet wird */
		double		y_angle;
		/** zurckgelegter Weg des Paketes, rcksetzbar */
		double		moved_way;
		/** Position und Farbe des Paketes */
		Point_3D	view;
		/** Position bei Einf&uuml;gen ins Kabel */
		Point_3D	first_pos;
		//@}

		/** Displaylisten f&uuml;r Quader und Beschriftung,
		 *  zur Beschleunigung des Zeichnens */
		GLuint		display_list_cuboids, display_list_text;

		/** Beschriftung des Paketes, und vorheriger Stand (old),
		 *  wird bei jeder neuen Beschriftung gesichert */
		string		packet_text[4], old_packet_text[4];
		/** die Quader, welche die Paketbestandteile darstellen:
		 *  K&ouml;pfe (headers) und Inhalt (content) */
		Cuboid		headers[3], content;
		/** Farbe und Schriftart der aktuellen Beschriftung */
		Stroke_Font	stroke_text_output[4];
		/** Farbe und Schriftart des vorherigen Standes,
		 *  (dieser wird auch mit gezeichnet) */
		Stroke_Font	stroke_old_text_output[4];

		void		loadXMLPos(xmlTextReaderPtr infile,
						int return_depth,
						Point_3D &pos) const;
		void		updateDisplaylistCuboids(void);
		void		updateDisplaylistText(void);
	public:
		Packet();
		Packet(Protocol new_l3_protocol);
		Packet(const Packet &forcopy);
		virtual ~Packet(void);
		// copies the data of the packet in its original class
		// The Argument "copy_of_packet" must be NULL to get a
		// new Packet in the original class, else only copy of the data
		virtual void copy(Packet* &copy_of_packet);
		virtual void loadXML(xmlTextReaderPtr infile);
		virtual void saveXML(xmlTextWriterPtr outfile) const;

		bool	isProtocol(Protocol test_protocol);
		void	setProtocol(unsigned char level, Protocol new_protocol);
		void	clearOldTextLines(void);
		string	getTextLine(unsigned char level) const;
		void	setTextLine(unsigned char level, string new_text);

		//@{ @name Positionen, Winkel, Gr&ouml;&szlig;e, Farbe
		void	setPos(double new_centerx, double new_centery, 
				double new_centerz);
		void	getPos(double &my_centerx, double &my_centery,
				double &my_centerz) const;
		void	setFirstPos(double new_firstx, double new_firsty,
					double new_firstz);
		void	getFirstPos(double &my_firstx, double &my_firsty,
					double &my_firstz) const;
		void	setAngle(double new_angle);
		double	getAngle(void) const;
                void	setSize(double new_width, double new_height,
				double new_depth);
		void	getSize(double &current_width, double &current_height,
				double &current_depth) const;
                void	setColor(double new_red, double new_green,
					double new_blue, double new_alpha);
		//@}

		//@{ @name Kopfverschiebungen, zur&uuml;ckgelegter Weg, Zeichnen
		void	moveHeader(unsigned char level, double move_x);
		double	getHeaderXPos(unsigned char level) const;
		void	setMovedWay(double new_moved_way = 0.0);
		double  getMovedWay(void) const;
		void	draw(bool level_of_details_on,
				Point_3D camera_pos) const;
		//@}
};

#endif

