/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "ethernet_packet.h"
#include "net_queue.h"

class EthernetQueue : public Net_Queue
/** Warteschlange f&uuml;r Ethernet-Pakete, Senden und Empfangen in getrennten
 *  Schlangen */
{
	private:
		/** Ethernet-Adresse der Schlange, als Sende-Adresse angegeben
		*/
		MacAddress		address;
		/** Liste der Pakete zum Senden/Empfangen */
		list<Ethernet_Packet*>	up_queue, down_queue;
		/** Baum von Variablen f&uuml;r das Dialogfenster, zum
		 *  interaktiven Ver&auml;ndern der Einstellungen */
		Variable_Tree		current_infos;

	public:
		EthernetQueue();
		~EthernetQueue();
		void	drawPackets(bool level_of_details_on,
					Point_3D camera_pos) const;
		void	init(void);
		void	setSize(double new_width, double new_height, 
				double new_depth);

		string	getAddress(void) const;
		void	getAddress(MacAddress &current_address) const;
		bool	setAddress(const char mac_string[]);

		void	getInfos(Variable_Tree* &infos);
		void	setInfos(Variable_Tree* infos);
		void	loadXML(xmlTextReaderPtr infile);
		void	saveXML(xmlTextWriterPtr outfile) const;

		//@{ @name Up, Down, Erstes Packet
		size_t	down_size(void) const;
		void	down_in(Ethernet_Packet* incoming_packet); // end 
		void	down_out(Ethernet_Packet* &outgoing_packet); // begin
		size_t	up_size(void) const;
		void	up_in(Ethernet_Packet* incoming_packet); // end of queue
		void	up_out(Ethernet_Packet* &outgoing_packet); // begin
		// returns true if there is a packet in one of the queues
		bool	getFirstPacketPos(Point_3D &packet_pos) const;
		//@}
};

