/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef NET_QUEUE_H
#define NET_QUEUE_H 1

#include <libxml/xmlreader.h>
#include <libxml/xmlwriter.h>

#include "packet.h"
#include "cuboid.h"
#include "variable_tree.h"

using namespace std;

class Net_Queue : public Cuboid
/** abstrakte Basisklasse aller Warteschlangen, teilweise implementiert */
{
	protected:
		/** Geschwindigkeitsmultiplikator f&uuml;r Pakettransport etc.
		 */
		double		animation_speed_factor;
		/** Darstellung einer Schlange durch zwei Quader */
		Cuboid		left, right;
		/** Beschriftung der Schlange, quer auf den Quadern */
		string		label_text;
		/** Schriftart, Gr&ouml;&szlig;e, Farbe der Beschriftung */
		Stroke_Font	label;

		template <class PacketClass>
		static void	internalLoadXMLPacket(xmlTextReaderPtr infile,
						const xmlChar xml_tag_name[],
						Packet* &new_packet);
		template <class PacketClass>
		void		loadXMLPackets(xmlTextReaderPtr infile,
					list<PacketClass*> &up_down_queue);
		template <class PacketClass>
		static bool	getFirstPacketPos(PacketClass *type_sample,
					Point_3D &packet_pos,
					const list<PacketClass*> &down_queue,
					const list<PacketClass*> &up_queue);
		template <class PacketClass>
		void		drawPackets(PacketClass *type_sample,
					const list<PacketClass*> &down_queue,
					const list<PacketClass*> &up_queue,
					bool level_of_details_on,
					Point_3D camera_pos) const;
		// direction: -1 / 0 / 1
		void		movePacket(Packet* moving_packet,
						char direction,
						unsigned char level);
		template <class PacketClass, typename IteratorType>
		void		down_movePackets(unsigned char level,
					list<PacketClass*> &down_queue);
		template <class PacketClass, typename IteratorType>
		void		up_movePackets(unsigned char level,
					list<PacketClass*> &up_queue);


		void	down_in(Packet* incoming_packet,
				unsigned char level);		// end of queue 
		bool	down_out(Packet* outgoing_packet);	// begin
		void	up_in(Packet* incoming_packet);		// end of queue
		bool	up_out(Packet* outgoing_packet);	// begin
	public:
		Net_Queue(void);
		virtual ~Net_Queue(void);
		void		init(void);
		void		updatePos(void);

		static void	loadPacketClasses(xmlTextReaderPtr infile,
							Packet* &new_packet);
		/** stellt den Variablenbaum f&uuml;r das Dialogfenster
		 *  zusammen, Zeiger darauf in 'infos' zur&uuml;ck */
		virtual void	getInfos(Variable_Tree* &infos) = 0;
		/** liest die Werte aus dem Variablenbaum ein */
		virtual void	setInfos(Variable_Tree* infos) = 0;
		/** l&auml;dt Einstellungen und Pakete aus XML-Datei 'infile'
		 */
		virtual void	loadXML(xmlTextReaderPtr infile) = 0;
		/** speichert Einstellungen und Pakete in XML-Datei 'outfile'
		 */
		virtual void	saveXML(xmlTextWriterPtr outfile) const = 0;

		void		setAnimationSpeed(double speed_factor);
		void		setColor(double new_red, double new_green,
					double new_blue, double new_alpha);
		void		drawQueue(void) const;
};

#include "../src/queues/net_queue_template.cpp"

#endif
