/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#ifndef VARIABLE_LIST_H
#define VARIABLE_LIST_H 1

#include <list>
#include <string>

#include "variable.h"

using namespace std;

class Variable_Tree
/** Baum von Variablen und Unterb&auml;umen ('Baum von B&auml;umen') zur
 *  hierarchischen Strukturierung von Variablen, jeder Baum hat einen Namen */
{
	private:
		/** Name des Baumes wurde &uuml;ber getNextVar() schon
		 *  herausgegeben ('true') */
		bool				name_was_given_out;
		/** Name dieses Baumes, zur Identifikation */
		String_Variable			name;
		/** Liste aller Variablen dieses Baumes */
		list<Variable*>			variables;
		/** Liste aller Unterb&auml;ume dieses Baumes */
		list<Variable_Tree*>		children;
		/** die n&auml;chste Variable, die mit getNextVar()
		 *  herausgegeben wird */
		list<Variable*>::iterator	current_var;
		/** der n&auml;chste Unterbaum, der in getNextVar() nach
		 *  Variablen gefragt wird (wenn current_var am Ende der
		 *  Variablenliste ist) */
		list<Variable_Tree*>::iterator	current_child;

	public:
		Variable_Tree(void);
		~Variable_Tree(void);

		//@{ @name Hinzuf&uuml;gen und Suchen
		void	addVar(Variable* new_variable);
		void	addChild(Variable_Tree* new_child);

		bool	findVar(string varname, Variable* &found_var);
		bool	findChild(string childname, 
					Variable_Tree* &found_child);
		//@}

			/* delete all */
		void	clear(void);
		void	gotoBegin(void);
		bool	getNextVar(Variable* &next_var, size_t &depth);

		size_t	getNumVars(void);
		string	getName(void);
		void	setName(string new_name);
};

#endif
