/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#ifdef WIN32

#include <winsock.h>

#ifndef INET_ATON_FUNC
#define INET_ATON_FUNC 1

/** Funktion zum Umwandeln eines Adress-Strings in eine numerische IP-Adresse, 
 * implementiert in der Datei win_funcs.cpp */
extern int inet_aton(const char *cp, struct in_addr * inp);

#endif	// INET_ATON_FUNC

/** Typ in_addr_t wird nicht in winsock.h (oder irgendeiner anderen Datei) 
 * definiert */
typedef		unsigned int	in_addr_t;

#ifndef CLOSE_FUNC
#define CLOSE_FUNC 1

/** schlie&szlig;t den Socket 'socket', nutzt closesocket(), <br>
 *  implementiert in win_funcs.cpp */
extern int close(int socket);

#endif  // CLOSE_FUNC

#ifndef FCNTL_FUNC
#define FCNTL_FUNC 1

#define F_SETFL		4
#define O_NONBLOCK	0x4000

/** setzt nicht-blockierende Ein-/Ausgabe auf dem Socket 'socket', wenn
 *  'operation' == 'F_SETFL' und 'parameter' = 'O_NONBLOCK', <br>
 *  es werden keine anderen Operationen von fcntl unterst&uuml;tzt, <br>
 *  implementiert in win_funcs.cpp */
extern int fcntl(int socket, int operation, int parameter);

#endif	// FCNTL_FUNC

#ifndef INIT_SOCKET_FUNC
#define INIT_SOCKET_FUNC 1

/** initialisiert die Socket-Umgebung unter MS Windows */
extern int init_socket(void);

#endif	// INIT_SOCKET_FUNC

#else

/**
 * Diese Datei deklariert die Funktion inet_aton() und den Typ in_addr_t.
 * Beide sind nicht in der Datei winsock.h (oder irgendeiner anderen) 
 * deklariert bzw. definiert <br>
 * <br>
 * extern int inet_aton(const char *cp, struct in_addr * inp); <br>
 * Funktion zum Umwandeln eines Adress-Strings in eine numerische IP-Adresse, 
 * implementiert in der Datei win_funcs.cpp <br>
 * <br>
 * typedef		unsigned int	in_addr_t; <br>
 * Typ in_addr_t wird nicht in winsock.h (oder irgendeiner anderen Datei) 
 * definiert <br>
 * <br>
 * extern int close(int socket); <br>
 * schlie&szlig;t den Socket 'socket', nutzt closesocket(), <br>
 * implementiert in win_funcs.cpp, <br>
 * <br>
 * extern int fcntl(int socket, int operation, int parameter); <br>
 * setzt nicht-blockierende Ein-/Ausgabe auf dem Socket 'socket', wenn
 * 'operation' == 'F_SETFL' und 'parameter' = 'O_NONBLOCK', <br>
 * es werden keine anderen Operationen von fcntl unterst&uuml;tzt, <br>
 * implementiert in win_funcs.cpp <br>
 * <br>
 * extern int init_socket(void); <br>
 * initialisiert die Socket-Umgebung unter MS Windows, gibt den Wert von
 * WSAStartup zur&uuml;ck; <br>
 * implementiert in win_funcs.cpp <br>
 */
#define DOXYGEN 1

#endif

