/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <iostream>

#include "glut.h"
#include "dialog.h"

/** Standardwert der Randbreite um den Text zum Dialogfensterrahmen */
const int		dialog_text_border = 10;
/** Anfangsgr&ouml;&szlig;e des Dialogfensters (Breite mal H&ouml;he in
 *  Zeichen) */
unsigned int		start_dialog_size[] = {70, 0};
/** Faktor zur Berechnung der H&ouml;he eines Zeichens: hier 2 * Breite */
const unsigned short	height_width_factor = 2;	// height = 2 * width
/** Standard-Schriftart von GLUT zur Textdarstellung im Dialog */
void			*dialog_font = GLUT_BITMAP_9_BY_15;

Dialog::Dialog(void): 
			border(dialog_text_border), 
			insert_mode(false), content_changed(false),
			window_id(-1), num_columns(0), num_rows(0),
			cursor_column(0), cursor_row(0),
			current_width(0), current_height(0),
			font(NULL)
/** Konstruktor, Initialisierung der Variablen: border, insert_mode,
 *  content_changed, window_id, num_columns, num_rows, cursor_column,
 *  cursor_row, current_width, current_height, font */
{
};

Dialog::~Dialog(void)
/** Destruktor, ruft nur destroy() zum L&ouml;schen des Fensters */
{
	destroy();
};

bool Dialog::isChanged(void)
/** gibt zur&uuml;ck, ob der Inhalt des Dialogs ge&auml;ndert wurde, also
 *  ob Variablenwerte angepasst wurden ('true', sonst 'false') */
{
	return (content_changed);
};

bool Dialog::isHidden(void)
/** gibt zur&uuml;ck, ob das Fenster versteckt ist, weil es gerade nicht
 *  gebraucht wird (Anzahl dargestellter Textzeilen == 0)
 *  ('true', sonst 'false') */
{
	return (text.size() == 0);
};

void Dialog::init(struct callback_struct &callback_functions)
/** Initialisierung (nach Start der GLUT-Umgebung): Schriftart einstellen,
 *  Fenster &ouml;ffnen und verstecken, Callback-Funktionen eintragen und
 *  Gr&ouml;&szlig;e setzen mit setSize() */
{
	int	old_window_id;

	// save old id, set font
	// cout << "Dialog::init start" << endl;
	old_window_id = glutGetWindow();
	font = dialog_font;

	// create window
	// cout << "Create Window" << endl;
	window_id = glutCreateWindow("Dialog");
	// cout << "Dialog::init created" << endl;
	glutSetWindow(window_id);
	glutHideWindow();
	// set callbacks
	glutDisplayFunc(callback_functions.display_callback);
	glutKeyboardFunc(callback_functions.keyboard_callback);
	glutSpecialFunc(callback_functions.specialkey_callback);
	// resize, return to former window
	setSize(start_dialog_size[0], start_dialog_size[1]);
	glutSetWindow(old_window_id);
	// cout << "Dialog::init end" << endl;
};

void Dialog::destroy(void)
/** l&ouml;scht das Fenster */
{
	if (window_id >= 0)
		glutDestroyWindow(window_id);
	window_id = -1;
};

void Dialog::draw(void)
/** zeichnet das Fenster mit Text */
{
	unsigned int	x, y;
	double		line_height, leftx, topy, char_height, char_width;
	double		width_to_cursor;

	// cout << "Dialog::draw: " << endl;
	if (num_rows == 0) return;

	// size, viewport, color
	setSize(num_columns, num_rows);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glViewport(0, 0, current_width, current_height);
	glColor3d(1.0, 1.0, 1.0);
	if (font)
	{
		// size of a character, begin of draw
		char_width = glutBitmapWidth(font, 'W');
		char_height = 1.0 * height_width_factor * char_width;
		line_height = (2.0 * char_height) / current_height;
		leftx = -1.0 + (2.0 * border / current_width);
		topy = 1.0 - (2.0 * border) / current_height;
		width_to_cursor = 0.0;
		// draw the text
		for(y = 0; y < text.size(); y++)
		{
			glRasterPos2d(leftx, topy - line_height * (y + 1));
			for(x = 0; x < text[y].length(); x++)
			{
				if ((y == cursor_row) && (x < cursor_column))
					width_to_cursor += glutBitmapWidth(
							font, text[y][x]);
				glutBitmapCharacter(font, text[y][x]);
			};
		};
		// draw the cursor
		topy = topy - line_height * (cursor_row + 1);
		glRasterPos2d(leftx +  2.0 * width_to_cursor / current_width, 
				topy - line_height * 0.1);
		glutBitmapCharacter(font, '_');
		// cout << "xy" << leftx + char_width * cursor_column << " "
		// 	<< topy - line_height * (cursor_row + 1) << endl;
		// cout << line_height << endl;
	};
	glFlush();
	glutSwapBuffers();
};

void Dialog::redraw(void)
/**
 *  - Fenster neu zeichnen und in den Vordergrund holen oder
 *  - Fenster verstecken, wenn es nichts darzustellen gibt */
{
	int	old_window_id;

	if (text.size() > 0)
	{
		old_window_id = glutGetWindow();
		glutSetWindow(window_id);
		glutShowWindow();
		glutSetWindow(old_window_id);
		glutPostWindowRedisplay(window_id);
	}
	else
	{
		old_window_id = glutGetWindow();
		glutSetWindow(window_id);
		glutHideWindow();
		glutSetWindow(old_window_id);
	};
};

bool Dialog::key(unsigned char k, int x, int y)
/** Callback-Funktion f&uuml;r Tastatureingaben, 'k' ist die gedr&uuml;ckte
 *  Taste, 'x' und 'y' geben die Mausposition relativ zum Fenster an, <br>
 *  gibt 'true' zur&uuml;ck, wenn die Werte des Variablenbaums wieder ins
 *  Programm &uuml;bernommen werden sollen (nach 'Enter'), sonst 'false' */
{
	bool	constant, do_update, do_delete;
	size_t	i;

	do_update = false;
	do_delete = false;
	constant = (left_editable[cursor_row] > right_editable[cursor_row]);
	// if (((k >= 'A') && (k <= 'Z')) || ((k >= 'a') && (k <= 'z')) ||
	//	((k >= ' ') && (k <= '?')) || (k == '_'))
	// insert normal letter/character in text for insert mode or
	// normal mode (overwrite)
	if ((k >= ' ') && (k <= '~') && (!constant))
	{
		if (text[cursor_row].size() < (cursor_column + 1))
		{
			text[cursor_row].resize(cursor_column + 1);
			right_editable[cursor_row]++;
		}
		else if (insert_mode)
		{
			text[cursor_row].append(1, ' ');
			right_editable[cursor_row]++;
			for(i = right_editable[cursor_row] - 1;
				i > cursor_column; i--)
				text[cursor_row][i] = text[cursor_row][i - 1];
		};
		text[cursor_row][cursor_column] = k;
		cursor_column++;
	};
	if ((k == 8) && (!constant))	// Backspace
		if (cursor_column > left_editable[cursor_row])
		{
			cursor_column--;
			do_delete = true;
		};
	if (k == 9)	// Tab
	{
		if ((cursor_row + 1) < num_rows) cursor_row++;
		else cursor_row = 0;
		cursor_column = right_editable[cursor_row];
	};
	if (k == 13)	// Enter
		do_update = true;
	if (k == 27)	// Escape
		setSize(num_columns, 0);	// hiding dialog
	if ((k == 127) && (!constant))	// Delete
		if (cursor_column < right_editable[cursor_row])
			do_delete = true;
	if (do_delete) // delete the character on cursor position
	{
		for(i = cursor_column; i < right_editable[cursor_row] - 1; i++)
			text[cursor_row][i] = text[cursor_row][i + 1];
		if (right_editable[cursor_row] > left_editable[cursor_row])
		{
			text[cursor_row].resize(text[cursor_row].size() - 1);
			right_editable[cursor_row]--;
		};
		check_cursor_column();
	};
	// status and redraw
	// cout << "Dialog: Key: " << k << endl;
	if (!do_update)  content_changed = true;
	if (num_rows == 0) content_changed = false;
	redraw();
	return do_update;
};

void Dialog::check_cursor_column()
/** pr&uuml;ft die Position des Cursors: muss immer im editierbaren Bereich
 *  liegen, Cursor wird dann bei Bedarf neu positioniert; keine Aktualisierung
 *  des Fensterinhaltes! */
{
	// cout << "Left: " << left_editable[cursor_row] << " Right: "
	//	<< right_editable[cursor_row] << endl;
	if (cursor_column < left_editable[cursor_row])
		cursor_column = left_editable[cursor_row];
	if (cursor_column > right_editable[cursor_row])
		cursor_column = right_editable[cursor_row];
	// cout << "cursor column: " << cursor_column << endl;
};

void Dialog::specialKey(int k, int x, int y)
/** Callback-Funktion f&uuml;r Tastatureingaben der Sondertasten,
 *  'k' ist die gedr&uuml;ckte Taste, 'x' und 'y' geben die Mausposition
 *  relativ zum Fenster an, <br>
 *  Sondertasten sind:
 *  - Cursor links, Cursor rechts, Cursur auf, Cursor ab
 *  - Einf&uuml;gen, Pos1, Ende, Bild auf, Bild ab
 *  - F1 - F12 (werden hier nicht genutzt) */
{
	// cout << "Dialog: specialKey: " << k;
	// cout << " " << GLUT_KEY_LEFT << " " << GLUT_KEY_RIGHT << endl;
	switch (k)
	{
		case GLUT_KEY_LEFT:	cursor_column--; 
					check_cursor_column();
					break;
		case GLUT_KEY_RIGHT: 	cursor_column++; 
					check_cursor_column();
					break;
		case GLUT_KEY_UP:	if (cursor_row > 0) cursor_row--;
					check_cursor_column();
					break;
		case GLUT_KEY_DOWN:	if (cursor_row < num_rows - 1) 
						cursor_row++;
					check_cursor_column();
					break;
		case GLUT_KEY_HOME:	cursor_column = 
						left_editable[cursor_row];
					break;
		case GLUT_KEY_END:	cursor_column =
						right_editable[cursor_row];
					break;
		case GLUT_KEY_PAGE_UP:	cursor_row = 0;
					check_cursor_column();
					break;
		case GLUT_KEY_PAGE_DOWN: cursor_row = num_rows - 1;
					check_cursor_column();
					break;
		case GLUT_KEY_INSERT:	insert_mode = !insert_mode; break;
	};
	redraw();
};

void Dialog::setSize(unsigned int new_num_columns, unsigned int new_num_rows)
/** setzt die Gr&ouml;&szlig;e des Fensters: Anzahl Zeichen je Zeile
 *  'new_num_columns' und Anzahl Zeilen 'new_num_rows', <br>
 *  Alle dargestellten Zeichen werden auf 'W' gesetzt, Breite und H&ouml;he in
 *  Pixeln wird neu berechnet, Fenster wird neu gezeichnet */
{
	int		old_window_id;
	int		char_width, line_height;
	unsigned int	y, x;
	
	if ((num_columns != new_num_columns) ||	(num_rows != new_num_rows))
	{
		// set new size
		num_columns = new_num_columns;
		num_rows = new_num_rows;
		text.resize(num_rows, " ");
		left_editable.resize(num_rows, num_columns + 1);
		right_editable.resize(num_rows, num_columns + 1);
		// change window text
		for(y = 0; y < text.size(); y++) 
		{
			text[y].resize(num_columns);
			for(x = 0; x < num_columns; x++) text[y][x] = 'W';
		};
		// calculate width and height in pixels
		char_width = 0; line_height = 0;
		if (font)
		{
			char_width = glutBitmapWidth(font, 'W');
			line_height = char_width * height_width_factor;
		};
		current_height = line_height * num_rows + 2 * border;
		current_width = char_width * num_columns + 2 * border;
		// cout << "Height: " << current_height << " " 
		//	<< line_height << endl;
	};
	// redraw
	old_window_id = glutGetWindow();
	glutSetWindow(window_id);
	glutReshapeWindow(current_width, current_height);
	glutSetWindow(old_window_id);
};

void Dialog::getText(Variable_Tree* &current_var_tree)
/** der Inhalt des Dialogfensters wird in den Variablenbaum 'current_var_tree'
 *  &uuml;bertragen, d.h. die im Dialogfenster eingetragenen Werte werden in
 *  die Variablen des Baums eingesetzt (Variable::setValue()),
 *  am Ende wird das Fenster geleert und versteckt; <br>
 *  die Reihenfolge der Zeilen im Dialog sollte der Reihenfolge der Variablen
 *  im Baum entsprechen (Variable_Tree::getNextVar()), so werden sie aber auch
 *  in setText() &uuml;bernommen */
{
	size_t		line_number, depth;
	Variable	*current_var;

	if (var_tree == NULL) return;
	// cout << "Dialog::getText" << endl;
	var_tree->gotoBegin();
	line_number = 0;
	// for all variables in tree take value out of dialog
	while(var_tree->getNextVar(current_var, depth))
	{
		// cout << "variable name: " << current_var->getName() << endl;
		text[line_number].erase(0, left_editable[line_number]);
		current_var->setValue(text[line_number]);
		//cout << "Variable value: " << current_var->getValue() << endl;
		text[line_number].resize(0);
		line_number++;
	};
	setSize(num_columns, 0);
	current_var_tree = var_tree;
	redraw();
};

void Dialog::setText(Variable_Tree* new_var_tree)
/** &uuml;bernimmt die Variablen des Baumes 'new_var_tree' in das
 *  Dialogfenster, zum Anpassen der Werte der Variablen im Baum; <br>
 *  die Reihenfolge der Variablen im Baum (Variable_Tree::getNextVar())
 *  entspricht der Reihenfolge der Zeilen im Dialog, so werden sie dann auch
 *  wieder zur&uuml;ckgegeben in getText() */
{
	size_t		line_number, depth, i;
	Variable	*current_var;
	
	// cout << "Dialog::setText" << endl;
	var_tree = new_var_tree;
	if (var_tree == NULL) setSize(num_columns, 0);
	if (var_tree == NULL) return;
	var_tree->gotoBegin();
	// cout << "num vars: " << var_tree->getNumVars() << endl;
	// number of lines is number of variables
	setSize(num_columns, var_tree->getNumVars());
	line_number = 0;
	// for all variables in tree
	while (var_tree->getNextVar(current_var, depth))
	{
		// cout << "variable: " << current_var << endl;
		text[line_number].resize(0);
		// "draw tree"
		for(i = 0; i < depth; i++) text[line_number].append("|");
		text[line_number].append("-");
		// take name
		text[line_number].append( current_var->getName());
		text[line_number].append( ": ");
		// take value and set editable frontiers
		left_editable[line_number] = text[line_number].size();
		text[line_number].append( current_var->getValue());
		right_editable[line_number] = text[line_number].size();
		if (current_var->isConstant())
			right_editable[line_number] = 
				left_editable[line_number] - 1;
		line_number++;
		// cout << "variable name: " << current_var->getName() << endl;
	};
	// set cursor
	if (num_rows > 0)
	{
		cursor_row = num_rows - 1;
		cursor_column = text[ cursor_row ].size();
	}
	else
	{
		cursor_column = 0;
		cursor_row = 0;
	};
	// cout << "Cursor pos: " << cursor_column << " " << cursor_row << endl;
	// status, redraw
	content_changed = false;
	redraw();
};

