/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"
#ifdef WIN32
#include "getopt.h"
#endif

#include <iostream>
#include <math.h>
#include <unistd.h>

#include "constants.h"
#include "glut.h"
#include "glutmain.h"

using namespace std;

//@{ @name Copyright-Text und verschiedene Einstellungen
/** Text des Copyright-Hinweises im Dialogfenster */
const char		copyright_strings[][70] =
{ "netview version 0.2",
  "Copyright (C) 2004,2005 Stefan Ziegler under GPL",
  "netview comes with ABSOLUTELY NO WARRANTY",
  "This is free software, and you are welcome to redistribute it",
  "under the conditions of the GPL",
  "Email: stefan.ziegler@s1998.tu-chemnitz.de",
  ""
};

/** Anzahl der Linien im dargestellten Gitter der Kartenansicht
 *  (jeweils waagerecht und senkrecht) */
const unsigned short	num_grid_lines = 10;
/** Mindestwartezeit in Mikrosekunden zwischen den Aktualisierungen des
 *  Fenstersinhaltes */
const int		idle_time = 10000;

/** Pfad, in dem die Screenshots und Bildfolgen der Videos gespeichert werden
 */
const char		glutmain_video_path[] = "/tmp/netview_video/";
/** Schriftart des dargestellten Namens des fokussierten Netzwerkelementes */
void*			current_name_font = GLUT_BITMAP_9_BY_15;
/** Schrittweite bei Erh&ouml;hung/Senkung des Geschwindigkeitsfaktors der
 *  Animation (Pakettransport, Netzknotenrotation) */
const double		animation_speed_step = 0.1;
//@}

//@{ @name Kommandozeilenoptionen
// For getopt
/** alle Optionen des Programms */
const char		option_string[] = "n:p:s:";
/** Option zum Laden einer Netzwerkdatei */
const int		option_load = 'n';
/** Option zum Einstellen der Portnummer, auf der der Paketserver l&auml;uft */
const int		option_server_port = 'p';
/** Option zum Setzen des Startwertes des Geschwindigkeitsfaktors der
 *  Animation (Pakettransport, Netzknotenrotation) */
const int		option_speed_factor = 's';
//@}

//@{ @name Gr&ouml;&szlig;en, Farben und Positionen
/** Anfangsgr&ouml;&szlig;e des Fensters in Pixeln (Breite, H&ouml;he) */
const unsigned short	start_window_size[2] = {800, 400};

/** Anteil der 3D-Weltansicht mit Navigation am gesamten Fenster (linke untere
 *  und rechte obere Ecke) */
const double		scene_viewport_size[4] = {0.0, 0.05, 0.5, 1.0};
/** Anteil der Karten-Draufsicht am gesamten Fenster (linke untere und
 *  rechte obere Ecke) */
const double		map_viewport_size[4] = {0.5, 0.0, 1.0, 1.0};
/** Anteil der zweiten 3D-Weltansicht ohne Navigation am gesamten Fenster
 *  (linke untere und rechte obere Ecke) */
const double		second_scene_viewport_size[4] = {0.5, 0.0, 1.0, 1.0};
/** Anteil der Darstellung des Namens des fokussierten Netzwerkelementes am
 *  gesamten Fenster (linke untere und rechte obere Ecke) */
const double		name_viewport_size[4] = {0.0, 0.0, 0.5, 0.05};

// for reshape
/** vordere Darstellungskante (Abstand von der Kamera) */
const double		front_plane = 0.1;
/** hintere Darstellungskante (Abstand von der Kamera) */
const double		back_plane = 80.0;
/** Darstellungsmatrix f&uuml;r orthogonale Projektion in der Darstellung des
 *  Namens des fokussierten Netzwerkelementes (glOrtho()) */
const double		name_viewport_matrix[6] =
					{-1.0, 1.0, -1.0, 1.0, -1.0, 1.0};

/** Farbe des Namens des fokussierten Netzwerkelementes */
const double		current_name_color[4] = {0.9, 0.9, 0.9, 1.0};
/** Position des Namens des fokussierten Netzwerkelementes in dessen
 *  Darstellungsbereich */
const double		current_name_pos[3] = {-0.95, -0.5, 0.0};
/** Gr&ouml;&szlig;e eines Buchstaben des Namens des fokussierten
 *  Netzwerkelementes */
const double		current_name_size[2] = {0.04, 0.8};
//@}

//@{ @name Statuswerte und Anfangsbelegung in main_states
/** Statuswert Fenster (teilweise) verdeckt (siehe windowStatus()) */
const unsigned int	state_window_covered = 1;
/** Statuswert Fenster ist sichtbar (siehe windowStatus()) */
const unsigned int	state_window_visible = 2;
/** Statuswert Men&uuml; wird angezeigt */
const unsigned int	state_menu_shown = 4;
/** Statuswert Netzwerk soll geladen werden */
const unsigned int	state_load_in_progress = 8;
/** Statuswert Netzwerk soll gespeichert werden */
const unsigned int	state_save_in_progress = 16;
/** Statuswert Kartenansicht ist aus */
const unsigned int	state_mapwindow_off = 32;
/** Statuswert Netzwerksimulation l&auml;uft in Endlosschleife */
const unsigned int	state_execute_net_loop = 64;
/** Statuswert Kamera soll w&auml;hrend der Simulation den Pakete folgen */
const unsigned int	state_follow_packets = 128;
/** Statuswert Level of Details ist an */
const unsigned int	state_level_of_details_on = 256;
/** Statuswert Kamera soll w&auml;hrend der Simulation nur mit der
 *  Blickrichtung den Paketen folgen, Position bleibt fest */
const unsigned int	state_follow_angle = 512;
/** Statuswert stelle zweite 3D-Weltansicht ohne Navigation dar (statt Karte)
 */
const unsigned int	state_second_scene = 1024;
/** Anfangswert der Statusvariable main_states mit allen Statuswerten */
const unsigned int	main_state_init_value = 00602;
						// 00 110 000 010b;
//@}

GlutMain::GlutMain(void) :
	main_states(main_state_init_value),
	mainwindow(-1), current_width(0), current_height(0),
	x_translation(0.0), y_translation(0.0), z_translation(0.0),
	animation_speed_factor(1.0), last_drawn(0),
	current_name_on_network("ABCD_0123456789"), display_list_name_text(0)
/** Konstruktor, Initialisierung der Variablen: main_states, mainwindow,
 *  current_width, current_height, x_translation, y_translation, z_translation,
 *  animation_speed_factor, last_drawn, current_name_on_network,
 *  display_list_name_text */
{
	// cout << "GlutMain: states: " << oct << main_states << endl;
};

GlutMain::~GlutMain(void)
/** Destruktor, l&ouml;scht die Displayliste display_list_name_text */
{
	// cout << "destruktor Glutmain end." << endl;
	if (glIsList(display_list_name_text))
		glDeleteLists(display_list_name_text, 1);
};

void GlutMain::createWindow(void)
/** erzeugt das Hauptfenster mit Titel (glutCreateWindow()) */
{
        mainwindow = glutCreateWindow("netview");
};

void GlutMain::dialog_display(void)
/** Aktualisierung des Fensterinhaltes im Dialog: ruft dessen Zeichenmethode
 *  Dialog::draw() */
{
	dialog_window.draw();
};

void GlutMain::display(void)
/** Hauptmethode zur Fensteraktualisierung:
 *  - l&auml;dt Netzwerk, falls Eingabe des Dateinamens abgeschlossen ist
	(Network::load())
 *  - einen Schritt in Endlosschleife der Netzwerksimulation ausf&uuml;hren
	(startAction() mit Action_Execute_Net), falls ausgew&auml;hlt
 *  - Neuzeichnen, wenn Fenster teilweise verdeckt ist: mit letztem
 *    Screenshot ohne Verdeckung (Capture::draw())
 *  - normales Neuzeichnen mit drawWindow() und Screenshot mit
 *    Capture::captureImage() */
{
	time_t		current_time;
	Point_3D	camera_pos;

	// load network if dialog window for file name input is closed
	if ((main_states & state_load_in_progress) && dialog_window.isHidden())
	{
		if (dialog_window.isChanged())
		{
			cout << "Call network_web.load" << endl;
			network_web.load();
		};
		main_states -= state_load_in_progress;
	};
	// window not visible -> don't need to redraw
	if (!(main_states & state_window_visible)) return;
	// if simulation loop: do one step in simulation, move cam, redraw net
	if ((image_capturer.isRunning()) ||
		(main_states & state_execute_net_loop))
	{
		startAction(Action_Execute_Net);
		mainwindow_cam.getPos(camera_pos);
		network_web.drawSetCameraPos(
			(main_states & state_level_of_details_on), camera_pos);
	};
	current_time = time(NULL);
	// window is partially covered: fast redraw by showing screenshot
	if ((main_states & (state_window_covered | state_menu_shown)) && 
		(current_time != last_drawn))
	{
		// cout << "display" << endl;
		image_capturer.draw();	// screenshot of last full draw
		glutSwapBuffers();
		glFlush();
		last_drawn = current_time;
	}
	// normal redraw: this window is in front of all others
	if ((!(main_states & state_window_covered) && 
		(main_states & state_window_visible)) || 
		(!image_capturer.isValid()))
	{
		// cout << "display" << endl;
		drawWindow();
		image_capturer.captureImage();
	};
};

void GlutMain::drawWindow(void)
/** stellt das Hauptfenster dar: inklusive Netzwerk (Network::draw()),
 *  Name (drawCurrentName()) und Kartenansicht (drawGrid() und Network::draw())
 *  bzw. zweiter 3D-Weltansicht */
{
	glEnable(GL_DEPTH_TEST);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// main world view: viewport, set camera, draw network
	scene_viewport.draw();
	mainwindow_cam.gotoPos();
	glTranslated(x_translation, y_translation, z_translation);
	network_web.draw();
	glTranslated(-x_translation, -y_translation, -z_translation);
		
	// second world view (without navigation)
	if (main_states & state_second_scene)
	{
		second_scene_viewport.draw();
		rightview_cam.gotoPos();
		glTranslated(x_translation, y_translation, z_translation);
		network_web.draw();
		glTranslated(-x_translation, -y_translation, -z_translation);
	}
	// draw map view with grid and network
	else
	{
		map_viewport.draw();
		glRotatef(90.0, 1, 0, 0);
		glTranslated(0, 0, -(map_sizes[2] + map_sizes[3]));
		drawGrid();
		network_web.draw(true);
		mainwindow_cam.draw();
		mainwindow_mouse.draw();
		glTranslated(0, 0, map_sizes[2] + map_sizes[3]);
		glRotatef(-90.0, 1, 0, 0);
	};
	// draw name of focused network object
	drawCurrentName();
	
	glViewport(0, 0, current_width, current_height);
	glutSwapBuffers();
	glFlush();
};

void GlutMain::drawCurrentName(void)
/** zeichnet den Namen des fokussierten Netzwerkelementes in der Simulation,
 *  d.h. ruft Viewport::draw() und die Displayliste auf */
{
	// cout << "drawCurrentName: " << current_name_on_network << endl;
	name_viewport.draw();
	if (glIsList(display_list_name_text))
		glCallList(display_list_name_text);
};

void GlutMain::updateDisplaylistNameText(void)
/** aktualisiert die Displayliste zum Namen des fokussierten Netzwerkelementes
 */
{
	if (glIsList(display_list_name_text))
	{
		glNewList(display_list_name_text, GL_COMPILE);
			name_output.print(current_name_on_network.c_str());
		glEndList();
	};
};

void GlutMain::drawGrid(void)
/** zeichnet das Gitternetz der Kartenansicht */
{
	int		x, z;
	double		shift;
	Point_3D	grid_color;

	grid_color.setColor(1.0, 1.0, 1.0, 1.0);
	grid_color.colorForDraw();
	glBegin(GL_LINES);
		shift = (map_sizes[3] - map_sizes[2]) / num_grid_lines;
		// cout << "shift: " << shift << endl;
		// horizontal lines and diagonal down
		for(z = 0; z <= num_grid_lines; z++)
		{
			// horizontal lines
			glVertex3d(map_sizes[0], 0.0, map_sizes[2] + shift * z);
			glVertex3d(map_sizes[1], 0.0, map_sizes[2] + shift * z);
			// diagonal lines down direction, bottom left half
			glVertex3d(map_sizes[0], 0.0, map_sizes[2] + shift * z);
			glVertex3d(map_sizes[1] - shift * z, 0.0, map_sizes[3]);
			// diagonal lines down direction, top right half
			glVertex3d(map_sizes[0] + shift * z, 0.0, map_sizes[2]);
			glVertex3d(map_sizes[1], 0.0, map_sizes[3] - shift * z);
		};
		shift = (map_sizes[1] - map_sizes[0]) / num_grid_lines;
		// vertical lines in top view (z axis, rotated by 90 degrees)
		// and diagonal lines up
		for(x = 0; x <= num_grid_lines; x++)
		{
			// vertical lines
			glVertex3d(map_sizes[0] + shift * x, 0.0, map_sizes[2]);
			glVertex3d(map_sizes[0] + shift * x, 0.0, map_sizes[3]);
			// diagonal lines up direction, top left half
			glVertex3d(map_sizes[1] - shift * x, 0.0, map_sizes[2]);
			glVertex3d(map_sizes[0], 0.0, map_sizes[3] - shift * x);
			// diagonal lines up direction, bottom right half
			glVertex3d(map_sizes[1], 0.0, map_sizes[2] + shift * x);
			glVertex3d(map_sizes[0] + shift * x, 0.0, map_sizes[3]);
		};
	glEnd();
};

void GlutMain::idle(void)
/** Funktion wird von GLUT gerufen, wenn gerade nichts zu tun ist (kein
 *  Neuzeichnen, keine Eingaben):
 *  - Verschiebung der Kamera bei der Navigation,
 *  - Neuzeichnen bei Verdeckung,
 *  - Neuzeichnen bei Videoaufnahmen und bei der Netzwerksimulation
 *    (hier mit Wartezeit)
 *  - Speichern des Screenshots bei Videoaufnahmen (Capture::captureVideo()) */
{
	bool		cam_is_moving;
	Point_3D	camera_pos;

	// move camera in navigation
	cam_is_moving = mainwindow_cam.is_moving();
	if (cam_is_moving)
	{
		startAction(Action_Cam_Step);
		glutSetWindow(mainwindow);
		mainwindow_cam.getPos(camera_pos);
		network_web.drawSetCameraPos(
			(main_states & state_level_of_details_on), camera_pos);
	};
	// redraw if partially covered
	if ((main_states & state_window_covered) && 
		(main_states & state_window_visible))
			glutPostRedisplay();
	// redraw for video or network simulation
	if (!(main_states & state_window_covered) && 
		(main_states & state_window_visible))
		if ((cam_is_moving) || (image_capturer.isRunning()) || 
			(main_states & state_execute_net_loop))
		{
			glutPostRedisplay();
			usleep(idle_time);
		};
	// save image for video
	if (image_capturer.isRunning())
	{
		image_capturer.captureVideo(glutmain_video_path); 
	};
};

void GlutMain::init(void (* menu_callback)(int entry), 
			struct callback_struct &dialog_callbacks)
/** Initialisierung:
 *  - Men&uuml;- (Menu::create() mit 'menu_callback') und Tastaturverarbeitung,
 *  - Netzwerk (Network::init()),
 *  - Gr&ouml;&szlig;en (u.a. setViewportSizes()), Farben, Positionen, 
 *  - Filmaufnahmen (Capture::init()), Maus, Lichter,
 *  - Blending, Dialogfenster (Dialog::init() mit 'dialog_callbacks') */
{
	// cout << "Init start " << endl;
	mainwindow_menu.create(menu_callback);
	mainwindow_keys.init();
	network_web.init();
	// cout << "Viewport init" << endl;
	current_width = start_window_size[0]; 
	current_height = start_window_size[1];
	setViewportSizes();
	scene_viewport.scale(current_width, current_height);
	map_viewport.scale(current_width, current_height);
	second_scene_viewport.scale(current_width, current_height);
	// cout << "Image & Mouse & Reshape & Lights & Cam " << endl;
	name_output.setColor(current_name_color[0], current_name_color[1],
				current_name_color[2], current_name_color[3]);
	name_output.setPos(current_name_pos[0], current_name_pos[1],
				current_name_pos[2]);
	name_output.setSize(current_name_size[0], current_name_size[1]);
	display_list_name_text = glGenLists(1);
	image_capturer.init();
	mainwindow_mouse.init(&mainwindow_cam, &dialog_window, &network_web,
				&map_viewport);
	glutReshapeWindow(current_width, current_height);
	mainwindow_lights.init();
	mainwindow_lights.update();
	startAction(Action_Cam_Init);
	// cout << "Blending & Dialog" << endl;
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	dialog_window.init(dialog_callbacks);
	// cout << "Init end" << endl;
};

void GlutMain::checkArgs(int argc, char *argv[])
/** liest die Kommandozeilenoptionen (Argumente) ein */
{
	int	current_opt;

	do
	{
		current_opt = getopt(argc, argv, option_string);
		if (current_opt == option_server_port)	// packet server port
			network_web.init(atoi(optarg));
		if (current_opt == option_load)		// network file name
			network_web.load(optarg);
		if (current_opt == option_speed_factor)
						// speed factor in animation
			animation_speed_factor = atof(optarg);
	} while (current_opt != -1);
};

void GlutMain::dialog_key(unsigned char k, int x, int y)
/** Verarbeitung der Tastatureingaben in Dialog: ruft Dialog::key() mit
 *  Taste 'k', die Position der Maus ('x' und 'y') ist egal, <br>
 *  - Dateinamen-Eingabe vor Laden/Speicherung des Netzwerkes
 *  - Aktualisierung der Einstellungen des Netzwerkes nach Eingabeende im
 *    Dialog
 *  - Speicherung des Netzwerkes nach Dateinameneingabe */
{
	double		last_info_posxz[2];
	Variable_Tree*	infos;

	if (dialog_window.key(k, x, y))
	{
		// save file name settings of network
		if (main_states & 
			(state_load_in_progress | state_save_in_progress))
		{
			dialog_window.getText(infos);
			network_web.setInfos(infos);
			glutPostWindowRedisplay(mainwindow);
		}
		// save settings of network element on 'last_info_posxz'
		else
		{
			dialog_window.getText(infos);
			mainwindow_mouse.getLastInfoPos(last_info_posxz);
			network_web.setInfos(last_info_posxz, infos);
			if (dialog_window.isChanged())
			{
				network_web.getInfos(last_info_posxz, infos);
				dialog_window.setText(infos);
			};
		};
		// save network
		if (main_states & state_save_in_progress)
		{
			network_web.save();
			main_states -= state_save_in_progress;
		};
	};
};

void GlutMain::key(unsigned char k, int x, int y)
/** Verarbeitung der Tastatureingaben im Hauptfenster:
 *  Key_Control::translateKeycode() mit Taste 'k',
 *  die Position der Maus ('x' und 'y') ist egal, <br>
 *  Neuzeichnen des Netzwerkes nach Ausf&uuml;hrung der Aktion hinter
 *  Taste 'k' (startAction()) */
{
	Point_3D	camera_pos;

	glutSetWindow(mainwindow);
	startAction( mainwindow_keys.translateKeycode(k) );
	mainwindow_cam.getPos(camera_pos);
	network_web.drawSetCameraPos(
		(main_states & state_level_of_details_on), camera_pos);
	if (mainwindow != -1) glutPostRedisplay();  // not on end of program
};

void GlutMain::menu_func(int entry)
/** Verarbeitung des Men&uuml;eintrages 'entry': Menu::execute() und
 *  Menu::translateEntryValue(), Ausf&uuml;hren der Aktion (startAction())
 *  dahinter und Netzwerk neu zeichnen */
{
	Point_3D	camera_pos;

	entry = mainwindow_menu.execute(entry);
	startAction( mainwindow_menu.translateEntryValue(entry) );
	mainwindow_cam.getPos(camera_pos);
	network_web.drawSetCameraPos(
		(main_states & state_level_of_details_on), camera_pos);

};

void GlutMain::mouse_func(int button, int state, int x, int y)
/** Verarbeitung der Mauseingabe mit Mouse::mouse_func(), <br>
 *  Maustaste 'button' hat Zustand 'state', Maus ist auf Position 'x','y',
 *  (relativ zum Fenster) */
{
	glutSetWindow(mainwindow);
	mainwindow_mouse.mouse_func(button, state, x, y,
				(main_states & state_level_of_details_on),
				!(main_states & state_second_scene));
};

void GlutMain::motion(int x, int y)
/** Verarbeitung der Mausverschiebung mit gedr&uuml;ckter Maustaste zur
 *  Position 'x','y' (relativ zum Fenster): Mouse::motion() */
{
	glutSetWindow(mainwindow);
	mainwindow_mouse.motion(x, y, main_states & state_level_of_details_on);
};

void GlutMain::passiveMotion(int x, int y)
/** Verarbeitung der Mausverschiebung OHNE gedr&uuml;ckte Maustaste zur
 *  Position 'x','y' (relativ zum Fenster): Mouse::passiveMotion() */
{
	mainwindow_mouse.passiveMotion(x, y);
};

void GlutMain::getMaxMinSize(int &max_size, int &min_size)
/** aus der aktuellen Fensterbreite und -h&ouml;he wird Maximum und Minimum
 *  bestimmt und zur&uuml;ckgegeben:
 *  'max_size' = max(Breite, H&ouml;he), 'min_size' = min(Breite, H&ouml;he) */
{
	if (current_width >= current_height)
	{
		max_size = current_width; min_size = current_height;
	}
	else
	{
		max_size = current_height; min_size = current_width;
	};
};

void GlutMain::setViewportSizes(void)
/** setzt die Gr&ouml;&szlig;e der einzelnen Fensterbereiche (Viewports)
 *  je nach den aktuellen Einstellungen:
 *  - Hauptansicht der 3D-Welt mit Navigation
 *  - Kartenansicht
 *  - zweite Ansicht der 3D-Welt ohne Navigation
 *  - Darstellung des Namens des fokussierten Netzwerkelementes */
{
	double	aspect, scaling[4];

	aspect = 1.0;
	// with another aspect, the scene_viewport stays as square
	// aspect = (current_height * 1.0) / current_width / 
	//		scene_viewport_size[2];
	// cout << "w h a: " << current_width << " " << current_height << " "
	//	<< aspect << endl;
	// no map window or second scene
	if (main_states & state_mapwindow_off)
	{
		scene_viewport.setPos(0.0, 0.05, 1.0, 1.0);
		map_viewport.setPos(1.0, 1.0, 1.0, 1.0);
		second_scene_viewport.setPos(1.0, 1.0, 1.0, 1.0);
		name_viewport.setPos(0.0, 0.0, 1.0, 0.05);
	}
	// normal settings
	else
	{
		scaling[0] = 1.0; scaling[1] = 1.0;
		scaling[2] = aspect; scaling[3] = 1.0;
		scene_viewport.setPos(scene_viewport_size, scaling);
		name_viewport.setPos(name_viewport_size, scaling);
		scaling[0] = aspect; scaling[2] = 1.0;
		map_viewport.setPos(map_viewport_size, scaling);
		second_scene_viewport.setPos(second_scene_viewport_size,
						scaling);
	};
};

void GlutMain::reshape(int width, int height)
/** passt die internen grafischen Einstellungen den neuen Fensterma&szlig;en
 *  an: Fensterbreite 'width' und Fensterh&ouml;he 'height'; <br>
 *  vor allem Viewport (setViewportSizes()) und Matrizen f&uuml;r Perspektiv-
 *  und Orthogonal-Projektion, aber auch Maus (Mouse::reshape()) und
 *  Videoaufnahme (Capture::setSize()) */
{
	double		angle, aspect, scaling[6];
	int		max_size, min_size;

	current_width = width;
	current_height = height;
	// cout << "Reshape" << endl;
	setViewportSizes();
	getMaxMinSize(max_size, min_size);

	// viewports and matrix for perspective projection of 3d world
	scene_viewport.scale(current_width, current_height);
	second_scene_viewport.scale(current_width, current_height);
	//angle = 60.0 * scene_viewport.getWidth() / scene_viewport.getHeight();
	angle = 60.0;
	aspect = 1.0 * scene_viewport.getWidth() / scene_viewport.getHeight();
	scene_viewport.setgluPerspective(angle, aspect,	// focus angle, aspect
						front_plane, back_plane);
	// glOrtho(-20.0, 20.0, -20.0, 20.0, -20.0, 20.0);
	second_scene_viewport.setProjectionMatrix();
	mainwindow_cam.gotoPos();
	scene_viewport.setModelMatrix();
	rightview_cam.gotoPos();
	second_scene_viewport.setModelMatrix();

	// viewport and orthogonal projection of map view
	map_viewport.scale(current_width, current_height);
	aspect = 1.0 * map_viewport.getWidth() / map_viewport.getHeight();
	scaling[0] = 1.0; scaling[1] = 1.0; scaling[2] = 1.0;
	scaling[3] = 1.0; scaling[4] = 1.0; scaling[5] = 1.0;
	if (aspect >= 1.0)
	{
		scaling[0] = aspect; scaling[1] = aspect;
	}
	else
	{
		scaling[2] = 1.0 / aspect; scaling[3] = 1.0 / aspect;
	};
	map_viewport.setglOrtho(map_sizes, scaling);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	map_viewport.setModelMatrix();

	// name viewport
	name_viewport.scale(current_width, current_height);
	scaling[0] = 1.0; scaling[1] = 1.0; scaling[2] = 1.0;
	scaling[3] = 1.0; scaling[4] = 1.0; scaling[5] = 1.0;
	name_viewport.setglOrtho(name_viewport_matrix, scaling);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	name_viewport.setModelMatrix();

	// Mouse and Capture
	mainwindow_mouse.reshape(current_width, current_height);
	image_capturer.setSize(current_width, current_height);
};

void GlutMain::dialog_specialKey(int k, int x, int y)
/** Verarbeitung der Eingaben von Sondertasten im Dialog:
 *  ruft nur Dialog::specialKey() mit Sondertaste 'k' und Mausposition 'x','y'
 *  (relativ zum Fenster) */
{
	dialog_window.specialKey(k, x, y);
};

void GlutMain::specialKey(int k, int x, int y)
/** Verarbeitung der Eingaben von Sondertasten im Hauptfenster:
 *  Auswahl der Aktion hinter Taste 'k' (Key_Control::translateKeycode()),
 *  Ausf&uuml;hrung dieser Aktion (startAction()), Neuzeichnen des Netzwerkes,
 *  <br> Mausposition 'x','y' (relativ zum Fenster) ist egal */
{
	Point_3D	camera_pos;

	// cout << "specialKey: " << k << endl;
	startAction( mainwindow_keys.translateKeycode(k) );
	mainwindow_cam.getPos(camera_pos);
	network_web.drawSetCameraPos((main_states & state_level_of_details_on),
					camera_pos);
	if (mainwindow != -1) glutPostRedisplay();  // not on end of program
};

void GlutMain::initLoadNet(void)
/** &Ouml;ffnen des Dialogfensters zur Eingabe des Dateinamens vor dem
 *  eigentlichen Laden (in display()) */
{
	Variable_Tree*	infos;

	// opening dialog window for file name input before loading
	main_states |= state_load_in_progress;
	network_web.getInfos(infos);
	dialog_window.setText(infos);
	dialog_window.redraw();
};

void GlutMain::initSaveNet(void)
/** &Ouml;ffnen des Dialogfensters zur Eingabe des Dateinamens vor dem
 *  eigentlichen Speichern (in dialog_key()) */
{
	Variable_Tree*	infos;

	// opening dialog window for file name input before saving
	main_states |= state_save_in_progress;
	network_web.getInfos(infos);
	dialog_window.setText(infos);
	dialog_window.redraw();
};

void GlutMain::executeNet(void)
/** Ausf&uuml;hren eines Schrittes der Netzwerksimulation:
 *  Network::executeNet(), danach Name des fokussierten Netzwerkelementes
 *  neuzeichnen und Kameraansicht aktualisieren (Kamerawinkel
 *  und/oder -position) */
{
	Point_3D	new_camera_pos;
	double		new_camera_y_angle;
	string		current_name;

	if ((network_web.executeNet(animation_speed_factor, new_camera_pos,
					new_camera_y_angle, current_name))
		&& (main_states & state_follow_packets))
	{
		// cout << "GlutMain: name: " << current_name << endl;
		current_name_on_network = current_name;	// save name
		updateDisplaylistNameText();	// update list
		// move/rotate the camera
		mainwindow_cam.updateCam(new_camera_pos, new_camera_y_angle,
					(main_states & state_follow_angle));
		rightview_cam.updateCam(new_camera_pos, new_camera_y_angle,
					true);
	};
	// rotation stepwise
	if (mainwindow_cam.needsTurn())
		if (main_states & state_follow_packets)
			mainwindow_cam.turnToAngleStep();
	if (rightview_cam.needsTurn()) rightview_cam.turnToAngleStep();
};

void GlutMain::showCopyright(void)
/** zeigt den Copyrighthinweis (copyright_strings) im Dialogfenster */
{
	size_t		i;
	String_Variable	*copyright_text;

	copyright_infos.clear();
	i = 0;
	while(copyright_strings[i][0] != 0)	// empty string
	{
		copyright_text = new String_Variable;
		copyright_text->setName("");
		copyright_text->setValue(copyright_strings[i]);
		copyright_text->setConstant(true);
		copyright_infos.addVar(copyright_text);
		i++;
	};
	dialog_window.setText(&copyright_infos);
	dialog_window.redraw();
};

void GlutMain::toggleState(int state)
/** Hilfsmethode zum &Auml;ndern eines Status mit dem Statuswert 'state'
 *  in main_states */
{
	if (main_states & state) main_states -= state;
	else main_states += state;
};

void GlutMain::startAction(Actions number)
/** f&uuml;hrt die Kommandos und Wertver&auml;nderungen hinter der
 *  Aktion 'number' aus, ein Teil davon wird &uuml;ber Mouse::startAction()
 *  aufgerufen; <br>
 *  Netzwerk wird am Ende neu gezeichnet */
{
	Point_3D	camera_pos;

	switch	(number)
	{
		// dummy, exit, copyright
		case Action_Dummy:	cout << "Dummy Action" << endl; break;
		case Action_Exit:	mainwindow_menu.destroy();
					dialog_window.destroy();
					glutDestroyWindow(mainwindow);
					mainwindow = -1;
					break;
		case Action_Show_Copyright:
					showCopyright();
					break;
		// translation actions
		case Action_Move_Left:	x_translation -= 0.05; break;
		case Action_Move_Right:	x_translation += 0.05; break;
		case Action_Move_Up:	y_translation += 0.05; break;
		case Action_Move_Down:	y_translation -= 0.05; break;
		case Action_Move_Front:	z_translation += 0.05; break;
		case Action_Move_Back:	z_translation -= 0.05; break;
		case Action_Move_Init:	x_translation = 0.0;
					y_translation = 0.0;
					z_translation = 0.0;
					cout << "Init" << endl;
					break;
		// set special position and direction of camera
		case Action_View_Left:	mainwindow_cam.setPos(-5.0, 0.0, 0.0);
					mainwindow_cam.setAngle(0.0, 0.0); 
					break;
		case Action_View_Right:	mainwindow_cam.setPos( 5.0, 0.0, 0.0);
					mainwindow_cam.setAngle(0.0, 180.0); 
					break;
		case Action_View_Top:	mainwindow_cam.setPos(0.0,  5.0, 0.0);
					mainwindow_cam.setAngle(-90.0, 0.0); 
					break;
		case Action_View_Bottom: mainwindow_cam.setPos(0.0, -5.0, 0.0);
					mainwindow_cam.setAngle(90.0, 0.0); 
					break;
		case Action_View_Front:	mainwindow_cam.setPos(0.0, 0.0, 5.0);
					mainwindow_cam.setAngle(0.0, 90.0); 
					break;
		case Action_View_Back:	mainwindow_cam.setPos(0.0, 0.0, -5.0);
					mainwindow_cam.setAngle(0.0, 270.0); 
					break;
		case Action_View_TLF:	mainwindow_cam.setPos(-5.0, 5.0, 5.0);
					mainwindow_cam.setAngle(315.0, 45.0); 
					break;
		case Action_View_TRF:	mainwindow_cam.setPos(5.0, 5.0, 5.0);
					mainwindow_cam.setAngle(315.0, 135.0); 
					break;
		case Action_View_BRB:	mainwindow_cam.setPos(5.0, -5.0, -5.0);
					mainwindow_cam.setAngle(45.0, 225.0); 
					break;

		// load / save of network, simulation of network
		case Action_Save_Net:	initSaveNet(); break;
		case Action_Load_Net:	initLoadNet(); break;
		case Action_Execute_Net: executeNet(); break;
		case Action_Execute_Net_Loop:
					toggleState(state_execute_net_loop); 
					break;
		case Action_FollowP_OnOff:
					toggleState(state_follow_packets);
					break;
		case Action_FollowAngle_OnOff:
					toggleState(state_follow_angle);
					break;
		// map view on/off, video start/stop
		case Action_Map_OnOff:	toggleState(state_mapwindow_off);
					startAction(Action_Mode_Cam);
					reshape(current_width, current_height);
					break;
		case Action_Video_StartStop:
					image_capturer.toggleRunning();
					break;
		// level of details on/off, second view of 3d world on/off
		case Action_LOD_OnOff:	toggleState(state_level_of_details_on);
					break;
		case Action_View_SceneRight:
					rightview_cam = mainwindow_cam;
					toggleState(state_second_scene);
					break;
		// speed of animation in network simulation
		case Action_Animation_Slower:
					animation_speed_factor -=
							animation_speed_step;
					animation_speed_factor =
						fabs(animation_speed_factor);
					break;
		case Action_Animation_Faster:
					animation_speed_factor +=
							animation_speed_step;
					animation_speed_factor =
						fabs(animation_speed_factor);
					break;
		// for anything else call Mouse
		default:		mainwindow_mouse.startAction(number);
					break;
	};
	mainwindow_cam.getPos(camera_pos);
	network_web.drawSetCameraPos(
		(main_states & state_level_of_details_on), camera_pos);
};

void GlutMain::windowStatus(int status)
/** aktualisiert die zwei Statuswerte zur Fenstersichtbarkeit in main_states,
 *  'status' ist der Fensterstatus als GLUT-Konstante */
{
	switch (status)
	{
		case GLUT_HIDDEN: 	// hidden: window minimized to icon
			if (main_states & state_window_visible) 
				main_states -= state_window_visible;
			if (main_states & state_window_covered)
				main_states -= state_window_covered;
			break;
		case GLUT_FULLY_RETAINED: // in front of all other windows
			main_states |= state_window_visible;
			if (main_states & state_window_covered)
				main_states -= state_window_covered;
			break;
		case GLUT_PARTIALLY_RETAINED:	// parts of window visible
			main_states |= state_window_visible;
			main_states |= state_window_covered;
			break;
		case GLUT_FULLY_COVERED:  // not hidden but no part visible
			if (main_states & state_window_visible)
				main_states -= state_window_visible;
			main_states |= state_window_covered;
			break;
	};
	// cout << "visible: " << window_visible << " covered: " 
	//	<< window_covered << endl;
};

void GlutMain::menuState(int state)
/** aktualisiert den Statuswert zur Men&uuml;sichtbarkeit in main_states,
 *  'status' ist der Fensterstatus als GLUT-Konstante */
{
	if (state == GLUT_MENU_IN_USE) main_states |= state_menu_shown;
	if (state == GLUT_MENU_NOT_IN_USE)
		if (main_states & state_menu_shown)
			main_states -= state_menu_shown;
	// cout << "menu shown: " << menu_shown << endl;
};

