/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <errno.h>
#include <iostream>
#include <stdio.h>
#include <string>
#include <string.h>
#include <sys/stat.h>

#include "capture.h"
#include "glut.h"

using namespace std;

/** Reihenfolge der Bildzeilen umdrehen (n&ouml;tig da glReadPixels das
 *  Fenster von unten nach oben einliest) */
const bool	capture_flip_image_lines = true;
/** Dateiendung des Bildes / der Bildfolge */
const char	capture_file_ending[] = ".pnm.gz";

Capture::Capture(void): video_is_running(false), image_is_valid(false),
			width(0), height(0), 
			frame_number(0), image(NULL)
/** Konstruktor, Werte initialsieren */
{
};

Capture::~Capture(void)
/** Destruktor, Bildspeicherbereich im RAM freigeben */
{
	if (image != NULL) delete [] image;
	image = NULL;
};

void Capture::draw(void) const
/** &Uuml;bertragen des gesicherten Abbildes (im RAM) in das aktuelle Fenster
 *  (zum schnellen Fenster-Neuzeichnen bei &Uuml;berdeckung) */
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
	glRasterPos2f(-0.999, -0.999);
	if (image) glDrawPixels(width, height, GL_RGB,
				GL_UNSIGNED_BYTE, image);
};

void Capture::init(void)
/** Werte initialisieren, Hauptspeicher belegen f&uuml;r Fensterabbild;
 *  geht nicht im Konstruktor, da die (virtuelle) Bildschirmgr&ouml;&szlig;e
 *  ermittelt wird (erst m&ouml;glich nach dem GLUT richtig initialisiert
 *  ist) */
{
	int	screen_width, screen_height;

	screen_width = glutGet(GLUT_SCREEN_WIDTH);
	screen_height = glutGet(GLUT_SCREEN_HEIGHT);
	setSize(screen_width, screen_height);
	if (image) delete [] image;
	image = new char [width * height * 3];
	if (image) memset(image, 0, width * height * 3);
	image_saver.init();
	image_saver.setImage(image, capture_flip_image_lines);
	image_saver.setSize(width, height, capture_flip_image_lines);
};

bool Capture::isRunning(void) const
/** gibt zur&uuml;ck, ob gerade eine Bildfolge aufgenommen wird,
 *  setzbar mit toggleRunning() */
{
	return video_is_running;
};

bool Capture::isValid(void) const
/** gibt Wert von image_is_valid zur&uuml;ck, also ob schon ein Abbild des
 *  Fensters eingelesen wurde, nachdem dessen Gr&ouml;&szlig;e ge&auml;ndert
 *  wurde (mit setSize() ) */
{
	return image_is_valid;
};

void Capture::setSize(unsigned short new_width, unsigned short new_height)
/** setzt die Gr&ouml;&szlig;e des einzulesenden Fensterbereiches:
 *  new_width (Fensterbreite), new_height (Fensterh&ouml;he) */
{
	if ((new_width == 0) || (new_height == 0)) return;
	if ((new_width == width) && (new_height == height)) return;
	width = new_width;
	height = new_height;
	image_saver.setSize(width, height, capture_flip_image_lines);
	cout << "Setting new size: " << width << " " << height << endl;
	frame_number = 0;	// frames with different sizes can't be 
				// in one video
	glPixelStorei(GL_PACK_ALIGNMENT, 1);
	glPixelStorei(GL_PACK_ROW_LENGTH, width);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, width);
	image_is_valid = false;
	// don't need to change the size of the image array, it always have
	// the biggest possible size,the (virtual) screen size.
};

void Capture::captureImage(void)
/** ein Abbild des Fensters erstellen, speichern mit saveImage() */
{
	GLenum	error;

	if (image)
	{
		while ((error = glGetError()) != GL_NO_ERROR)
			cout << "Errors before: " << error << endl;
		memset(image, 0, width * height * 3);
		glReadPixels(0, 0, width, height, GL_RGB,
				GL_UNSIGNED_BYTE, image);
		while ((error = glGetError()) != GL_NO_ERROR)
			cout << "Errors after: " << error << endl;
		image_is_valid = true;
	};
};

bool Capture::saveImage(const char image_name[])
/** Abbild des Fensters (im RAM) in die Datei "image_name" schreiben, <br>
 *  R&uuml;ckgabe "true" wenn fehlerfrei, sonst (bei Fehler) "false" */
{
	bool	error;

	cout << "Saving... frame: " << frame_number << " ";
	cout << "width: " << width << " height: " << height << endl;
	error = (image_saver.writePNM_GZ(image_name) != 0);
	image_saver.init();
	image_saver.setImage(image, capture_flip_image_lines);
	image_saver.setSize(width, height, capture_flip_image_lines);
	return error;
};

bool Capture::captureVideo(const char path_name[])
/** ein Schritt beim Speichern einer Bildfolge ins Verzeichnis "path_name",
 *  ist f&uuml;r jeden Frame (Einzelbild) aufzurufen;
 *  Dateiname mit Framenummer wird generiert,
 *  Verzeichnis wird erstellt (mkdir); <br>
 *  R&uuml;ckgabe "true" wenn fehlerfrei, sonst (bei Fehler) "false" */
{
	char	frame_nr_text[8];
	string	frame_filename;
	
	if (frame_number == 0)
	{
		errno = 0;
		mkdir(path_name, 0700);
		if (errno != EEXIST) return false;
	};
	if (video_is_running)
	{
		snprintf(frame_nr_text, sizeof(frame_nr_text), "%06d", 
				frame_number);
		frame_filename = path_name;
		frame_filename += "video_";
		frame_filename += frame_nr_text;
		frame_filename += capture_file_ending;
		frame_number++;
		return saveImage(frame_filename.data());
	}
	else return true;
};

bool Capture::toggleRunning(void)
/** zum Starten/Beenden der Bildfolge, schaltet Status isRunning() um und
 *  setzt Framenummer (Nummer des Einzelbildes) auf 0 */
{
	video_is_running = !video_is_running;	// toggle
	frame_number = 0;
	return !video_is_running;		// last state
};

