/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <iostream>
#include <GL/gl.h>

#include "cuboid.h"

using namespace std;

Cuboid::Cuboid(void) : width(0.0), height(0.0), depth(0.0)
/**
 * Konstruktor, nur Initialisierung
 */
{
};

Cuboid::~Cuboid(void)
/**
 * Destruktor, ist leer
 */
{
};

double Cuboid::getWidth(void) const
/**
 * Gibt die Breite des Quaders zur&uuml;ck
 */
{
	return width;
};

double Cuboid::getHeight(void) const
/**
 * Gibt die H&ouml;he des Quaders zur&uuml;ck
 */
{
	return height;
};

double Cuboid::getDepth(void) const
/**
 * Gibt die Tiefe des Quaders zur&uuml;ck
 */
{
	return depth;
};

void Cuboid::updatePos(void)
/**
 * Berechnet die Positionen der Seitenfl&auml;chen neu,
 * anhand von width, height, depth */ 
{
	// bottom side, rotated by -90 degrees x-axis
	// untere Seite
	sides[0].setPos(- width / 2.0, - depth / 2.0,
			+ width / 2.0, + depth / 2.0);	
	// front side / Vorderseite
	sides[1].setPos(- width / 2.0, - height / 2.0,
			+ width / 2.0, + height / 2.0);
	// left side, rotated by -90 degrees y-axis
	// linke Seitenflche
	sides[2].setPos(- depth / 2.0, - height / 2.0,
			+ depth / 2.0, + height / 2.0);
	// back side, rotated by 180 degrees y-axis
	// Rckseite
	sides[3].setPos(- width / 2.0, - height / 2.0,
			+ width / 2.0, + height / 2.0);
	// right side, rotated by 90 degrees y-axis
	// rechte Seitenflche
	sides[4].setPos(- depth / 2.0, - height / 2.0,
			+ depth / 2.0, + height / 2.0);
	// top side, rotated by 90 degrees x-axis
	// Oberseite
	sides[5].setPos(- width / 2.0, - depth / 2.0,
			+ width / 2.0, + depth / 2.0);	
};

void Cuboid::getPos(double &current_centerx, double &current_centery,
			double &current_centerz) const
/** gibt die Position des Quaders zur&uuml;ck (aktueller x-, y- und z-Wert) */
{
	current_centerx = pos.getX();
	current_centery = pos.getY();
	current_centerz = pos.getZ();
};

void Cuboid::setPos(double new_centerx, double new_centery, 
			double new_centerz)
/**
 * Mittelpunkt des Quaders setzen (x, y, z) */
{
	pos.setPos(new_centerx, new_centery, new_centerz);
};

void Cuboid::setSize(double new_width, double new_height, double new_depth)
/**
 * Ausma&szlig;e des Quaders ver&auml;ndern (L&auml;nge, Breite, H&ouml;he)
 */
{
	width = new_width; height = new_height; depth = new_depth;
	updatePos();
};

void Cuboid::setColor(double new_red, double new_green, double new_blue,
			double new_alpha)
/**
 * Farbe des Quaders ver&auml;ndern (rot, gr&uuml;n, blau, alpha)
 * mit Wertebereich 0.0 - 1.0 */
{
	int i;

	pos.setColor(new_red, new_green, new_blue, new_alpha);	
	for(i = 0; i < 6; i++) sides[i].setColor(new_red, new_green, 
						 new_blue, new_alpha);
};

void Cuboid::draw(void) const
/**
 * Zeichnet den Quader ins OpenGL-Fenster
 */
{
	// cout << "Cuboid::draw start" << endl;
	pos.translate(1);

	glRotatef(90, 1, 0, 0);	glTranslated(0, 0, height / 2.0);
	sides[0].draw();	// bottom / unten
	glTranslated(0, 0, -height / 2.0); glRotatef(-90, 1, 0, 0);

	glTranslated(0, 0, -depth / 2.0);
	sides[1].draw();	// front / vorn
	glTranslated(0, 0, depth / 2.0);

	glRotatef(90, 0, 1, 0);	glTranslated(0, 0, width / 2.0);
	sides[2].draw();	// left / links
	glTranslated(0, 0, -width / 2.0); glRotatef(-90, 0, 1, 0);

	glRotatef(180, 0, 1, 0); glTranslated(0, 0, -depth / 2.0);
	sides[3].draw();	// back / hinten
	glTranslated(0, 0, depth / 2.0); glRotatef(-180, 0, 1, 0);

	glRotatef(-90, 0, 1, 0); glTranslated(0, 0, width / 2.0);
	sides[4].draw();	// right / rechts
	glTranslated(0, 0, -width / 2.0); glRotatef(90, 0, 1, 0);

	glRotatef(-90, 1, 0, 0); glTranslated(0, 0, height / 2.0);
	sides[5].draw();	// top / oben
	glTranslated(0, 0, -height / 2.0); glRotatef(90, 1, 0, 0);

	pos.translate(-1);
	// cout << "Cuboid::draw end" << endl;
};

