/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <iostream>

#include "cylinder.h"

using namespace std;

/** Anzahl der Segmente der Kreisgrundfl&auml;che 
 *  (Anzahl der Ecken des Vielecks) */
const unsigned int cylinder_slices = 20;
/** Anzahl der (rechteckigen) Segmente entlang der Zylinderh&ouml;he
 *  (senkrecht stehend) */
const unsigned int cylinder_stacks = 5;
/** Anzahl der Unterteilungen der Grund/Deckfl&auml;chen (Kreisfl&auml;chen),
 *  also die Anzahl der konzentrischen Ringe um den Mittelpunkt */
const unsigned int cylinder_disk_loops = 5;

Cylinder::Cylinder(void):
		width(0.0), height(0.0), depth(0.0), top_radius(0.0)
/** Konstruktor, Initialisierung, Speicherbereiche (Quadrics) belegen */
{
	quadric = gluNewQuadric();
	top = gluNewQuadric();
	bottom = gluNewQuadric();
};

Cylinder::~Cylinder(void)
/** Destruktor, Speicherbereiche freigeben */
{
	// cout << "destructor Cylinder." << endl;
	gluDeleteQuadric(quadric);
	gluDeleteQuadric(top);
	gluDeleteQuadric(bottom);
};

void Cylinder::setPos(double new_centerx, double new_centery,
                        double new_centerz)
/**
 * Mittelpunkt des Zylinders setzen (x, y, z)
 */
{
        pos.setPos(new_centerx, new_centery, new_centerz);
};

void Cylinder::setSize(double new_width, double new_height, double new_depth)
/**
 * Ausma&szlig;e des Zylinders &auml;ndern (L&auml;nge, Breite, H&ouml;he)
 */
{
        width = new_width; height = new_height; depth = new_depth;
	setTopRadius(width / 2.0);
};

void Cylinder::setTopRadius(double new_top_radius)
/**
 * Radius des oberen Endes am Cylinder, muss <= width / 2.0 sein
 */
{
	if (new_top_radius < width / 2.0) top_radius = new_top_radius;
	else top_radius = width / 2.0;
};                                                                                
void Cylinder::setColor(double new_red, double new_green, double new_blue,
                        double new_alpha)
/**
 *  Farbe des Zylinders ver&auml;ndern (rot, gr&uuml;n, blau, alpha) 
 *  mit Wertebereich 0.0 - 1.0
 */
{
        pos.setColor(new_red, new_green, new_blue, new_alpha);
};

void Cylinder::draw(bool with_top) const
/**
 * Zeichnet den Zylinder ins OpenGL-Fenster
 */
{
	// cout << "Cylinder draw" << quadric << " " << top << endl;
	pos.translate(1);
	pos.colorForDraw();
	glScaled(1.0, 1.0, depth / width);
	glRotatef(270.0, 1.0, 0.0, 0.0);

	glTranslated(0.0, 0.0, -height / 2.0);
	gluCylinder(quadric, top_radius, width / 2.0, height,
			cylinder_slices, cylinder_stacks);
	glTranslated(0.0, 0.0, height);
	gluDisk(bottom, 0.0, width / 2.0, cylinder_slices,
			cylinder_disk_loops);
	glTranslated(0.0, 0.0, -height);
	if (with_top) gluDisk(top, 0.0, top_radius, cylinder_slices,
				cylinder_disk_loops);
	glTranslated(0.0, 0.0, height / 2.0);

	glRotatef(-270.0, 1.0, 0.0, 0.0);
	glScaled(1.0, 1.0, width / depth);
	pos.translate(-1);
};

