/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <iostream>

#include "lights.h"

/** Grundhelligkeit der Szene (Farbwert RGBA, je 0.0 - 1.0) */
const GLfloat	init_light_amb[] = {0.4, 0.4, 0.4, 1.0};
/** Position des 0. Lichtes (Punkt-Licht) */
const GLfloat	init_light0_pos[] = {0.0, 1.0, 15.0, 1.0};
/** Lichtfarbe des 0. Lichtes (RGBA je 0.0 - 1.0) */
const GLfloat	init_light0_amb[] = {0.4, 0.4, 0.4, 1.0};
/** Lichtfarbe des diffusen Anteils des 0. Lichtes (RGBA je 0.0 - 1.0) */
const GLfloat	init_light0_diff[] = {0.4, 0.4, 0.4, 1.0};
/** Lichtfarbe des spekul&auml;ren (spiegelnden) Anteils des 0. Lichtes
 *  (RGBA je 0.0 - 1.0) */
const GLfloat	init_light0_spec[] = {0.6, 0.6, 0.6, 1.0};
/** Richtung des 0. Lichtes (x, y, z) */
const GLfloat	init_light0_dir[] = {0.0, -0.1, -1.0};
/** Position des 1. Lichtes (Punkt-Licht) */
const GLfloat	init_light1_pos[] = {0.0, 15.0, 0.0, 1.0};
/** Lichtfarbe des 1. Lichtes (RGBA je 0.0 - 1.0) */
const GLfloat	init_light1_amb[] = {0.4, 0.4, 0.4, 1.0};
/** Lichtfarbe des diffusen Anteils des 1. Lichtes (RGBA je 0.0 - 1.0) */
const GLfloat	init_light1_diff[] = {0.4, 0.4, 0.4, 1.0};
/** Lichtfarbe des spekul&auml;ren (spiegelnden) Anteils des 1. Lichtes
 *  (RGBA je 0.0 - 1.0) */
const GLfloat	init_light1_spec[] = {0.6, 0.6, 0.6, 1.0};
/** Richtung des 1. Lichtes (x, y, z) */
const GLfloat	init_light1_dir[] = {0.0, -1.0, 0.0};


Lights::Lights()
/** Konstruktor, ist leer, g&uuml;nstigerweise ist init() und setValues()
 *  aufzurufen */
{
};

Lights::~Lights()
/** Destruktor, ist leer */
{
};

void Lights::init(void)
/** Initialisierung der Werte (Grundhelligkeit und zwei Lichtquellen) aus den
 *  Konstanten */
{
	light_amb[0] = init_light_amb[0];
	light_amb[1] = init_light_amb[1];
	light_amb[2] = init_light_amb[2];
	light_amb[3] = init_light_amb[3];
	setValues(0, GL_POSITION, init_light0_pos);
	setValues(0, GL_AMBIENT, init_light0_amb);
	setValues(0, GL_DIFFUSE, init_light0_diff);
	setValues(0, GL_SPECULAR, init_light0_spec);
	setValues(0, GL_SPOT_DIRECTION, init_light0_dir);
	setValues(1, GL_POSITION, init_light1_pos);
	setValues(1, GL_AMBIENT, init_light1_amb);
	setValues(1, GL_DIFFUSE, init_light1_diff);
	setValues(1, GL_SPECULAR, init_light1_spec);
	setValues(1, GL_SPOT_DIRECTION, init_light1_dir);
};

bool Lights::setValues(size_t light_number, GLenum pname,
			const GLfloat new_values[])
/** setzen der Werte "new_values" in der Einstellung "pname" f&uuml;r 
 *  die Lichtquelle "light_number", entspricht glLightfv 
 *  (beachte: OpenGL hat nur eine begrenzte Anzahl von Lichtquellen) */
{
	if (new_values == NULL) return false;
	if (light.size() < (light_number + 1)) light.resize(light_number + 1);
	if (light.size() < (light_number + 1)) 
		return false;	// error, could not resize
	// cout << "size: " << sizeof(light[light_number].position) << " ";
	// cout << "GLfloat: " << sizeof(GLfloat) << endl;
	if (pname == GL_POSITION)
		memcpy(light[light_number].position, new_values,
			sizeof(light[light_number].position));
	if (pname == GL_AMBIENT)
		memcpy(light[light_number].ambient, new_values,
			sizeof(light[light_number].ambient));
	if (pname == GL_DIFFUSE)
		memcpy(light[light_number].diffuse, new_values,
			sizeof(light[light_number].diffuse));
	if (pname == GL_SPECULAR)
		memcpy(light[light_number].specular, new_values,
			sizeof(light[light_number].specular));
	if (pname == GL_SPOT_DIRECTION)
		memcpy(light[light_number].direction, new_values,
			sizeof(light[light_number].direction));
	return true;
};

void Lights::update(void)
/** &Uuml;bernahme der eingestellten Werte in die OpenGL-Darstellung
 *  ("Lichter einschalten") */
{
	size_t	i;
	GLenum	light_number;

	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light_amb);
	glEnable(GL_LIGHTING);

	for(i = 0; i < light.size(); i++)
	{
		switch (i)
		{
			case 0:		light_number = GL_LIGHT0;
			case 1:		light_number = GL_LIGHT1;
			case 2: 	light_number = GL_LIGHT2;
			case 3: 	light_number = GL_LIGHT3;
			case 4: 	light_number = GL_LIGHT4;
			case 5: 	light_number = GL_LIGHT5;
			case 6: 	light_number = GL_LIGHT6;
			case 7: 	light_number = GL_LIGHT7;
			default:	light_number = GL_LIGHT1;
		};
		glEnable(light_number);
		glLightfv(light_number, GL_POSITION, light[i].position);
		glLightfv(light_number, GL_AMBIENT, light[i].ambient);
		glLightfv(light_number, GL_DIFFUSE, light[i].diffuse);
		glLightfv(light_number, GL_SPECULAR, light[i].specular);
		glLightfv(light_number, GL_SPOT_DIRECTION, light[i].direction);
	};
};

