/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <iostream>

#include "png_image_writer.h"

using namespace std;

// taken from "man libpng"

PNG_Image_Writer::PNG_Image_Writer(void) :
	image_data(NULL), image_width(0), image_height(0),
	image_bit_depth(png_image_writer_bit_depth),
	image_color_type(png_image_writer_color_type),
	png_write_pointer(NULL), png_info_pointer(NULL), row_pointers(NULL)
/** Konstruktor, Werte initialisieren */
{
	// cout << "PNG_Image_Writer: Constructor" << endl;
};

PNG_Image_Writer::~PNG_Image_Writer(void)
/** Destruktor, ruft clear() */
{
	// cout << "PNG_Image_Writer: Destructor" << endl;
	clear();
};

void PNG_Image_Writer::clear(void)
/** Freigabe der Datenstrukturen (von der Dateispeicherung) und r&uuml;cksetzen
 *  auf Anfangswerte */
{
	// cout << "PNG_Image_Writer::clear" << endl;
	image_data = NULL;
	image_width = 0;
	image_height = 0;
	image_bit_depth = png_image_writer_bit_depth;
	image_color_type = png_image_writer_color_type;

	// cout << "PNG_Image_Writer: calling png_destroy_write_struct"
	//	<< endl;
	if (png_write_pointer)
		png_destroy_write_struct(&png_write_pointer,
						&png_info_pointer);
	else	png_destroy_info_struct(png_write_pointer,
						&png_info_pointer);
	png_write_pointer = NULL;
	png_info_pointer = NULL;
	// cout << "PNG_Image_Writer: deleting row_pointers" << endl;
	if (row_pointers) delete [] row_pointers;
	row_pointers = NULL;
};

int PNG_Image_Writer::init(int new_bit_depth, int new_color_type,
				int image_compression_level)
/** Anlegen der Datenstrukturen zum Speichern und &Uuml;bernahme der
 *  Einstellungen zum Bild (ausser denen in setImage() und setSize() ), <br>
 *  vor jedem Speichern eines Bildes aufzurufen, ruft selbst auch clear(),
 * <br> new_bit_depth: Farbtiefe je Farbkanal, new_color_type: Farbtyp,
 *      image_compression_level: Kompressionsrate, 
 * <br> m&ouml;gliche Werte siehe libpng (3); Vorgabewerte sollten 
 *      &uuml;bernommen werden, also Aufruf ohne Parameter;
 * <br> R&uuml;ckgabe von "-1" bei Fehlern */
{
	int	error;

	// cout << "PNG_Image_Writer::init" << endl;
	error = 0;
	clear();
	image_bit_depth = new_bit_depth;
	image_color_type = new_color_type;

	png_write_pointer = png_create_write_struct(PNG_LIBPNG_VER_STRING,
							NULL, NULL, NULL);
	if (png_write_pointer == NULL) error = -1;
	png_info_pointer = png_create_info_struct(png_write_pointer);
	if (png_info_pointer == NULL) error = -1;
	if (png_write_pointer)
		png_set_compression_level(png_write_pointer,
						image_compression_level);	
	return error;
};

int PNG_Image_Writer::setImage(char* new_data, bool flip_lines)
/** &Uuml;bergabe des Zeigers auf die Bilddaten (deren Organisation
 *  entsprechend bit_depth und color_type, zur Zeit nur RGB mit 8 Bit je Kanal)
 *  mit Boolean-Wert ob die Reihenfolge der Bildzeilen umzudrehen ist
 *  ("flip_lines"), ruft intern setSize() sofern Gr&ouml;&szlig;e bekannt,
 *  <br> R&uuml;ckgabe von "-1" bei Fehlern */
{

	int	error;

	// cout << "PNG_Image_Writer::setImage" << endl;
	error = 0;
	image_data = new_data;
	if ((image_width > 0) && (image_height > 0) && (image_data))
		error = setSize(image_width, image_height, flip_lines);
	return error;
};

int PNG_Image_Writer::setSize(size_t new_width, size_t new_height,
				bool flip_lines)
/** Gr&ouml;&szlig;e des Bildes setzen: Breite "new_width",
 *  H&ouml;he "new_height" und Boolean-Wert (Umdrehen der Reihenfolge der
 *  Bildzeilen),
 *  <br> R&uuml;ckgabe von "-1" bei Fehlern */
{
	int	error;
	size_t	y;

	// cout << "PNG_Image_Writer::setSize" << endl;
	error = 0;
	image_width = new_width;
	image_height = new_height;

	if ((png_write_pointer) && (png_info_pointer))
	{
		png_set_IHDR(png_write_pointer, png_info_pointer, image_width,
				image_height, image_bit_depth,
				image_color_type, PNG_INTERLACE_NONE,
				PNG_COMPRESSION_TYPE_DEFAULT,
				PNG_FILTER_TYPE_DEFAULT);
	}
	else error = -1;
	if (row_pointers != NULL) delete [] row_pointers;
	if (!error) row_pointers = new png_byte* [image_height];
	if ((row_pointers) && (image_data))
	{
		if (!flip_lines)
			for(y = 0; y < image_height; y++)
			{
				row_pointers[y] = reinterpret_cast<png_byte*>
					(&(image_data[y * image_width * 3]));
			}
		else
			for(y = 0; y < image_height; y++)
			{
				row_pointers[y] = reinterpret_cast<png_byte*>
					(&(image_data[(image_height - y - 1)
					* image_width * 3]));
			};
		if ((png_write_pointer) && (png_info_pointer))
		{
			cout << "PNG_Image_Writer: png_set_rows" << endl;
			png_set_rows(png_write_pointer, png_info_pointer,
					row_pointers);
		};
	}
	else error = -1;

	return error;
};

int PNG_Image_Writer::writePNG(const char* const filename) const
/** eigentliche Bildabspeicherung in die Datei "filename",
 *  <br> R&uuml;ckgabe von "-1" bei Fehlern */
{
	int	error;
	FILE*	image_file;

	// cout << "PNG_Image_Writer::writePNG" << endl;
	error = 0;
	if (filename == NULL) error = -1;
	image_file = NULL;
	if (!error)
	{
		// cout << "PNG_Image_Writer: open file" << endl;
		image_file = fopen(filename, "wb");
		if (image_file == 0) error = -1;
	};
	if ((png_write_pointer == NULL) || (png_info_pointer == NULL))
		error = -1;
	if (!error)
		if (setjmp(png_jmpbuf(png_write_pointer))) error = -1;
	if (!error)
	{
		// cout << "PNG_Image_Writer: Init IO, write image" << endl;
		png_init_io(png_write_pointer, image_file);
		png_write_png(png_write_pointer, png_info_pointer,
				PNG_TRANSFORM_IDENTITY, NULL);
		png_write_end(png_write_pointer, png_info_pointer);
	};

	if (image_file)
	{
		if (fclose(image_file) != 0) error = -1;
		if (error) cout << "PNG_Image_Writer: error closing file "
				<< filename << endl;
	};
	return error;
};

