/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <iostream>
#include <stdio.h>

#include "pnm_image_writer.h"

using namespace std;

PNM_Image_Writer::PNM_Image_Writer(void):
	flip_image_lines(false), image_data(NULL), 
	image_compression_level(pnm_image_writer_compression_level),
	image_width(0), image_height(0)
/** Konstruktor, Werte initialisieren */
{
};

PNM_Image_Writer::~PNM_Image_Writer(void)
/** Destruktor, ruft clear() */
{
	clear();
};

void PNM_Image_Writer::clear(void)
/** Werte zur&uuml;cksetzen */
{
	image_data = NULL;
	image_compression_level = pnm_image_writer_compression_level;
	image_width = 0;
	image_height = 0;
};

int PNM_Image_Writer::init(int new_compression_level)
/** ruft clear() und &uuml;bernimmt Kompressionsrate (new_compression_level)
 *  m&ouml;gliche Werte siehe zlib (gzsetparams), <br>
 *  zur Zeit keine Fehler m&ouml;glich, also R&uuml;ckgabe immer "0" */
{
	clear();
	image_compression_level = pnm_image_writer_compression_level;
	return 0;
};

int PNM_Image_Writer::setImage(char* new_data, bool flip_lines)
/** &Uuml;bergabe des Zeigers auf die Bilddaten (RGB 8 Bit je Farbkanal)
 *  und Boolean-Wert ob Bildzeilen umzudrehen sind,
 *  ruft intern setSize() sofern Bildgr&ouml;&szlig;e bekannt,
 *  <br> R&uuml;ckgabe von "0" bzw. Fehlerwert von setSize()
 *  (zur Zeit auch "0") */
{
	int	error;

	error = 0;
	flip_image_lines = flip_lines;
	image_data = new_data;
	if ((image_width > 0) && (image_height > 0) && (image_data))
		error = setSize(image_width, image_height, flip_lines);
	return error;
};

int PNM_Image_Writer::setSize(size_t new_width, size_t new_height,
				bool flip_lines)
/** Gr&ouml;&szlig;e des Bildes setzen: Breite "new_width",
 *  H&ouml;he "new_height" und Boolean-Wert (Umdrehen der Reihenfolge der
 *  Bildzeilen),
 *  <br> R&uuml;ckgabe von "0" (zur Zeit keine Fehler m&ouml;glich) */
{
	int	error;

	error = 0;
	flip_image_lines = flip_lines;
	image_width = new_width;
	image_height = new_height;
	return error;
};

int PNM_Image_Writer::writePNM(const char* const filename) const
/** eigentliches Schreiben des Bildes ohne Kompression (!) in Datei "filename",
 *  <br> R&uuml;ckgabe von "-1" bei Fehlern */
{
	int	error;
	size_t	y;
	FILE    *image_file;
                      
	error = 0;                                                  
	image_file = fopen(filename, "wb");
	if (image_file)
	{
		fprintf(image_file, "P6\n%d %d %d\n", image_width,
			image_height, 255);
		if (!flip_image_lines)
		{
			for(y = 0; y < image_height; y++)
				fwrite(image_data + image_width * 3 * y,
					image_width * 3, 1, image_file);

		}
		else
		{
			for(y = image_height - 1; y < image_height; y--)
				fwrite(image_data + image_width * 3 * y,
					image_width * 3, 1, image_file);
		};
		if (fclose(image_file)) error = -1;
        }
	else error = -1;
	return error;

};

// taken from /usr/share/doc/zlib-devel-1.2.0.7/example.c

int PNM_Image_Writer::writePNM_GZ(const char* const filename) const
/** eigentliches Schreiben des Bildes mit Kompression (!) in Datei "filename",
 *  passiert entsprechend angegebener Kompressionsrate mit Hilfe von zlib;
 *  <br> R&uuml;ckgabe von "-1" bei Fehlern */
{
	int	error;
	size_t	y;
	gzFile  image_file;

	error = 0;
	image_file = gzopen(filename, "wb");
	gzsetparams(image_file, image_compression_level, Z_DEFAULT_STRATEGY);
	if (image_file)
	{
		gzprintf(image_file, "P6\n%d %d %d\n", image_width,
				image_height, 255);
		if (!flip_image_lines)
		{
			for(y = 0; y < image_height; y++)
				gzwrite(image_file,
					image_data + image_width * 3 * y,
					image_width * 3);

		}
		else
		{
			for(y = image_height - 1; y < image_height; y--)
				gzwrite(image_file,
					image_data + image_width * 3 * y,
					image_width * 3);
		};
		if (gzclose(image_file)) 
		{
			error = -1;
			cout << "PNM_Image_Writer: " << "Error saving: "
				<< filename << endl;
		};
		
        }
	else error = -1;	
	return error;
};

