/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <iostream>
#include <math.h>
#include <string.h>
#include <GL/gl.h>

#include "constants.h"
#include "rectangle.h"

using namespace std;

double Rectangle::getWidth(void)
/** gibt die Breite des Rechtecks zur&uuml;ck */
{
   return fabs(right - left);
};

double Rectangle::getHeight(void)
/** gibt die H&ouml;he des Rechtecks zur&uuml;ck */
{
   return fabs(bottom - top);
};

void Rectangle::setPos(double new_left, double new_top, 
			    double new_right, double new_bottom)
/** neue Eckpunkte fr das Rechteck setzen (links, oben, rechts, unten) */
{
   left  = new_left;  top    = new_top;
   right = new_right; bottom = new_bottom;
};

void Rectangle::setColor(double new_red,  double new_green, 
			      double new_blue, double new_alpha)
/** neue Farbwerte (RGBA) setzen, Wertebereich 0.0 - 1.0, 
 * alpha-Wert funktioniert nur mit Blending */
{
   red  = new_red;  green = new_green;
   blue = new_blue; alpha = new_alpha;
};

bool Rectangle::isInRect(double x, double y) const
/** R&uuml;ckgabe 'true', falls Punkt innerhalb des Rechteckes liegt, sonst
 *  'false',<br> Rand wird als 'false' gewertet */
{
  return ((x > left) && (x < right) && 
 	  (y > top)  && (y < bottom)); 
};

void Rectangle::draw(void) const
/** zeichnet das Rechteck */
{
	GLfloat material[] = {red, green, blue, alpha};
	
	// cout << red << " " << green << " "<< blue << " " 
	//     << alpha << endl;
	// Farbe setzen
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material);
	memcpy(material, material_specular_std, sizeof(material));
	material[3] = alpha;
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, material_shininess_std);
	glBegin(GL_QUADS);
  	  glColor4d(red, green, blue, alpha);
	  // Eckpunkte des Rechtecks zeichnen, OpenGL verbindet sie
	  // Texturkoordinaten werden mit gesetzt 
	  glTexCoord2f(0.0, 0.0);
	  // cout << "v1" << endl; 
	  glVertex3d(left,  top,    0);
	  glTexCoord2f(0.0, 1.0);
	  // cout << "v2" << endl; 
	  glVertex3d(left,  bottom, 0);
	  glTexCoord2f(1.0, 1.0);
	  glVertex3d(right, bottom, 0);
	  glTexCoord2f(1.0, 0.0);
	  glVertex3d(right, top,    0);
	glEnd();
};

