/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <iostream>
#include <string.h>

#include "stroke_font.h"

using namespace std;

const double	stroke_font_character_width = 8.0;
const double	stroke_font_character_height = 10.0;

//@{ @name Buchstaben A - Z, &Auml;
static CharPoint Adata[] = {
	{ 0, 0, PT}, {0, 6, PT}, {1.5, 10, PT}, {3.5, 10, PT}, {5, 6, PT}, 
	{5, 0, STROKE}, {0, 5, PT}, {5, 5, END}
};

static CharPoint Bdata[] = {
    {1, 5, PT}, {4, 5, PT}, {5, 4, PT}, {5, 1, PT}, {4, 0, PT},
    {0, 0, PT}, {0, 10, PT}, {4, 10, PT}, {5, 9, PT},
    {5, 6, PT}, {4, 5, END}
};

static CharPoint Cdata[] = {
	{5, 0, PT}, {3, 0, PT},
	{0, 3, PT}, {0, 7, PT}, {3, 10, PT}, {5, 10, END}
};

static CharPoint Ddata[] = {
	{0, 0, PT}, {0, 10, PT}, {4, 10, PT}, {5, 9, PT}, {5, 1, PT},
	{4, 0, PT}, {0, 0, END}
};

static CharPoint Edata[] = {
	{5, 0, PT}, {0, 0, PT}, {0, 10, PT}, {5, 10, STROKE},
	{0, 5, PT}, {4, 5, END}
};

static CharPoint Fdata[] = {
	{0, 0, PT}, {0, 10, PT}, {5, 10, STROKE},
	{0, 5, PT}, {4, 5, END}
};

static CharPoint Gdata[] = {
	{2, 5, PT}, {5, 5, PT}, {5, 1, PT}, {4, 0, PT}, {1, 0, PT},
	{0, 1, PT}, {0, 9, PT}, {1, 10, PT}, {5, 10, END}
};

static CharPoint Hdata[] = {
	{0, 0, PT}, {0, 10, STROKE}, {5, 0, PT}, {5, 10, STROKE},
	{0, 5, PT}, {5, 5, END}
};

static CharPoint Idata[] = {
	{2.5, 0, PT}, {2.5, 10, STROKE},
	{0, 0, PT}, {5, 0, STROKE},
	{0, 10, PT}, {5, 10, END}
};

static CharPoint Jdata[] = {
	{0, 3, PT}, {1, 0, PT}, {2, 0, PT}, {3, 3, PT},
	{3, 3, PT}, {3, 10, END},
};

static CharPoint Kdata[] = {
	{0, 0, PT}, {0, 10, STROKE},
	{5, 0, PT}, {0, 5, STROKE},
	{5, 10, PT}, {0, 5, END},
};

static CharPoint Ldata[] = {
	{5, 0, PT}, {0, 0, PT}, {0, 10, END}
};

static CharPoint Mdata[] = {
	{0, 0, PT}, {0, 10, PT}, {2.5, 5, PT}, {5, 10, PT}, {5, 0, END}
};

static CharPoint Ndata[] = {
	{0, 0, PT}, {0, 10, PT}, {5, 0, PT}, {5, 10, END}
};

static CharPoint Odata[] = {
	{5, 1.5, PT}, {3.5, 0, PT}, {1.5, 0, PT},
	{0, 1.5, PT}, {0, 8.5, PT}, {1.5, 10, PT}, {3.5, 10, PT}, {5, 8.5, PT},
	{5, 1.5, END}
};

static CharPoint Pdata[] = {
	{0, 0, PT}, {0, 10, PT},  {4, 10, PT}, {5, 9, PT}, {5, 6, PT},
	{4, 5, PT}, {0, 5, END}
};

static CharPoint Qdata[] = {
	{5, 1.5, PT}, {3.5, 0, PT}, {1.5, 0, PT},
	{0, 1.5, PT}, {0, 8.5, PT}, {1.5, 10, PT}, {3.5, 10, PT}, {5, 8.5, PT},
	{5, 1.5, STROKE}, {2.5, 4, PT}, {5, 0, END}
};

static CharPoint Rdata[] = {
	{0, 0, PT}, {0, 10, PT},  {4, 10, PT}, {5, 9, PT}, {5, 6, PT},
	{4, 5, PT}, {0, 5, STROKE}, {3, 5, PT}, {5, 0, END}
};

static CharPoint Sdata[] = {
	{0, 1, PT}, {1, 0, PT}, {4, 0, PT}, {5, 1, PT}, {5, 4, PT},
	{4, 5, PT}, {1, 5, PT}, {0, 6, PT}, {0, 9, PT}, {1, 10, PT},
	{4, 10, PT}, {5, 9, END}
};

static CharPoint Tdata[] = {
	{0, 10, PT}, {5, 10, STROKE},  {2.5, 10, PT}, {2.5, 0, END}
};

static CharPoint Udata[] = {
	{5, 10, PT}, {5, 2, PT}, {3, 0, PT}, {2, 0, PT},
	{0, 2, PT}, {0, 10, END}
};

static CharPoint Vdata[] = {
	{0, 10, PT}, {2.5, 0, PT}, {5, 10, END}
};

static CharPoint Wdata[] = {
	{0, 10, PT}, {0, 0, PT}, {2.5, 5, PT}, {5, 0, PT},
	{5, 10, END}
};

static CharPoint Xdata[] = {
	{0, 10, PT}, {5, 0, STROKE},
	{0, 0, PT}, {5, 10, END}
};

static CharPoint Ydata[] = {
	{0, 10, PT}, {2.5, 5, PT}, {5, 10, STROKE},
	{2.5, 5, PT}, {2.5, 0, END}
};

static CharPoint Zdata[] = {
	{0, 10, PT}, {5, 10, PT}, {0, 0, PT}, {5, 0, END},
};

static CharPoint Aedata[] = {
	{0, 0, PT}, {0, 6, PT}, {1.5, 8, PT}, {3.5, 8, PT}, {5, 6, PT}, 
	{5, 0, STROKE}, {0, 3, PT}, {5, 3, STROKE},
	{0, 9, PT}, {0, 10, STROKE}, {5, 9, PT}, {5, 10, END}
};
//@}

//@{ @name Ziffern 0 - 9
static CharPoint ZEROdata[] = {
	{5, 1, PT}, {4, 0, PT}, {1, 0, PT},
	{0, 1, PT}, {0, 9, PT}, {1, 10, PT}, {4, 10, PT}, {5, 9, PT},
	{5, 1, STROKE}, 
	{5, 10, PT}, {0, 0, END}
};

static CharPoint ONEdata[] = {
	{0, 5, PT}, {3, 10, PT}, {3, 0, END}
};

static CharPoint TWOdata[] = {
	{0, 8, PT}, {1, 10, PT}, {4, 10, PT}, {5, 9, PT},
	{0, 0, PT}, {5, 0, END}
};

static CharPoint THREEdata[] = {
	{0, 0, PT}, {5, 0, PT}, {5, 10, PT}, {0, 10, STROKE},
	{0, 5, PT}, {5, 5, END}
};

static CharPoint FOURdata[] = {
	{0, 10, PT}, {0, 5, PT}, {5, 5, PT}, {5, 10, STROKE},
	{5, 5, PT}, {5, 0, END}
};

static CharPoint FIVEdata[] = {
	{0, 0, PT}, {5, 0, PT}, {5, 5, PT}, {0, 5, PT}, {0, 10, PT},
	{5, 10, END}
};

static CharPoint SIXdata[] = {
	{5, 10, PT}, {0, 10, PT}, {0, 0, PT}, {5, 0, PT}, {5, 5, PT},
	{0, 5, END}
};

static CharPoint SEVENdata[] = {
	{0, 10, PT}, {5, 10, PT}, {0, 0, END}
};

static CharPoint EIGHTdata[] = {
	{0, 0, PT}, {0, 10, PT}, {5, 10, PT}, {5, 0, PT}, {0, 0, STROKE},
	{0, 5, PT}, {5, 5, END}
};

static CharPoint NINEdata[] = {
	{0, 0, PT}, {5, 0, PT}, {5, 10, PT}, {0, 10, PT}, {0, 5, PT},
	{5, 5, END}
};
//@}

//@{ @name Sonderzeichen
static CharPoint SPACEdata[] = {
	{0, 0, END}
};

static CharPoint COLONdata[] = {
	{3, 2, PT}, {3, 4, STROKE}, {3, 6, PT}, {3, 8, END}
};

static CharPoint PLUSdata[] = {
	{0, 5, PT}, {5, 5, STROKE}, {3, 9, PT}, {3, 2, END}
};

static CharPoint MINUSdata[] = {
	{0, 5, PT}, {5, 5, END}
};

static CharPoint SLASHdata[] = {
	{5, 10, PT}, {0, 0, END}
};

static CharPoint UNDERLINEdata[] = {
	{0, 0, PT}, {5, 0, END}
};

static CharPoint DOTdata[] = {
	{2, 0, PT}, {3, 0, PT}, {3, 1, PT}, {2, 1, PT}, {2, 0, END}
};
//@}

Stroke_Font::Stroke_Font(void) : width(1.0), height(1.0)
/** Konstruktor, Initialisierung der Variablen width und height, ruft init() */
{
	init();
};

Stroke_Font::~Stroke_Font(void)
/** Destruktor, ist leer */
{	
};

void Stroke_Font::init(void)
/** Initialisierung des Feldes mit den Zeigern auf die Buchstaben- bzw.
 *  Zeichenbeschreibung (letter_data) */
{
	short	i;

	for(i = 0; i <= 255; i++) letter_data[i] = NULL;
	letter_data[0 + 'A'] = Adata;
	letter_data[0 + 'B'] = Bdata;
	letter_data[0 + 'C'] = Cdata;
	letter_data[0 + 'D'] = Ddata;
	letter_data[0 + 'E'] = Edata;
	letter_data[0 + 'F'] = Fdata;
	letter_data[0 + 'G'] = Gdata;
	letter_data[0 + 'H'] = Hdata;
	letter_data[0 + 'I'] = Idata;
	letter_data[0 + 'J'] = Jdata;
	letter_data[0 + 'K'] = Kdata;
	letter_data[0 + 'L'] = Ldata;
	letter_data[0 + 'M'] = Mdata;
	letter_data[0 + 'N'] = Ndata;
	letter_data[0 + 'O'] = Odata;
	letter_data[0 + 'P'] = Pdata;
	letter_data[0 + 'Q'] = Qdata;
	letter_data[0 + 'R'] = Rdata;
	letter_data[0 + 'S'] = Sdata;
	letter_data[0 + 'T'] = Tdata;
	letter_data[0 + 'U'] = Udata;
	letter_data[0 + 'V'] = Vdata;
	letter_data[0 + 'W'] = Wdata;
	letter_data[0 + 'X'] = Xdata;
	letter_data[0 + 'Y'] = Ydata;
	letter_data[0 + 'Z'] = Zdata;

	letter_data[0 + 'a'] = Adata;
	letter_data[0 + 'b'] = Bdata;
	letter_data[0 + 'c'] = Cdata;
	letter_data[0 + 'd'] = Ddata;
	letter_data[0 + 'e'] = Edata;
	letter_data[0 + 'f'] = Fdata;
	letter_data[0 + 'g'] = Gdata;
	letter_data[0 + 'h'] = Hdata;
	letter_data[0 + 'i'] = Idata;
	letter_data[0 + 'j'] = Jdata;
	letter_data[0 + 'k'] = Kdata;
	letter_data[0 + 'l'] = Ldata;
	letter_data[0 + 'm'] = Mdata;
	letter_data[0 + 'n'] = Ndata;
	letter_data[0 + 'o'] = Odata;
	letter_data[0 + 'p'] = Pdata;
	letter_data[0 + 'q'] = Qdata;
	letter_data[0 + 'r'] = Rdata;
	letter_data[0 + 's'] = Sdata;
	letter_data[0 + 't'] = Tdata;
	letter_data[0 + 'u'] = Udata;
	letter_data[0 + 'v'] = Vdata;
	letter_data[0 + 'w'] = Wdata;
	letter_data[0 + 'x'] = Xdata;
	letter_data[0 + 'y'] = Ydata;
	letter_data[0 + 'z'] = Zdata;

	letter_data[196] = Aedata;	// Ä
	letter_data[228] = Aedata;	// ä
	letter_data[0 + '0'] = ZEROdata;
	letter_data[0 + '1'] = ONEdata;
	letter_data[0 + '2'] = TWOdata;
	letter_data[0 + '3'] = THREEdata;
	letter_data[0 + '4'] = FOURdata;
	letter_data[0 + '5'] = FIVEdata;
	letter_data[0 + '6'] = SIXdata;
	letter_data[0 + '7'] = SEVENdata;
	letter_data[0 + '8'] = EIGHTdata;
	letter_data[0 + '9'] = NINEdata;

	letter_data[0 + ' '] = SPACEdata;
	letter_data[0 + ':'] = COLONdata;
	letter_data[0 + '+'] = PLUSdata;
	letter_data[0 + '-'] = MINUSdata;
	letter_data[0 + '/'] = SLASHdata;
	letter_data[0 + '_'] = UNDERLINEdata;
	letter_data[0 + '.'] = DOTdata;
};

void Stroke_Font::drawLetter(CharPoint *letter) const
/** zeichnet den Buchstaben mit der Beschreibung 'letter' */
{
	bool	is_end;

	if (letter == NULL) return;

	glBegin(GL_LINE_STRIP);
	is_end = false;
	while (!is_end)
	{
		switch (letter->type)
		{
			case PT: 	glVertex2fv(&letter->x);
					break;
			case STROKE:	glVertex2fv(&letter->x);
					glEnd();
					glBegin(GL_LINE_STRIP);
					break;
			case END:	glVertex2fv(&letter->x);
					is_end = true;
					glEnd();
					glTranslated(
						stroke_font_character_width,
						0.0, 0.0);
					break;
			default:	break;
		};
		letter++;
	};
};

void Stroke_Font::print(const char text[]) const
/** zeichnet die Buchstaben/Zeichen des Textes 'text' */
{
	size_t		i, text_len;
	unsigned char	letter;

	if (text == NULL) return;

	// glPushMatrix();
	pos.translate(1);
	pos.colorForDraw();
	glScaled(width / stroke_font_character_width,
			height / stroke_font_character_height, 1.0);
	text_len = strlen(text);
	for(i = 0; i < text_len; i++)
	{
		letter = text[i];
		// cout << oct << static_cast<unsigned short>(letter) << endl;
		drawLetter(letter_data[ letter ]);
	};
	glTranslated(-1.0 * text_len * stroke_font_character_width, 0.0, 0.0);
	glScaled(stroke_font_character_width / width,
			stroke_font_character_height / height, 1.0);
	pos.translate(-1);
	// glPopMatrix();
};

void Stroke_Font::setColor(double new_red, double new_green,
				double new_blue, double new_alpha)
/** setzt die Farbe des Textes, Wertebereich 0.0 - 1.0,
 *  'new_red' = rot, 'new_green' = gr&uuml;n, 'new_blue' = blau,
 *  'new_alpha' = Transparenz (1.0 undurchsichtig, nur mit Blending) */
{
	pos.setColor(new_red, new_green, new_blue, new_alpha);
};

// draw position of print(): left bottom corner
void Stroke_Font::setPos(double new_centerx, double new_centery,
				 double new_centerz)
/** setzt die Anfangsposition (links unten) der Textausgabe mit print()
 *  (x, y, z) */
{
	pos.setPos(new_centerx, new_centery, new_centerz);
};

void Stroke_Font::setSize(double new_width, double new_height)
/** Gr&ouml;&szlig;e eines Buchstaben setzen, 'new_width' = Breite,
 *  'new_height' = H&ouml;he */
{
	width = new_width;
	height = new_height;
};

