/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <iostream>
#include <string>

#include "constants.h"
#include "glut.h"
#include "menu.h"

/** Name der Einstellungsdatei mit den Men&uuml;eintr&auml;gen */
const char	menufile_filename[] = "etc/menu.xml";

Menu::Menu(void): menu_id(-1), last_entry_value(0)
/** Konstruktor, Initialisierung der Variablen: last_entry_value und menu_id */
{
};

Menu::~Menu(void)
/** Destruktor, ist leer */
{
};

void Menu::create(void (* callback)(int entry))
/** erzeugt das Men&uuml; und verbindet es mit der Callback-Funktion
 *  'callback' und der rechten Maustaste, und l&auml;dt die
 *  Men&uuml;eintr&auml;ge aus der Einstellungsdatei (loadMenu()) */
{
	// int	submenu_id;

	// cout << "Create Menu" << endl;
	menu_id = glutCreateMenu(callback);
	glutAddMenuEntry("", last_entry_value + 1);
	last_entry_value++;
	menu_callback = callback;
	/* glutAddMenuEntry("camera init", 200);
	submenu_id = glutCreateMenu(callback);
	glutAddMenuEntry("Map on/off", 310);
	glutAddMenuEntry("camera pos", 320);
	glutAddMenuEntry("info", 330);
	glutAddMenuEntry("insert", 340);
	glutAddMenuEntry("move", 350);
	glutAddMenuEntry("delete", 360);
	glutSetMenu(menu_id);
	glutAddSubMenu("Map  >", submenu_id);
	glutAddMenuEntry("save network", 400);
	glutAddMenuEntry("load network", 500);
	glutAddMenuEntry("execute network", 600);*/
	loadMenu(menufile_filename);
	glutAttachMenu(GLUT_RIGHT_BUTTON); // connect to right mouse button
};

int Menu::execute(int entry)
/** zur Ausf&uuml;hrung einer Aktion, die mit dem Men&uuml;eintrag 'entry' 
 *  verbunden ist, gibt aber nur den Wert von 'entry' zur&uuml;ck */
{
	// cout << "Menu entry: " << entry << endl;
	// do an action belonging to 'entry'
	return entry;
};

void Menu::destroy(void)
/** l&ouml;scht das Men&uuml; mit glutDestroyMenu() */
{
	glutDestroyMenu(menu_id);
};

void Menu::loadXMLAction(xmlTextReaderPtr infile)
/** l&auml;dt den Aktionsnamen im Attribut 'name' des Tags 'action' in der
 *  XML-Datei 'infile', verbindet ihn mit einer Aktionsnummer ('Action_Dummy'
 *  als Aktion ohne Wirkung bei unbekanntem Namen) und tr&auml;gt
 *  die Aktion in die Tabelle 'menu_entry_actions' ein */
{
	size_t		i;
	Actions		found_action;
	const char	*test_string;
	xmlChar		*content;
	string		action_name;

	found_action = Action_Dummy;
	// read action name from attribute 'name'
	if (xmlTextReaderHasAttributes(infile))
	{
		content = xmlTextReaderGetAttribute(infile,
						xml_syntax_name);
		if (content) action_name = reinterpret_cast<char*>(content);
	};
	// convert action name to action number by Action_Strings
	i = 0;
	test_string = Action_Strings[i];
	while ((strlen(test_string) > 0) && (action_name != ""))
	{
		test_string = Action_Strings[i];
		// compare action_name with entry in Action_Strings
		if ((strncmp(test_string, action_name.c_str(),
			action_name.size()) == 0) &&
			(action_name.size() == strlen(test_string)))
				found_action = Actions(i);
		i++;
	};
	// set value in table
	// cout << "Action: " << action_name << " " << found_action << endl;
	menu_entry_actions[last_entry_value] = found_action;
};

void Menu::loadXMLEntry(xmlTextReaderPtr infile)
/** liest einen Men&uuml;eintrag aus der XML-Datei 'infile' ein */
{
	int	state, depth, return_depth, entry_value;
	xmlChar	*content;
	string	entry_name;

	return_depth = xmlTextReaderDepth(infile);
	entry_name = "empty name";
	entry_value = last_entry_value + 1;
	last_entry_value++;
	// read attribute 'name' as text of menu entry in window
	if (xmlTextReaderHasAttributes(infile))
	{
		content = xmlTextReaderGetAttribute(infile,
						xml_syntax_name);
		if (content) entry_name = reinterpret_cast<char*>(content);
		// cout << "Menu entry: " << entry_name << " ";
		// cout << "Value: " << entry_value << endl;
	};
	// add entry with text to menu
	glutAddMenuEntry(entry_name.c_str(), entry_value);
	// cout << "return_depth: " << return_depth << endl;
	// read all subtags
	do
	{
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		// cout << "loadXMLEntry Depth: " << depth << " ";
		// cout << "Tag: " << xmlTextReaderConstName(infile) << endl;
		// found tag 'action'
		if (xmlStrcasecmp( xmlTextReaderConstName(infile),
			xml_syntax_menu_action) == 0)
		{
			loadXMLAction(infile);
		};
	} while ((state >= 0) && (depth > return_depth));
};

void Menu::loadXML(xmlTextReaderPtr infile)
/** l&auml;dt ein Men&uuml; oder Untermen&uuml; aus der XML-Datei 'infile' */
{
	int	state, depth, return_depth, old_menu_id, submenu_id;
	xmlChar	*content;
	string	menu_name;

	return_depth = xmlTextReaderDepth(infile);
	menu_name = "empty name";
	// read name of menu as text in window (only usable for submenus)
	if (xmlTextReaderHasAttributes(infile))
	{
		content = xmlTextReaderGetAttribute(infile,
						xml_syntax_name);
		if (content) menu_name = reinterpret_cast<char*>(content);
		// cout << "Menu name: " << menu_name << endl;
	};
	old_menu_id = glutGetMenu();	// current menu
	submenu_id = 0;
	if (return_depth >= 2)		// for submenu entries in existing menu
		submenu_id = glutCreateMenu(menu_callback);
	do
	{
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		// cout << "loadXML: Depth: " << depth << " ";
		// cout << "Tag: " << xmlTextReaderConstName(infile) << endl;
		// found tag 'entry' for a menu entry
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_menu_entry) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
		{
			loadXMLEntry(infile);
		}
		// found tag 'menu' as a submenu
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_menu) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
		{
			loadXML(infile);
		};
	} while ((state >= 0) && (depth > return_depth));
	// add submenu to father menu
	glutSetMenu(old_menu_id);
	if (return_depth >= 2)
		glutAddSubMenu(menu_name.c_str(), submenu_id);
};

void Menu::loadMenu(const char filename_to_load[])
/** l&auml;dt ein Men&uuml; (die ganze Men&uuml;hierarchie) aus der XML-Datei
 *  mit dem Namen 'filename_to_load' */
{
	bool			found;
	int			state, depth;
	string			filename;
	xmlTextReaderPtr	infile;

	// copy to intern string
	filename = "";
	if (filename_to_load != NULL)
		if (filename_to_load[0] != 0)	// strlen > 0
			filename = filename_to_load;
	cout << "Loading menu from: " << filename << endl;
	// open file
	infile = xmlNewTextReaderFilename(filename.c_str());
	found = false; state = 1;
	// read until tag 'netview' (top level tag) found
	while ((!found) && (state >= 0))
	{
		state = xmlTextReaderRead(infile);
		if (xmlStrcasecmp( xmlTextReaderConstName(infile),
				xml_syntax_netview) == 0)
			found = true;
	};
	state = xmlTextReaderRead(infile);	// get next node
	depth = xmlTextReaderDepth(infile);
	// read the tags for the menu
	while((state >= 0) && (depth > 0))
	{
		// cout << "Depth: " << depth << endl;
		// found tag 'menu' for a menu
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_menu) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
		{
			loadXML(infile);
		}
		else state = xmlTextReaderRead(infile);
		state = xmlTextReaderReadState(infile);
		depth = xmlTextReaderDepth(infile);
	};
	// close file, free memory
	xmlTextReaderClose(infile);
	xmlFreeTextReader(infile);
};

Actions Menu::translateEntryValue(int entry_value)
/** gibt zum Men&uuml;eintrag 'entry_value' die zugeh&ouml;rige Aktion laut
 *  interner Tabelle 'menu_entry_actions' zur&uuml;ck */
{
	return menu_entry_actions[entry_value];
};

