/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <iostream>
#include <math.h>

#include "constants.h"
#include "glut.h"
#include "mouse.h"

using namespace std;

/** Standardfarbe f&uuml;r die gezeichnete Linie des Mausweges,
 *  siehe Mouse::mouse_line_color */
const double	mouse_line_color_init[4] = {0.0, 1.0, 0.0, 1.0};
/** Breite der gezeichnete Linie des Mausweges, siehe Mouse::mouse_line_color
 */
const float	mouse_line_width = 3.0;
/** Schrittweite einer Kameradrehung, 0.0 - 360.0 Grad */
const double	cam_angle_step = 0.25;
/** Schrittweite der Geschwindigkeits&auml;nderung in der Kameraverschiebung */
const double	cam_speed_step = 0.001;
/** Skalierungsfaktor f&uuml;r die Umrechnung des Mausweges in die
 *  Geschwindigkeit der Kameraverschiebung */
const double	cam_speed_scale = 0.001;
/** Skalierungsfaktor f&uuml;r die Umrechnung des Mausweges in eine
 *  Kameraverschiebung */
const double	cam_pos_scale = 0.0002;
/** Skalierungsfaktor f&uuml;r die Umrechnung des Mausweges in einen Winkel
 *  zur Kamerarotation */
const double	cam_angle_scale = 0.5;
/** Hysteresefaktor zur Bestimmung des gew&uuml;nschten Winkels einer
 *  Kamerarotation  */
const double	angle_hysterese = 0.80;
/** Anfangsposition der Kamera in der 3D-Welt (x, y, z) */
const double	cam_init_pos[3] = {0.0, 1.0, 15.0};
/** Anfangswinkel der Kamera nach Programmstart (x, y), 0.0 - 360.0 Grad; <br>
 *  y = 90.0: geradeaus nach vorn Richtung negative Z-Achse */
const double	cam_init_angle[2] = {0.0, 90.0};	// x, y

Mouse::Mouse(void) : 
		first_is_valid(false),
		current_active_modifiers(0),
		current_window_width(0), current_window_height(0),
		map_mode(cam_mode), 
		mouse_pos_x(0), mouse_pos_y(0), mouse_button(-1),
		pos_x_onenterbutton(0), pos_y_onenterbutton(0),
		mainwindow_cam(NULL), dialog_window(NULL),
		network_web(NULL), map_viewport(NULL)
/** Konstruktor, Initialisierung der Variablen und Zeiger:
 *  - first_is_valid, current_active_modifiers, current_window_width,
 *    current_window_height
 *  - mouse_pos_x, mouse_pos_y, mouse_button, pos_x_onenterbutton,
 *    pos_y_onenterbutton, map_mode
 *  - mainwindow_cam, dialog_window, network_web, map_viewport */
{
};

Mouse::~Mouse(void)
/** Destruktor, ist leer */
{
};

void Mouse::draw(void)
/** zeichnet den Weg der Maus bei gedr&uuml;ckter Maustaste,
 *  von 'first_is_valid' nach 'scaled_pos' */
{
	
	// cout << "Mouse::draw " << endl;
	if (first_is_valid)
	{
		mouse_line_color.colorForDraw();
		glLineWidth(mouse_line_width);
		glBegin(GL_LINES);
			glVertex3d(first_posxz[0], 0.0, first_posxz[1]);
			glVertex3d(scaled_pos[0], 0.0, scaled_pos[1]);
		glEnd();
		glLineWidth(1.0);
	};
};

void Mouse::getLastInfoPos(double return_last_info_posxz[2])
/** gibt die letzte Position der Maus in 'return_last_info_posxz' zur&uuml;ck,
 *  bei der im Info-Modus ein Objekt der Karte ausgew&auml;hlt wurde */
{
	return_last_info_posxz[0] = last_info_posxz[0];
	return_last_info_posxz[1] = last_info_posxz[1];
};

void Mouse::init(Camera *new_camera, Dialog *new_dialog,
			Network *new_network, Viewport *new_map_view)
/** Initialisierung der Zeiger (Kamera 'new_camera',
 *  Dialogfenster 'new_dialog', Netzwerk 'new_network' und
 *  Karten-Fensterbereich 'new_map_view') und der Farbe des Mausweges
 *  'mouse_line_color' */
{
	mainwindow_cam = new_camera;
	dialog_window = new_dialog;
	network_web = new_network;
	map_viewport = new_map_view;
	mouse_line_color.setColor(mouse_line_color_init[0],
					mouse_line_color_init[1],
					mouse_line_color_init[2],
					mouse_line_color_init[3]);
};

void Mouse::do_cam_mode(int state)
/** f&uuml;r den Kameramodus:
 *  - bei Maustastenstatus 'state' == GLUT_UP (Taste losgelassen): Position und
 *    Winkel der Kamera setzen
 *  - bei 'state' == GLUT_DOWN (Taste gerade gedr&uuml;ckt): Position in
 *    Mouse::first_posxz merken f&uuml;r Mauswegdarstellung und
 *    Kamerarichtung */
{
	double	y_angle;

	if (mainwindow_cam == NULL) return;

	mainwindow_cam->move(0.0, 0.0);	// stop
	if (state == GLUT_DOWN)		// button pressed -> save pos
	{
		first_posxz[0] = scaled_pos[0];
		first_posxz[1] = scaled_pos[1];
		first_is_valid = true;
		glutSetCursor(GLUT_CURSOR_CYCLE);	
	};
	// button leaved: set pos and angle of camera
	if ((state == GLUT_UP) && (first_is_valid))
	{
		mainwindow_cam->setPos(first_posxz[0], 1.9, first_posxz[1]);
		y_angle = atan((first_posxz[1] - scaled_pos[1]) / 
				(scaled_pos[0] - first_posxz[0]));
		y_angle *= 180.0 / M_PI;
		if (scaled_pos[0] - first_posxz[0] < 0.0) y_angle += 180.0;
		if ((first_posxz[0] == scaled_pos[0]) &&
			(first_posxz[1] == scaled_pos[1]))
				y_angle = 90.0;
		// cout << "Angle: " << y_angle << endl;
		mainwindow_cam->setAngle(0.0, y_angle);
		glutSetCursor(GLUT_CURSOR_LEFT_ARROW);
		first_is_valid = false;
	};
};

void Mouse::do_info_mode(int state)
/** f&uuml;r den Info-Modus:
 *  - bei Maustastenstatus 'state' == GLUT_DOWN (Taste gerade gedr&uuml;ckt):
 *    Dialogfenster mit Informationen des Objektes &ouml;ffnen, welches gerade
 *    mit der Maus ausgew&auml;hlt wurde */
{
	Variable_Tree*	infos;

	if (dialog_window == NULL) return;
	if (network_web == NULL) return;

	if (state == GLUT_DOWN)
	{	// call dialog with infos of network object on mouse pos
		last_info_posxz[0] = scaled_pos[0];
		last_info_posxz[1] = scaled_pos[1];
		network_web->getInfos(scaled_pos, infos);
		dialog_window->setText(infos);
		dialog_window->redraw();
	};
};

void Mouse::do_insert_mode(int state)
/** f&uuml;r den Einf&uuml;gemodus:
 *  - bei Maustastenstatus 'state' == GLUT_UP (Taste losgelassen):
 *    - bei linker Maustaste ohne Verschiebung: Host einf&uuml;gen
 *    - bei mittlerer Maustaste ohne Verschiebung: Router einf&uuml;gen
 *    - bei Mausverschiebung (mit gedr&uuml;ckter Taste): Kabel einf&uuml;gen
 *  - bei 'state' == GLUT_DOWN (Taste gerade gedr&uuml;ckt): Position in
 *    Mouse::first_posxz merken f&uuml;r Mauswegdarstellung und
 *    Einf&uuml;geort */
{
	if (network_web == NULL) return;

	if (state == GLUT_DOWN)		// button pressed -> save pos
	{
		first_posxz[0] = scaled_pos[0];
		first_posxz[1] = scaled_pos[1];
		first_is_valid = true;
		glutSetCursor(GLUT_CURSOR_CYCLE);
	};
	// button leaved: insert host/router/cable on mouse pos
	if ((state == GLUT_UP) && (first_is_valid))
	{
		if ((scaled_pos[0] == first_posxz[0]) &&
			(scaled_pos[1] == first_posxz[1]))
		{
			if (mouse_button == GLUT_LEFT_BUTTON)
				network_web->insert_host(scaled_pos[0], 
								scaled_pos[1]);
			if (mouse_button == GLUT_MIDDLE_BUTTON)
				network_web->insert_router(scaled_pos[0], 
								scaled_pos[1]);
		}
		else
		{
			network_web->insert_cable(first_posxz, scaled_pos);
		};
		glutSetCursor(GLUT_CURSOR_LEFT_ARROW);
		first_is_valid = false;
	};
};

void Mouse::do_move_mode(int state)
/** f&uuml;r den Verschiebemodus:
 *  - bei Maustastenstatus 'state' == GLUT_UP (Taste losgelassen):
 *    Verschiebe Objekt im Netzwerk vom Anfang des Mausweges zur aktuellen
 *    Position
 *  - bei 'state' == GLUT_DOWN (Taste gerade gedr&uuml;ckt): Position in
 *    Mouse::first_posxz merken f&uuml;r Mauswegdarstellung und
 *    Verschiebungsanfang */
{
	if (network_web == NULL) return;

	if (state == GLUT_DOWN)
	{
		first_posxz[0] = scaled_pos[0];
		first_posxz[1] = scaled_pos[1];
		first_is_valid = true;
		glutSetCursor(GLUT_CURSOR_CYCLE);
	};
	if ((state == GLUT_UP) && (first_is_valid))
	{
		network_web->move(first_posxz, scaled_pos);
		glutSetCursor(GLUT_CURSOR_LEFT_ARROW);
		first_is_valid = false;
	};
};

void Mouse::do_scaling(void)
/** berechnet aus der Mausposition und der bekannten Gr&ouml;&szlig;e der
 *  Kartendarstellung die Mausposition in 3D-Weltkoordinaten, gerundet auf
 *  ganze Zahlen */
{
	double	posxy[2], map_width_height[2], aspect;

	if (map_viewport == NULL) return;

	// sizes of map in window
	aspect = 1.0 * map_viewport->getWidth() / map_viewport->getHeight();
	map_width_height[0] = map_sizes[1] - map_sizes[0];
	map_width_height[1] = map_sizes[3] - map_sizes[2];
	if (aspect >= 1.0)	map_width_height[0] *= aspect;
	else			map_width_height[1] /= aspect;
	// mouse position -> scale to map size
	posxy[0] = mouse_pos_x; posxy[1] = mouse_pos_y;
	map_viewport->scalePoint(current_window_height, map_width_height,
					posxy, scaled_pos);
	// calculate with size aspect of map
	if (aspect >= 1.0)
	{
		scaled_pos[0] += map_sizes[0] * aspect ;
		scaled_pos[1] = map_sizes[3] - scaled_pos[1];
		// scaled_pos[1] += map_sizes[2];
	}
	else
	{
		scaled_pos[0] += map_sizes[0];
		scaled_pos[1] = map_sizes[3] / aspect - scaled_pos[1];
		// scaled_pos[1] += map_sizes[2];
	};
	// round
	scaled_pos[0] = rint(scaled_pos[0]);
	scaled_pos[1] = rint(scaled_pos[1]);
	// cout << "Mouse: Scaled: " << scaled_pos[0] << " " << scaled_pos[1] 
	//	<< endl;
};

void Mouse::mouse_func(int button, int state, int x, int y,
			bool level_of_details_on, bool map_is_shown)
/** Verarbeitung der Mauseingabe: Taste 'button' mit Status 'state' an Position
 *  'x', 'y' (relativ zum Fenster) gedr&uuml;ckt/losgelassen, <br>
 *  - wenn Karte dargestellt wird ('map_is_shown' == 'true') und Maus sich
 *    darin befindet: Skalierung der Mausposition (do_scaling()) und rufe
 *    Methode f&uuml;r den aktuellen Modus: do_cam_mode(), do_insert_mode(),
 *    do_move_mode(), do_info_mode() bzw. Objekt an Mausposition im Netzwerk
 *    l&ouml;schen; <br>
 *  - sonst verschiebe Kamera mit startAction() Action_Cam_Step/Action_Cam_Stop
 *  - zeichne Netzwerk neu mit Status des Level of Details
 *    'level_of_details_on' */
{
	Point_3D	camera_pos;

	if (network_web == NULL) return;
	if (map_viewport == NULL) return;
	if (network_web == NULL) return;

	mouse_pos_x = x;	// save pos
	mouse_pos_y = y;
	mouse_button = button;
	current_active_modifiers = glutGetModifiers();
	if (state == GLUT_DOWN)	// save pos
	{
		pos_x_onenterbutton = x;
		pos_y_onenterbutton = y;
	};
	if ((map_viewport->isInViewport(current_window_height, x, y))
		&& (map_is_shown))
	{
		do_scaling();	// scale mouse pos
		// call method for mode
		if (map_mode == cam_mode) do_cam_mode(state);
		if (map_mode == insert_mode) do_insert_mode(state);
		if ((map_mode == remove_mode) && (state == GLUT_DOWN))
			network_web->remove(scaled_pos[0], scaled_pos[1]);
		if (map_mode == move_mode) do_move_mode(state);	
		if (map_mode == info_mode) do_info_mode(state);
	}
	else
	{	// move camera
		if (mouse_button == GLUT_LEFT_BUTTON)
		{
			if (state  == GLUT_DOWN) startAction(Action_Cam_Step);
			if (state  == GLUT_UP)	 startAction(Action_Cam_Stop);
		};
	};
	// cout << "mouse_func: " << button << " " << state << endl;
	if (state == GLUT_UP) mouse_button = -1;
	// redraw
	mainwindow_cam->getPos(camera_pos);
	network_web->drawSetCameraPos(level_of_details_on, camera_pos);
	glutPostRedisplay();
};

void Mouse::do_motion_shift(bool direction_changed)
/** verschiebe Kamera anhand des Mausweges, wenn sich die Richtung der
 *  Mausverschiebung nicht ge&auml;ndert hat ('direction_changed' == true):
 *  - linke Maustaste: X- und Z-Achse
 *  - mittlere Maustaste: X- und Y-Achse */
{
	double	move_x, move_y, move_z;

	if (mainwindow_cam == NULL) return;

	move_x = 0.0; move_y = 0.0; move_z = 0.0;
	if (direction_changed)	// direction of mouse move changed
	{			// it's a begin of a new movement
		pos_x_onenterbutton = mouse_pos_x;
		pos_y_onenterbutton = mouse_pos_y;
	};
	if (mouse_button == GLUT_LEFT_BUTTON)
	{
		move_x = (mouse_pos_x - pos_x_onenterbutton) * cam_pos_scale;
		move_z = (mouse_pos_y - pos_y_onenterbutton) * cam_pos_scale;
	};
	if (mouse_button == GLUT_MIDDLE_BUTTON)
	{
		move_x = (mouse_pos_x - pos_x_onenterbutton) * cam_pos_scale;
		move_y = (mouse_pos_y - pos_y_onenterbutton) * cam_pos_scale;
		move_y *= -1;
	};
	mainwindow_cam->movePos(move_x, move_y, move_z);
};

void Mouse::motion(int x, int y, bool level_of_details_on)
/** Verarbeitung einer Mausverschiebung mit gedr&uuml;ckter Maustaste zu
 *  Position 'x', 'y' (relativ zum Fenster): Berechnungen, Skalierung der
 *  Mausposition (do_scaling()), Geschwindigkeit und Richtung der Kamera
 *  anpassen:
 *  - linke Maustaste: Verschiebung und Drehung um die senkrechte Achse
 *    (nach links/rechts),
 *  - mittlere Maustaste: Drehung um die waagerechte Achse (nach oben/unten),
 *  - bei gedr&uuml;ckter Shift-Taste: Verschiebung mit do_motion_shift(),
 *
 *  am Ende Netzwerk neu zeichnen */
{
	bool		shift_on, direction_changed;
	int		move_x, move_y;
	double		new_speed, new_y_angle_need, new_y_angle, old_y_angle;
	double		new_x_angle_need, new_x_angle, old_x_angle;
	Point_3D	camera_pos;

	if (map_viewport == NULL) return;
	if (mainwindow_cam == NULL) return;
	if (network_web == NULL) return;

	old_y_angle = (mouse_pos_x - pos_x_onenterbutton) * cam_angle_scale;
	old_x_angle = (mouse_pos_y - pos_y_onenterbutton) * cam_angle_scale;
	move_x = x - mouse_pos_x;
	move_y = y - mouse_pos_y;
	// direction of mouse move changed
	direction_changed = (abs(mouse_pos_x - pos_x_onenterbutton) >
				abs(x - pos_x_onenterbutton)) ||
				(abs(mouse_pos_y - pos_y_onenterbutton) >
					abs(y - pos_y_onenterbutton));
	mouse_pos_x = x;
	mouse_pos_y = y;
	shift_on = false;
	do_scaling();
	// in 3d world and not in map
	if (!map_viewport->isInViewport(current_window_height,
			pos_x_onenterbutton, pos_y_onenterbutton))
	{
		// with presse 'shift' button do_motion_shift()
		shift_on = (current_active_modifiers & GLUT_ACTIVE_SHIFT) > 0;
		if (shift_on) do_motion_shift(direction_changed);
		// left mouse button: angle (left/right) and speed
		if ((mouse_button == GLUT_LEFT_BUTTON) && (!shift_on))
		{
			new_speed = mouse_pos_y - pos_y_onenterbutton;
			new_y_angle_need = mouse_pos_x - pos_x_onenterbutton;
			if (direction_changed)
			{
				pos_x_onenterbutton = mouse_pos_x;
				pos_y_onenterbutton = mouse_pos_y;
				new_speed = mouse_pos_y - pos_y_onenterbutton;
			};
			new_speed *= cam_speed_scale * -1.0;
			new_y_angle_need *= cam_angle_scale;
			new_y_angle = old_y_angle - (1 - angle_hysterese) *
					(new_y_angle_need - old_y_angle);
			new_y_angle = new_y_angle - old_y_angle;
			mainwindow_cam->move(0.0, 0.0); // stop
			mainwindow_cam->move(new_speed, 1.0);
			mainwindow_cam->turn(0.0, new_y_angle);
		};
		// middle mouse button: camera angle (up/down)
		if ((mouse_button == GLUT_MIDDLE_BUTTON) && (!shift_on))
		{
			new_x_angle_need = mouse_pos_y - pos_y_onenterbutton;
			new_x_angle_need *= cam_angle_scale;
			new_x_angle = old_x_angle - (1 - angle_hysterese) *
					(new_x_angle_need - old_x_angle);
			new_x_angle = new_x_angle - old_x_angle;
			mainwindow_cam->turn(new_x_angle, 0.0);
		};
		// cout << "motion " << move_y << endl;
	};
	// redraw
	mainwindow_cam->getPos(camera_pos);
	network_web->drawSetCameraPos(level_of_details_on, camera_pos);
	glutPostRedisplay();
};

void Mouse::passiveMotion(int x, int y)
/** Mausverschiebung ohne gedr&uuml;ckte Maustaste zu Position 'x', 'y'
 *  (relativ zum Fenster): nur sichern der Mausposition, <br>
 *  hier sollte nur wenig getan werden (wenig Zeit verbraucht werden), da
 *  die Methode sehr oft von GLUT gerufen wird (&uuml;ber netview.cpp und
 *  GlutMain)
 */
{
	mouse_pos_x = x;	// save pos: mouse move without button press
	mouse_pos_y = y;
	// cout << "passive motion" << endl;
};

void Mouse::reshape(int width, int height)
/** sichern der aktuellen Ausma&szlig;e des Fensters: Breite 'width' und
 *  H&ouml;he 'height', <br> f&uuml;r sp&auml;tere Berechnungen n&ouml;tig */
{
	current_window_width = width;
	current_window_height = height;
};

void Mouse::startAction(Actions number)
/** f&uuml;hrt die Kommandos und Wertver&auml;nderungen hinter der
 *  Aktion 'number' aus; der Teil, der nicht von GlutMain::startAction() getan
 *  wird */
{
	if (mainwindow_cam == NULL) return;

	switch	(number)
	{
		// rotate camera
		case Action_Cam_Left:
			mainwindow_cam->turn( 0.0,  cam_angle_step); break;
		case Action_Cam_Right:
			mainwindow_cam->turn( 0.0, -cam_angle_step); break;
		case Action_Cam_Up:
			mainwindow_cam->turn( cam_angle_step,  0.0); break;
		case Action_Cam_Down:
			mainwindow_cam->turn(-cam_angle_step,  0.0); break;
		// set camera speed
		case Action_Cam_Faster:
			mainwindow_cam->move( cam_speed_step, 1.0); break;
		case Action_Cam_Slower:
			mainwindow_cam->move(-cam_speed_step, 1.0); break;
		case Action_Cam_Step:
			mainwindow_cam->move( 0.0,  1.0); break;
		case Action_Cam_Stop:
			mainwindow_cam->move( 0.0,  0.0); break;
		// init cam position and direction
		case Action_Cam_Init:
			mainwindow_cam->init();
			mainwindow_cam->setAngle(cam_init_angle[0],
							cam_init_angle[1]);
			mainwindow_cam->setPos(cam_init_pos[0], cam_init_pos[1],
						cam_init_pos[2]);
			mainwindow_cam->move( 0.0,  0.0);
			break;
		// change mode
		case Action_Mode_Cam:
			map_mode = cam_mode; break;
		case Action_Mode_Info:
			map_mode = info_mode; break;
		case Action_Mode_Insert:
			map_mode = insert_mode; break;
		case Action_Mode_Move:
			map_mode = move_mode; break;
		case Action_Mode_Remove:
			map_mode = remove_mode; break;
		default:		break;
	};
};

