/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <iostream>

#include "glut.h"
#include "glutmain.h"

using namespace std;

/** Hauptobjekt, alle Eingabeverarbeitungen und Darstellungen werden
 *  dar&uuml;ber abgewickelt, siehe GlutMain */
GlutMain        mainglut;

/** Text zu den Kommandozeilen-Argumenten des Programms, wird bei jedem
 *  Start ausgegeben, siehe main() und GlutMain::checkArgs() */
const char	usage_string[] =
"Usage: netview [-n network_file] [-p server_port] [-s animation_speed]";
		// command line evaluated by GlutMain::checkArgs()

static void display(void)
/** Callback-Funktion zum Neuzeichnen des Hauptfensters,
 *  ruft nur GlutMain::display() */
{
        mainglut.display();
};

static void dialog_display(void)
/** Callback-Funktion zum Neuzeichnen des Dialogfensters,
 *  ruft nur GlutMain::dialog_display() */
{
	mainglut.dialog_display();
};                                                                                
static void reshape(int width, int height)
/** Callback-Funktion bei Gr&ouml;&szlig;en&auml;nderung des Hauptfensters
 *  (neue Breite 'width', neue H&ouml;he 'height'),
 *  ruft nur GlutMain::reshape() */
{
        mainglut.reshape(width, height);
};

static void windowStatus(int status)
/** Callbackfunktion bei &Auml;nderung des Fensterstatus 'status'
 *  (Sichtbarkeit) des Hauptfensters, ruft nur GlutMain::windowStatus() */
{
	mainglut.windowStatus(status);
};

//@{ @name Tastatureingaben
static void key(unsigned char k, int x, int y)
/** Callback-Funktion zur Bearbeitung von Tastatureingaben (normale Tasten)
 *  im Hauptfenster: Taste 'k', Mausposition 'x', 'y' (relativ zum Fenster);
 *  ruft nur GlutMain::key() */
{
        mainglut.key(k, x, y);
};

static void dialog_key(unsigned char k, int x, int y)
/** Callback-Funktion zur Bearbeitung von Tastatureingaben (normale Tasten)
 *  im Dialogfenster: Taste 'k', Mausposition 'x', 'y' (relativ zum Fenster);
 *  ruft nur GlutMain::dialog_key() */
{
	mainglut.dialog_key(k, x, y);
};

static void specialKey(int k, int x, int y)
/** Callback-Funktion zur Bearbeitung von Tastatureingaben (Sondertasten)
 *  im Hauptfenster: Taste 'k', Mausposition 'x', 'y' (relativ zum Fenster);
 *  ruft nur GlutMain::specialKey(); <br>
 *  Sondertasten sind: Ins, Pos1, Ende, Bild auf, Bild ab, Cursortasten,
 *  F1 - F12 */
{
        mainglut.specialKey(k, x, y);
};

static void dialog_specialKey(int k, int x, int y)
/** Callback-Funktion zur Bearbeitung von Tastatureingaben (Sondertasten)
 *  im Dialogfenster: Taste 'k', Mausposition 'x', 'y' (relativ zum Fenster);
 *  ruft nur GlutMain::dialog_specialKey(); <br>
 *  Sondertasten sind: Ins, Pos1, Ende, Bild auf, Bild ab, Cursortasten,
 *  F1 - F12 */
{
	mainglut.dialog_specialKey(k, x, y);
};
//@}
                
static void idle(void)
/** diese Callback-Funktion wird von GLUT gerufen, wenn (im Hauptfenster)
 *  gerade nichts zu tun ist (kein Neuzeichnen, keine Eingaben), <br>
 *  ruft nur GlutMain::idle() */
{
        mainglut.idle();
};

//@{ @name Maus und Men&uuml;
static void mouse(int button, int state, int x, int y)
/** Callback-Funktion f&uuml;r Mauseingaben im Hauptfenster: Maustaste 'button'
 *  hat Status 'state', Mausposition 'x', 'y' (relativ zum Fenster), <br>
 *  ruft nur GlutMain::mouse_func() */
{
        mainglut.mouse_func(button, state, x, y);
};
                                                                                
static void motion(int x, int y)
/** Callback-Funktion f&uuml;r Mausbewegungen mit gedr&uuml;ckter Maustaste im
 *  Hauptfenster: Mausposition 'x', 'y' (relativ zum Fenster), <br>
 *  ruft GlutMain::motion() */
{
        mainglut.motion(x, y);
};

static void passiveMotion(int x, int y)
/** Callback-Funktion f&uuml;r Mausbewegungen OHNE gedr&uuml;ckte Maustaste im
 *  Hauptfenster: Mausposition 'x', 'y' (relativ zum Fenster), <br>
 *  ruft GlutMain::passiveMotion() */
{
        mainglut.passiveMotion(x, y);
};

void menu(int entry)
/** Callback-Funktion zur Verarbeitung des Aufrufs eines Men&uuml;eintrages
 *  'entry' im Hauptfenster, <br> ruft nur GlutMain::menu_func() */
{
	mainglut.menu_func(entry);
};

static void menuState(int state)
/** Callback-Funktion bei &Auml;nderung des Men&uuml;status 'status' (in
 *  Benutzung oder nicht) im Hauptfenster, ruft nur GlutMain::menuState() */
{
	mainglut.menuState(state);
};

//@}

void functions(void)
/** Bekanntgabe aller Callback-Funktionen des Hauptfensters an GLUT */
{
        glutDisplayFunc(display);
        glutReshapeFunc(reshape);
	glutWindowStatusFunc(windowStatus);
	glutMenuStateFunc(menuState);
        glutKeyboardFunc(key);
        glutSpecialFunc(specialKey);
        glutIdleFunc(idle);
        glutMouseFunc(mouse);
        glutMotionFunc(motion);
        glutPassiveMotionFunc(passiveMotion);
};

int main(int argc, char* argv[])
/** Hauptprogramm:
 *  - Ausgabe des usage_string,
 *  - Initialisierung von GLUT,
 *  - Fenster &ouml;ffnen: GlutMain::createWindow(),
 *  - Callback-Funktionen bekanntgeben: functions(),
 *  - Callback-Funktionen des Dialogs in callback_struct eintragen
 *  - Initialisierung GlutMain::init() mit Callback menu() und callback_struct
 *  - Verarbeitung der Kommandozeilen-Argumente: GlutMain::checkArgs(),
 *  - Eintritt in die Ereignisschleife (event loop): glutMainLoop()
 *
 * gibt immer '0' zur&uuml;ck */
{
	struct callback_struct	dialog_callbacks;

	cout << usage_string << endl;
	glutInit(&argc, argv);
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
        // glutInitDisplayString("double");
	mainglut.createWindow();

	functions();
	dialog_callbacks.display_callback = dialog_display;
	dialog_callbacks.keyboard_callback = dialog_key;
	dialog_callbacks.specialkey_callback = dialog_specialKey;
	mainglut.init(menu, dialog_callbacks);
	mainglut.checkArgs(argc, argv);
	glutMainLoop();
	cout << "program end." << endl;
	return 0;
};

