/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include "constants.h"
#include "nat.h"

/** Name des Variablenfeldes zum Quellnetzwerk von NAT im Info-Variablenbaum
 *  zum Dialog */
const char nat_varname_source_network_mask[] = "NAT source network netmask";
/** Name des Variablenfeldes zum Portbereich f&uuml;r PAT im Info-Variablenbaum
 *  zum Dialog */
const char nat_varname_minport_maxport[] =
					"single IP NAT: min port / max port";
/** Name des Variablenfeldes zum Zielnetzwerk von NAT im Info-Variablenbaum zum
 *  Dialog */
const char nat_varname_free_address_space[] = 
				"free address space: network netmask";

NAT::NAT(void) : min_port(0), max_port(65535)
/** Konstruktor, Initialisierung von min_port und max_port */
{
};

NAT::~NAT(void)
/** Destruktor, ist leer */
{
};

void NAT::init(void)
/** Initialisierung, ist leer */
{
};

void NAT::getInfos(Variable_Tree* &infos)
/** stellt den Variablenbaum f&uuml;r das Dialogfenster zusammen, Zeiger
 *  darauf in 'infos' zur&uuml;ck, <br>
 *  Quellnetzwerk, Zielnetzwerk und Portbereich */
{
	Numeric_Variable<IpAddress>		*var_source_network_mask;
	Numeric_Variable<IpAddress>		*var_free_address_space;
	Numeric_Variable<unsigned short>	*var_minport_maxport;

	current_infos.clear();
	var_source_network_mask = new Numeric_Variable<IpAddress>;
	// source network
	if (var_source_network_mask)
	{
		var_source_network_mask->setName(
					nat_varname_source_network_mask);
		var_source_network_mask->setLength(2, 2);
		var_source_network_mask->setMaxmin(INADDR_NONE, INADDR_ANY);
		var_source_network_mask->setValue(0, source_network);
		var_source_network_mask->setValue(1, source_netmask);
		current_infos.addVar(var_source_network_mask);
	};
	// target network
	var_free_address_space = new Numeric_Variable<IpAddress>;
	if (var_free_address_space)
	{
		var_free_address_space->setName(nat_varname_free_address_space);
		var_free_address_space->setLength(2, 2);
		var_free_address_space->setMaxmin(INADDR_NONE, INADDR_ANY);
		var_free_address_space->setValue(0, target_network);
		var_free_address_space->setValue(1, target_netmask);
		current_infos.addVar(var_free_address_space);
	};
	// port range for PAT (single IP in target network)
	var_minport_maxport = new Numeric_Variable<unsigned short>;
	if (var_minport_maxport)
	{
		var_minport_maxport->setName(nat_varname_minport_maxport);
		var_minport_maxport->setLength(2, 2);
		var_minport_maxport->setMaxmin(65535, 0);
		var_minport_maxport->setValue(0, min_port);
		var_minport_maxport->setValue(1, max_port);
		current_infos.addVar(var_minport_maxport);
	};
	infos = &current_infos;
};

void NAT::setInfos(Variable_Tree* infos)
/** liest die Werte aus dem Variablenbaum ein, der Zeiger 'infos' sollte
 *  auf den internen Baum zeigen, wird aber nicht beachtet, <br>
 *  Quellnetzwerk, Zielnetzwerk und Portbereich */
{
	string					value_text;
	Variable				*current_variable;
	Numeric_Variable<IpAddress>		*var_source_network_mask;
	Numeric_Variable<IpAddress>		*var_free_address_space;
	Numeric_Variable<unsigned short>	*var_minport_maxport;

	// source network
	if (current_infos.findVar(nat_varname_source_network_mask,
					current_variable))
	{
		var_source_network_mask = reinterpret_cast
			<Numeric_Variable<IpAddress>*>(current_variable);
		var_source_network_mask->getValue(0, source_network);
		var_source_network_mask->getValue(1, source_netmask);
	};
	// target network
	if (current_infos.findVar(nat_varname_free_address_space,
					current_variable))
	{
		var_free_address_space = reinterpret_cast
			<Numeric_Variable<IpAddress>*>(current_variable);
		var_free_address_space->getValue(0, target_network);
		var_free_address_space->getValue(1, target_netmask);
	};
	// port range for PAT
	if (current_infos.findVar(nat_varname_minport_maxport, 
					current_variable))
	{
		var_minport_maxport = reinterpret_cast
			<Numeric_Variable<unsigned short>*>(current_variable);
		var_minport_maxport->getValue(0, min_port);
		var_minport_maxport->getValue(1, max_port);
	};
};

void NAT::loadXML(xmlTextReaderPtr infile)
/** l&auml;dt die Einstellungen des NAT aus der XML-Datei 'infile',
 *  Portbereich, Quellnetzwerk und Zielnetzwerk */
{
        int     	state, depth;
	unsigned long	value;
	char		*value_text, *end;
        xmlChar 	*content;
 
	if (xmlTextReaderHasAttributes(infile))
	{	// port range
		content = xmlTextReaderGetAttribute(infile,
						xml_syntax_nat_minport);
		if (content)
		{
			value_text = reinterpret_cast<char*>(content);
			end = 0;
			value = strtoul(value_text, &end, 10);
			if (value < 65536) min_port = value;
		};
		content = xmlTextReaderGetAttribute(infile,
						xml_syntax_nat_maxport);
		if (content)
		{
			value_text = reinterpret_cast<char*>(content);
			end = 0;
			value = strtoul(value_text, &end, 10);
			if (value < 65536) max_port = value;
		};
	};
	do
	{
                state = xmlTextReaderRead(infile);
                depth = xmlTextReaderDepth(infile);
		// source network
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_nat_source_network) == 0))
			source_network.loadXML(infile);
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_nat_source_netmask) == 0))
			source_netmask.loadXML(infile);
		// target network
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_nat_target_network) == 0))
			target_network.loadXML(infile);
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_nat_target_netmask) == 0))
			target_netmask.loadXML(infile);
	} while ((state >= 0) && (depth > 2));
};

void NAT::saveXML(xmlTextWriterPtr outfile) const
/** speichert die Einstellungen des Hosts in der XML-Datei 'outfile',
 *  Portbereich, Quellnetzwerk und Zielnetzwerk */
{
	xmlTextWriterStartElement(outfile, xml_syntax_nat);
	// port range
	xmlTextWriterWriteFormatAttribute(outfile, xml_syntax_nat_minport,
						"%d", min_port);
	xmlTextWriterWriteFormatAttribute(outfile, xml_syntax_nat_maxport,
						"%d", max_port);
	// source network
	xmlTextWriterStartElement(outfile, xml_syntax_nat_source_network);
	source_network.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	xmlTextWriterStartElement(outfile, xml_syntax_nat_source_netmask);
	source_netmask.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	// target network
	xmlTextWriterStartElement(outfile, xml_syntax_nat_target_network);
	target_network.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	xmlTextWriterStartElement(outfile, xml_syntax_nat_target_netmask);
	target_netmask.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	xmlTextWriterEndElement(outfile);
};

bool NAT::isInNetwork(IpAddress test_address, IpAddress network,
			IpAddress netmask)
/** ist IP-Adresse 'test_address' im Adressbereich des Netzwerkes, angegeben
 *  durch 'network' und 'netmask' ?
 *  'true' wenn ja, sonst 'false' */
{
	bool		is_in_network;
	struct in_addr	test_inaddress, network_inaddress, netmask_inaddress;

	is_in_network = false;
	test_address.getAddress(test_inaddress);
	network.getAddress(network_inaddress);
	netmask.getAddress(netmask_inaddress);
	test_inaddress.s_addr =
		test_inaddress.s_addr & netmask_inaddress.s_addr;
	if (test_inaddress.s_addr == network_inaddress.s_addr)
		is_in_network = true;
	return is_in_network;
};

void NAT::findAndSetSourcePAT(IPv4_Packet *ip_packet)
/** PAT: Quellnetzwerk zum Zielnetzwerk: setzte Quelladresse und Quellport
 *  von IP-Paket 'ip_packet' neu (anhand Tabelle bzw. unbenutzten Port finden),
 *  <br> bei Fehlern bleibt Paket unbearbeitet */
{
	bool						found;
	unsigned short					found_port;
	struct ipaddr_port				source_ipaddr_port;
	map<unsigned short, ipaddr_port>::iterator	current_pat_translation;
	pair<unsigned short, ipaddr_port>		translation_pair;

	if (ip_packet == NULL) return;
	// read source address and source port
	source_ipaddr_port.port = ip_packet->getSourcePort();
	ip_packet->getSourceIpAddress(source_ipaddr_port.address);
	cout << "Source Address:Port : "
		<< source_ipaddr_port.address << ":"
		<< source_ipaddr_port.port << endl;
	current_pat_translation = pat_translations.begin();
	found = false;
	found_port = 0;
	// search for it in table
	while ((current_pat_translation != pat_translations.end()) && (!found))
	{
		translation_pair = *current_pat_translation;
		if ((translation_pair.second.port == 
				source_ipaddr_port.port) &&
				(translation_pair.second.address ==
				source_ipaddr_port.address))
		{
			found = true;
			found_port = translation_pair.first;	
		};
		if (!found) current_pat_translation++;
	};
	if (!found)
	{	// find free port in defined range, if not found
		found_port = min_port;
		while ((found_port <= max_port) && (!found))
		{
			if (pat_translations.find(found_port) == 
					pat_translations.end())
				found = true;
			if (!found) found_port++;
		};
		if (found)	// into table
			pat_translations[found_port] = source_ipaddr_port;
	};
	if (found)
	{	// set found values
		cout << "NAT: set source to: " << target_network << ":"
			<< found_port << endl;
		ip_packet->setSourcePort(found_port);
		ip_packet->setSourceIpAddress(target_network.getAddress());
	};
};

void NAT::incIpAddress(IpAddress &address)
/** erh&ouml;ht 'address' um eins, d.h. die n&auml;chste Adresse des Netzwerkes
 *  wird zur&uuml;ckgegeben, <br>
 *  &uuml;berspringt *.0 und *.255: es wird dann .1 gesetzt */
{
	in_addr_t	address_part;
	struct in_addr	in_address;

	address.getAddress(in_address);
	address_part = ntohl(in_address.s_addr);
	in_address.s_addr = htonl(++address_part);
	if ((in_address.s_addr & IN_CLASSC_HOST) == INADDR_ANY)
		address_part++;
	if ((in_address.s_addr & IN_CLASSC_HOST) == 
			(INADDR_BROADCAST & IN_CLASSC_HOST))
		address_part += 2;
	in_address.s_addr = htonl(address_part);
	address.setAddress(in_address);
	// cout << "increased to: " << address << endl;
};

void NAT::findAndSetSourceNAT(IPv4_Packet *ip_packet)
/** NAT: Quellnetzwerk zum Zielnetzwerk: setzte Quelladresse von IP-Paket
 *  'ip_packet' neu (anhand Tabelle bzw. unbenutzte Adresse finden mit
 *  Hilfe von incIpAddress() und isInNetwork()), <br>
 *  bei Fehlern bleibt Paket unbearbeitet */
{
	bool					found;
	IpAddress				found_address, source_address;
	IpAddress				destination_address;
	map<IpAddress, IpAddress>::iterator	current_nat_translation;
	pair<IpAddress, IpAddress>		translation_pair;

	if (ip_packet == NULL) return;
	// get source address and destination address
	ip_packet->getSourceIpAddress(source_address);
	ip_packet->getDestinationIpAddress(destination_address);
	cout << "Source Address: "	<< source_address << " ";
	cout << "Destination Address: " << destination_address << endl;
	current_nat_translation = nat_translations.begin();
	found = false;
	found_address = 0;
	// search for it in table
	while ((current_nat_translation != nat_translations.end()) && (!found))
	{
		translation_pair = *current_nat_translation;
		if (translation_pair.second == source_address)
		{
			found = true;
			found_address = translation_pair.first;	
		};
		if (!found) current_nat_translation++;
	};
	if (!found)
	{	// not found -> find free address in target network
		found_address = target_network;
		incIpAddress(found_address);
		while ((isInNetwork(found_address, target_network, 
					target_netmask)) && (!found))
		{
			if (nat_translations.find(found_address) == 
					nat_translations.end())
				found = true;
			if (!found) incIpAddress(found_address);
		};
		if (found)	// write it into table
			nat_translations[found_address] = source_address;
	};
	if (found)
	{	// set found value
		cout << "NAT: set source to: " << found_address << endl;
		ip_packet->setSourceIpAddress(found_address.getAddress());
	}
	else
	{
		cout << "NAT: could not find a free address." << endl;
	};

};

void NAT::findAndSetDestinationPAT(IPv4_Packet* &ip_packet)
/** PAT Zielnetzwerk zu Quellnetzwerk: setze Zieladresse und Zielport von
 *  IP-Paket 'ip_packet' neu (anhand von Tabelle), <br>
 *  bei Fehlern oder wenn Paket nicht als Antwort auf ein Paket des
 *  Quellnetzwerkes (kein Tabelleneintrag), wird IP-Paket gel&ouml;scht */
{
	bool						found;
	IpAddress					source_ip, dest_ip;
	unsigned short					destination_port;
	struct ipaddr_port				destination_ipaddr_port;
	pair<unsigned short, ipaddr_port>		translation_pair;
	map<unsigned short, ipaddr_port>::iterator	found_pat_translation;

	if (ip_packet == NULL) return;
	// get addresses and port
	ip_packet->getSourceIpAddress(source_ip);
	ip_packet->getDestinationIpAddress(dest_ip);
	destination_port = ip_packet->getDestinationPort();
	cout << "Source: " << source_ip << ":" << ip_packet->getSourcePort()
		<< endl;
	cout << "Destination: " << dest_ip << ":" << destination_port << endl;	
	found = false;
	// search in table for target port of packet
	found_pat_translation = pat_translations.find(destination_port);
	if (found_pat_translation != pat_translations.end()) found = true;
	if (found)
	{	// set found values
		translation_pair = *found_pat_translation;
		destination_ipaddr_port.address =
					translation_pair.second.address;
		destination_ipaddr_port.port = translation_pair.second.port;
		ip_packet->setDestinationPort(destination_ipaddr_port.port);
		ip_packet->setDestinationIpAddress(
				destination_ipaddr_port.address.getAddress());
		cout << "NAT: Set Destination to: " 
			<< destination_ipaddr_port.address.getAddress() << " "
			<< destination_ipaddr_port.port << endl;
	}
	else
	{
		cout << "NAT: couldn't find translation so deleting packet."
			<< endl;
		delete ip_packet;
		ip_packet = NULL;
	};
};

void NAT::findAndSetDestinationNAT(IPv4_Packet* &ip_packet)
/** NAT Zielnetzwerk zu Quellnetzwerk: setze Zieladresse des IP-Paketes
 *  'ip_packet' neu (anhand von Tabelle), <br>
 *  bei Fehlern oder wenn Paket nicht als Antwort auf ein Paket des
 *  Quellnetzwerkes (kein Tabelleneintrag), wird IP-Paket gel&ouml;scht */
{
	bool		found;
	IpAddress				found_address;
	IpAddress				destination_address;
	map<IpAddress, IpAddress>::iterator	found_nat_translation;
	pair<IpAddress, IpAddress>		translation_pair;

	if (ip_packet == NULL) return;
	// read destination address
	ip_packet->getDestinationIpAddress(destination_address);
	found = false;
	// search it in table
	found_nat_translation = nat_translations.find(destination_address);
	if (found_nat_translation != nat_translations.end()) found = true;
	if (found)
	{	// set found value
		translation_pair = *found_nat_translation;
		found_address = translation_pair.second;
		ip_packet->setDestinationIpAddress(found_address.getAddress());
		cout << "NAT: Set Destination to: " << found_address << endl;
	}
	else
	{
		cout << "NAT: couldn't find translation so deleting packet."
			<< endl;
		delete ip_packet;
		ip_packet = NULL;
	};
};

void NAT::translate(IPv4_Packet* &ip_packet)
/** &uuml;bersetzte Paket 'ip_packet', falls es aus dem Quell- oder
 *  Zielnetzwerk kommt */
{
	IpAddress	source_address, destination_address;
	bool		use_pat, use_nat;

	if (ip_packet == NULL) return;
	cout << "NAT::translate" << endl;
	// PAT or NAT ?
	use_pat = false; use_nat = false;
	if (target_netmask == IpAddress(INADDR_NONE)) use_pat = true;
	else use_nat = true;

	// read addresses of packet
	ip_packet->getSourceIpAddress(source_address);
	ip_packet->getDestinationIpAddress(destination_address);
	if (use_pat)
	{
		// from internal net to external net
		if (isInNetwork(source_address, source_network,
				source_netmask))
		{
			findAndSetSourcePAT(ip_packet);
		}
		// from external net to internal net
		else if (isInNetwork(destination_address, target_network,
				target_netmask))
		{
			findAndSetDestinationPAT(ip_packet);
		};
	};
	if (use_nat)
	{
		// from internal net to external net
		if (isInNetwork(source_address, source_network,
				source_netmask))
		{
			findAndSetSourceNAT(ip_packet);
		}
		// from external net to internal net
		else if (isInNetwork(destination_address, target_network,
				target_netmask))
		{
			findAndSetDestinationNAT(ip_packet);
		};
	};
};

