/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include "constants.h"
#include "route.h"

Route::Route(void)
/** Konstruktor, setzt Zeiger zum Feld mit dem Interface-Namen (Name der
 *  Netzwerkkarte) auf NULL */
{
	route_entry.rt_dev = NULL;
};

Route::~Route(void)
/** Destruktor, l&ouml;scht das Feld mit dem Interface-Namen */
{
	if (route_entry.rt_dev) delete [] route_entry.rt_dev;
};

void Route::updateDeviceName(string new_device)
/** altes Feld mit dem Interface-Namen wird gel&ouml;scht und neues Feld
 *  passender L&auml;nge wird angelegt, Inhalt von 'new_device' wird
 *  hineinkopiert */
{
	if (route_entry.rt_dev) delete [] route_entry.rt_dev;
	route_entry.rt_dev = new char [new_device.size() + 1];
	if (route_entry.rt_dev)
	{
		memset(route_entry.rt_dev, 0, new_device.size() + 1);
		strncpy(route_entry.rt_dev, new_device.c_str(), 
			new_device.size());
	};
};

void Route::init(void)
/** Initialisierung des Routeneintrags in alle verwendeten Datenfeldern:
 *  Zielnetzwerk, Gateway-Adresse und Netzmaske auf '0.0.0.0' und Interface-Name
 *  auf 'eth0' */
{
	struct sockaddr_in	socket_address;

	// cout << "Route::init" << endl;
	socket_address.sin_addr.s_addr = INADDR_ANY;
	memcpy(&route_entry.rt_dst, &socket_address, sizeof(struct sockaddr));
	memcpy(&route_entry.rt_gateway, &socket_address, 
		sizeof(struct sockaddr));
	memcpy(&route_entry.rt_genmask, &socket_address, 
		sizeof(struct sockaddr));
	updateDeviceName("eth0");
	// cout << "Route::init end" << endl;
};

void Route::genIpAddress(IpAddress &destination, IpAddress &gateway,
				IpAddress &genmask) const
/** erzeugt aus dem Routeneintrag die Objekte der Klasse IpAddress mit den
 *  Adressen Zielnetzwerk 'destination', Gateway 'gateway' und
 *  Netzmaske 'genmask' */
{
	struct sockaddr_in	socket_address;

	memcpy(&socket_address, &route_entry.rt_dst,
		sizeof(struct sockaddr_in));
	destination.setAddress(socket_address.sin_addr);
	memcpy(&socket_address, &route_entry.rt_gateway, 
		sizeof(struct sockaddr_in));
	gateway.setAddress(socket_address.sin_addr);
	memcpy(&socket_address, &route_entry.rt_genmask,
		sizeof(struct sockaddr_in));
	genmask.setAddress(socket_address.sin_addr);
};

string Route::getRoute(void)
/** erzeugt aus dem Routeneintrag einen Text der Route, fast so wie es bei
 *  "/sbin/route" aussieht, dieser wird zur&uuml;ckgegeben */
{
	IpAddress		destination, gateway, genmask;
	string			route_text;

	genIpAddress(destination, gateway, genmask);
	route_text = destination.getAddress();
	route_text += " ";
	route_text += gateway.getAddress();
	route_text += " ";
	route_text += genmask.getAddress();
	route_text += " ";
	route_text += route_entry.rt_dev;
	return route_text;
};

string Route::getInterface(void)
/** gibt nur den Interface-Namen dieses Routeneintrags zur&uuml;ck */
{
	string	interface_text;

	interface_text = route_entry.rt_dev;
	return interface_text;
};

void Route::getGatewayAddress(IpAddress &gateway_address)
/** gibt die Adresse des Gateway dieser Route in 'gateway_address' zur&uuml;ck
 */
{
	struct sockaddr_in	socket_address;

	memcpy(&socket_address, &route_entry.rt_gateway, 
		sizeof(struct sockaddr_in));
	gateway_address.setAddress(socket_address.sin_addr);
};

bool Route::takeAddress(string &new_route, struct sockaddr &target)
/** nimmt sich die erste Adresse aus dem Text 'new_route', d.h. alle
 *  Zeichen bis zum ersten Leerzeichen werden als Adresse interpretiert,
 *  diese Adresse wird in 'target' zur&uuml;ckgegeben, wenn sie g&uuml;ltig
 *  ist, sonst bleibt der alte Wert erhalten, <br>
 *  bei Fehler (Text keine g&uuml;ltige Adresse) wird 'false'
 *  zur&uuml;ckgegeben, sonst 'true' ohne Fehler */
{
	bool			error;
	size_t			separator_place;
	struct sockaddr_in	socket_address;
	IpAddress		ip_address;
	string			value_text;

	error = false;
	separator_place = new_route.find(' ');
	value_text = new_route.substr(0, separator_place);
	// cout << "Route: |" << value_text.c_str() << '|' << endl;
	// copy old value to save it
	memcpy(&socket_address, &target, sizeof(struct sockaddr_in));
	ip_address.setAddress(socket_address.sin_addr);
	// convert text to address, if error value not changed
	if (!ip_address.setAddress(value_text.c_str())) error = true;
	else
	{
		// copy address
		ip_address.getAddress(socket_address.sin_addr);
		memcpy(&target, &socket_address, sizeof(struct sockaddr));
	};
	new_route.erase(0, separator_place + 1);
	// cout << "new_route: " << new_route << endl;
	return !error;
};

bool Route::setRoute(string new_route)
/** &uuml;bernimmt die Route, die in 'new_route' als Text beschrieben ist,
 *  nutzt dazu takeAddress() und updateDeviceName(),
 *  wenn ein Aufruf (von drei) von takeAddress() einen Fehler meldet, liefert
 *  die Methode 'false' zur&uuml;ck und die alte Route bleibt erhalten,
 *  sonst 'true' bei Erfolg */
{
	bool	error;
	string	old_route;
	
	error = false;
	old_route = getRoute();
	if (!takeAddress(new_route, route_entry.rt_dst)) error = true;
	if (!takeAddress(new_route, route_entry.rt_gateway)) error = true;
	if (!takeAddress(new_route, route_entry.rt_genmask)) error = true;
	updateDeviceName(new_route);
	if (error) setRoute(old_route);
	return !error;
};

void Route::loadXML(xmlTextReaderPtr infile)
/** l&auml;dt den Routeneintrag aus der XML-Datei 'infile', <br>
 *  Zielnetzwerk, Gateway-Adresse, Netzmaske und Interface-Name */
{
	int			state, depth;
	xmlChar			*content;
	string			name;
	IpAddress		ip_address;
	struct sockaddr_in	socket_address;

	do
	{
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		// destination of route
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_route_destination) == 0))
		{
			ip_address.loadXML(infile);
			ip_address.getAddress(socket_address.sin_addr);
			memcpy(&route_entry.rt_dst, &socket_address, 
				sizeof(struct sockaddr));
		};
		// gateway address
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_route_gateway) == 0))
		{
			ip_address.loadXML(infile);
			ip_address.getAddress(socket_address.sin_addr);
			memcpy(&route_entry.rt_gateway, &socket_address, 
				sizeof(struct sockaddr));
		};
		// netmask of route (genmask)
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_route_genmask) == 0))
		{
			ip_address.loadXML(infile);
			ip_address.getAddress(socket_address.sin_addr);
			memcpy(&route_entry.rt_genmask, &socket_address, 
				sizeof(struct sockaddr));
		};
		// interface name (device name)
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_route_interface) == 0))
		{
			if (xmlTextReaderHasAttributes(infile))
			{
				content = xmlTextReaderGetAttribute(infile,
							xml_syntax_name);
				if (content)
				{
					name = reinterpret_cast<char*>(content);
					updateDeviceName(name);
				};
			};	
		};
	} while ((state >= 0) && (depth > 3));
};

void Route::saveXML(xmlTextWriterPtr outfile) const
/** speichert den Routeneintrag in die XML-Datei 'outfile', <br>
 *  Zielnetzwerk, Gateway-Adresse, Netzmaske und Interface-Name */
{
	IpAddress		destination, gateway, genmask;

	genIpAddress(destination, gateway, genmask);
	xmlTextWriterStartElement(outfile, xml_syntax_route);
	// destination
	xmlTextWriterStartElement(outfile, xml_syntax_route_destination);
	destination.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	// gateway
	xmlTextWriterStartElement(outfile, xml_syntax_route_gateway);
	gateway.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	// netmask (genmask)
	xmlTextWriterStartElement(outfile, xml_syntax_route_genmask);
	genmask.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	// interface name (device name)
	xmlTextWriterStartElement(outfile, xml_syntax_route_interface);
	xmlTextWriterWriteFormatAttribute(outfile, xml_syntax_name,
						"%s", route_entry.rt_dev);
	xmlTextWriterEndElement(outfile);
	xmlTextWriterEndElement(outfile);
};

short Route::matchRoute(IPv4_Packet *ip_packet) const
/** vergleicht den Routeneintrag mit der Zieladresse des IP-Paketes
 *  'ip_packet', je gr&ouml;&szlig;er der R&uuml;ckgabewert, desto besser
 *  passt die Route, <br>
 *  zur Zeit wird mit der Netzmaske der Netzwerkteil der Zieladresse des
 *  Paketes bestimmt, dieser wird mit der Adresse des Zielnetzwerkes
 *  (auch verkn&uuml;pft mit der Netzmaske) verglichen: ohne v&ouml;llige
 *  &Uuml;bereinstimmung wird '-1' geliefert,
 *  ansonsten werden die Bits auf '1' in der Netzmaske gez&auml;hlt,<br>
 *  ('192.168.1.0/255.255.255.0 passt besser als '192.168.0.0/255.255.0.0' auf
 *  die Paketadresse 192.168.1.1) */
{
	short		matching, i;
	IpAddress	destination, gateway, genmask, packet_address;
	struct in_addr	dest_inaddress, mask_inaddress, packet_inaddress;

	if (ip_packet == NULL) return 0;
	// get addresses out ouf route
	genIpAddress(destination, gateway, genmask);
	destination.getAddress(dest_inaddress);
	genmask.getAddress(mask_inaddress);
	// mask & destination -> destination
	dest_inaddress.s_addr = dest_inaddress.s_addr & mask_inaddress.s_addr;
	// (destination address of packet) & mask -> packet_address
	ip_packet->getDestinationIpAddress(packet_address);
	packet_address.getAddress(packet_inaddress);
	packet_inaddress.s_addr = packet_inaddress.s_addr & 
					mask_inaddress.s_addr;
	// match packet_address and destination
	if (packet_inaddress.s_addr != dest_inaddress.s_addr)
		matching = -1;
	else
	{
		matching = 0;
		// cout << "Mask: " << genmask << endl;
		// count '1' bits in netmask to get better route out of two
		// matching
		for(i = 0; i < 32; i++)
		{
			if ((mask_inaddress.s_addr & 1) > 0) matching++;
			mask_inaddress.s_addr = mask_inaddress.s_addr >> 1;
		};
		// cout << "Route: Matching: " << int(matching) << endl;
	};
	return matching;
};

