/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include <iostream>

#include "host_executenet.h"
#include "http_service.h"
#include "udp_service_packet.h"

using namespace std;

Host_ExecuteNet::Host_ExecuteNet(void)
/** Konstruktor, ist leer */
{
};

Host_ExecuteNet::~Host_ExecuteNet(void)
/** Destruktor, l&ouml;scht alle Dienste in der Liste (und die Liste selber) */
{
	Service*			current_service;
	list<Service*>::iterator	service_entry;

	service_entry = services.begin();
	while(service_entry != services.end())
	{
		current_service = *service_entry;
		if (current_service != NULL) delete current_service;
		*service_entry = NULL;
		service_entry++;
	};
	services.erase(services.begin(), services.end());
};

bool Host_ExecuteNet::serviceSend(void)
// steps to simulate virtual network, service level send
/** Netzwerksimulation: Abfrage der Dienste, ob sie Pakete zu senden haben,
 *  erstes Paket wird entnommen und am Ende der TCP-/UDP-Schlangen zum Senden
 *  angef&uuml;gt, 'true' wenn Paket gesendet wurde */
{
	size_t				counter;
	Service				*current_service;
	Service_Packet			*data_packet;
	TCP_Packet			*tcp_packet;
	UDP_Packet			*udp_packet;
	list<Service*>::iterator	all_services;

	// *** down *** -> Sending packets
	data_packet = NULL;
	tcp_packet = NULL;
	udp_packet = NULL;
	all_services = services.begin();
	// cout << "Services: " << services.size() << endl;
	counter = 0;
	while ((counter < services.size()) && (data_packet == NULL))
	{
		current_service = *(all_services);
		// cout << "Testing Service: " << current_service << endl;
		if (current_service)
		{
			current_service->out(data_packet);
			if (data_packet)
			{
				cout << "Host: Getting packet from service: ";
				cout << data_packet << endl;
			};
		};
		all_services++;
		counter++;
	};
	if (data_packet)
	{
		if (data_packet->isProtocol(TCP))
			tcp_packet = reinterpret_cast<TCP_Packet*>(data_packet);
		if (data_packet->isProtocol(UDP))
			udp_packet = reinterpret_cast<UDP_Packet*>(data_packet);
	};
	if (tcp_packet)
	{
		cout << "Host: Inserting packet " << tcp_packet 
			<< " in tcp queue." << endl;
		tcp_queue.down_in(tcp_packet);
	}
	if (udp_packet)
	{
		cout << "Host: Inserting packet in udp queue." << endl;
		udp_queue.down_in(udp_packet);
	};
	// return if packet got from service
	return (data_packet != NULL);
};

bool Host_ExecuteNet::tcpSend(void)
// steps to simulate virtual network, tcp level send
/** Netzwerksimulation: Paket am Anfang der TCP-Schlange zum Senden wird
 *  entnommen und mit intoInterface() dem Routing &uuml;bergeben,
 *  'true' wenn Paket gefunden wurde */
{
	TCP_Packet	*tcp_packet;
	IPv4_Packet	*ip_packet;

	tcp_packet = NULL;
	ip_packet = NULL;
	tcp_queue.down_out(tcp_packet);
	if (tcp_packet != NULL)
	{
		if (tcp_packet->isProtocol(IPv4))
			ip_packet = reinterpret_cast<IPv4_Packet*>(tcp_packet);
		cout << "Host: Getting packet from tcp queue: "
			<< ip_packet << endl;
	};
	if (ip_packet != NULL) intoInterface(ip_packet);
	// return if packet got from tcp_queue
	return (tcp_packet != NULL);
};

bool Host_ExecuteNet::udpSend(void)
// steps to simulate virtual network, udp level send
/** Netzwerksimulation: Paket am Anfang der UDP-Schlange zum Senden wird
 *  entnommen und mit intoInterface() dem Routing &uuml;bergeben,
 *  'true' wenn Paket gefunden wurde */
{
	UDP_Packet	*udp_packet;
	IPv4_Packet	*ip_packet;

	udp_packet = NULL;
	ip_packet = NULL;
	udp_queue.down_out(udp_packet);
	if (udp_packet)
	{
		if (udp_packet->isProtocol(IPv4))
			ip_packet = reinterpret_cast<IPv4_Packet*>(udp_packet);
		cout << "Host: Getting packet from tcp queue: "
			<< ip_packet << endl;
	};
	if (ip_packet != NULL) intoInterface(ip_packet);
	// return if packet got from udp_queue
	return (udp_packet != NULL);
};

void Host_ExecuteNet::intoInterface(IPv4_Packet* ip_packet)
// after TCPSend/UDPSend, transfer packet to interface
/** Routing eines Paketes: Anhand der IP-Zieladresse des Paketes wird
 *  Routentabelle (Route_Table::findInterface()) abgefragt, der Name der
 *  Netzwerkkarte aus der Route entnommen und wenn der Name zur (einzigen)
 *  Karte im Host passt, wird das Paket ihr &uuml;bergeben; <br>
 *  bei Fehler (keine Route etc.) wird Paket gel&ouml;scht; <br>
 *  IP-Quelladresse im Paket wird gesetzt, */
{
	bool	no_delete;
	string	interface_name;
	Route*	found_route;

	interface_name = "";
	found_route = NULL;
	no_delete = false;
	cout << "Host::executeNetIntoInterface" << endl;
	if (ip_packet)
	{
		cout << "Host: setting source address: "
			<< network_card.getIpAddress() << endl;
		// set source address
		ip_packet->setSourceIpAddress(network_card.getIpAddress());
		// cout << "Finding interface" << endl;
		// find route for packet
		interface_name = route_table.findInterface(ip_packet,
								found_route);
		cout << "Host: Insert in interface: "
			<< interface_name << endl;
		// cout << "Host: Found route: " << found_route << endl;
		// give to interface
		if ((interface_name != "") && (found_route != NULL))
		{
			ip_packet->setRoute(found_route);
			if (interface_name == network_card.getName())
			{
				network_card.down_in(ip_packet);
				no_delete = true;
			};
		}
		if (!no_delete)
		{	// or delete
			cout << "Host_ExecuteNet: Can't find route for packet "
				<< ip_packet << ". Deleting it." << endl;
			delete ip_packet;
			ip_packet = NULL;
		}
	};
};

bool Host_ExecuteNet::ipReceive(void)
// steps to simulate virtual network, interface level receive 
/** Netzwerksimulation: empfangenes Paket wird von der Netzwerkkarte
 *  &uuml;bernommen und ans Ende der TCP- oder UDP-Schlangen zum Empfangen
 *  gesetzt, 'true' wenn Paket empfangen wurde */
{
	IPv4_Packet	*ip_packet;
	TCP_Packet	*tcp_packet;
	UDP_Packet	*udp_packet;

	tcp_packet = NULL;
	udp_packet = NULL;
	ip_packet = NULL;
	network_card.up_out(ip_packet);
	if (ip_packet != NULL)
	{
		cout << "Host: Received ip_packet " << ip_packet << endl;
		if (ip_packet->isProtocol(TCP))
			tcp_packet = reinterpret_cast<TCP_Packet*>(ip_packet);
		if (ip_packet->isProtocol(UDP))
			udp_packet = reinterpret_cast<UDP_Packet*>(ip_packet);
	};
	// cout << "Host: Testing tcp_packet" << endl;
	if (tcp_packet != NULL)
	{
		tcp_queue.up_in(tcp_packet);
	};
	if (udp_packet != NULL)
	{
		udp_queue.up_in(udp_packet);
	};
	// return if packet got from interface
	return (ip_packet != NULL);
};

bool Host_ExecuteNet::tcpReceive(void)
/** Paket am Anfang der TCP-Empfangsschlange wird entnommen und per
 *  serviceReceive() einem Dienst zugeordnet (falls es ein Dienstpaket ist),
 *  'true' wenn Paket &uuml;bergeben wurde */
{
	bool		received;
	TCP_Packet	*tcp_packet;
	Service_Packet	*data_packet;

	tcp_packet = NULL;
	data_packet = NULL;
	received = false;
	tcp_queue.up_out(tcp_packet);
	if (tcp_packet != NULL)
	{
		cout << "*** Received TCP packet *** " << tcp_packet << endl;
		received = true;
		if (tcp_packet->isProtocol(SERVICE_PACKET))
			data_packet =
				reinterpret_cast<Service_Packet*>(tcp_packet);
		// cout << "Data packet: " << data_packet << endl;
		if (data_packet != NULL) serviceReceive(data_packet);
	};
	// return if packet got from tcp_queue
	return (received);
};

bool Host_ExecuteNet::udpReceive(void)
/** Paket am Anfang der UDP-Empfangsschlange wird entnommen und per
 *  serviceReceive() einem Dienst zugeordnet (falls es ein Dienstpaket ist),
 *  'true' wenn Paket &uuml;bergeben wurde */
{
	bool		received;
	UDP_Packet	*udp_packet;
	Service_Packet	*data_packet;

	udp_packet = NULL;
	data_packet = NULL;
	received = false;
	udp_queue.up_out(udp_packet);
	if (udp_packet != NULL)
	{
		cout << "*** Received UDP packet *** " << udp_packet << endl;
		received = true;
		if (udp_packet->isProtocol(SERVICE_PACKET))
			data_packet =
				reinterpret_cast<Service_Packet*>(udp_packet);
		// cout << "Data packet: " << data_packet << endl;
		if (data_packet != NULL) serviceReceive(data_packet);
	};
	// return if packet got from udp_queue
	return (received);
};

bool Host_ExecuteNet::serviceReceive(Service_Packet *data_packet)
/** Paket wird in TCP_Service_Packet oder UDP_Service_Packet umgewandelt,
 *  ein passender Dienst gesucht: alle Dienste werden abgefragt, ob Sie den
 *  Zielport des Paketes nutzen (Service::usesPort()), der erste Dienst
 *  der zusagt, bekommt das Paket, <br>
 *  liefert immer 'false' !? */
{
	bool				found_used_port;
	unsigned short			destination_port;
	Protocol			tcp_or_udp;
	Service				*current_service;
	list<Service*>::iterator	all_services;
	TCP_Service_Packet		*tcp_service_packet;
	UDP_Service_Packet		*udp_service_packet;

	cout << "Host: Service Receive: " << data_packet << endl;
	tcp_service_packet = NULL;
	udp_service_packet = NULL;
	// transform to TCP_Service_Packet or UDP_Service_Packet
	if (data_packet->isProtocol(TCP))
		tcp_service_packet = reinterpret_cast<TCP_Service_Packet*>
						(data_packet);
	if (data_packet->isProtocol(UDP))
		udp_service_packet = reinterpret_cast<UDP_Service_Packet*>
						(data_packet);
	tcp_or_udp = NO_PROTOCOL;
	destination_port = 0;
	if (tcp_service_packet)
	{
		destination_port = tcp_service_packet->getDestinationPort();
		tcp_or_udp = TCP;
		cout << "Host: TCP_Service_Packet: "
			<< tcp_service_packet << endl;
	};
	if (udp_service_packet)
	{
		destination_port = udp_service_packet->getDestinationPort();
		tcp_or_udp = UDP;
		cout << "Host: UDP_Service_Packet: "
			<< udp_service_packet << endl;
	};
	// find service, which uses target port of packet
	all_services = services.begin();
	found_used_port = false;
	while ((all_services != services.end()) && (!found_used_port))
	{
		current_service = *(all_services);
		cout << "Host: Testing service " << current_service << endl;
		if (current_service)
			if (current_service->usesPort(destination_port,
							tcp_or_udp))
			{
				current_service->in(data_packet);
				found_used_port = true;
			};
		all_services++;
	};
	// else delete packet
	if (!found_used_port)
	{
		delete data_packet;
		cout << "Host_ExecuteNet: Deleting service packet "
			<< data_packet << endl;
	};
	data_packet = NULL;
	return false;
};

void Host_ExecuteNet::importPacket(Packet* cable_packet)
/** importiert Paket 'cable_packet', d.h. es kommt ans Ende der passenden
 *  Warteschlange (TcpQueue oder UdpQueue zum Senden) bzw. wird geroutet mit
 *  intoInterface() */
{
	IPv4_Packet	*ip_packet;
	TCP_Packet	*tcp_packet;
	UDP_Packet	*udp_packet;

	ip_packet = NULL;
	tcp_packet = NULL;
	udp_packet = NULL;

	// cout << "Host::executeNetImportPacket: " << cable_packet << endl;
	if (cable_packet != NULL)
	{
		if (cable_packet->isProtocol(IPv4))
			ip_packet = reinterpret_cast<IPv4_Packet*>
					(cable_packet);
	};
	if (ip_packet)
	{
		if (ip_packet->isProtocol(TCP))
			tcp_packet = reinterpret_cast<TCP_Packet*>(ip_packet);
		else if (ip_packet->isProtocol(UDP))
			udp_packet = reinterpret_cast<UDP_Packet*>(ip_packet);
		else
		{	// only ip packet -> routing
			cout << "Host: imported IPv4 packet." << endl;
			intoInterface(ip_packet);
		};
	};
	if (tcp_packet)
	{	// to end of tcp queue send
		cout << "Host: imported TCP packet." << endl;
		tcp_queue.down_in(tcp_packet);
	};
	if (udp_packet)
	{	// to end of udp queue send
		cout << "Host: imported UDP packet." << endl;
		udp_queue.down_in(udp_packet);
	};
};

void Host_ExecuteNet::executeNet(double speed_factor, double &direction_angle)
/** Ausf&uuml;hren eines Schrittes der Netzwerksimulation, <br>
 *  Geschwindigkeitsfaktor 'speed_factor' f&uuml;r Pakettransport und
 *  Knotenrotation wird an die Schlangen gegeben, jeweils nur eines der Teile:
 *  Dienstpaket senden, TCP-Paket senden, UDP-Paket senden, Simulation der
 *  Netzwerkkarte aufrufen (Interface::executeNet()), TCP-Paket empfangen,
 *  UDP-Paket empfangen, wird (in der Priorit&auml;t dieser Reihenfolge)
 *  ausgef&uuml;hrt <br>
 *  Rotationswinkel der Netzwerkkarte wird mit 'direction_angle'
 *  &uuml;bernommen */
{
	bool	send, received;
	double	new_direction;

	tcp_queue.setAnimationSpeed(speed_factor);
	udp_queue.setAnimationSpeed(speed_factor);
	send = false;
	// cout << "Host sending." << endl;
	if (serviceSend()) send = true;
	else if (tcpSend()) send = true;
	else if (udpSend()) send = true;
	else (network_card.executeNet(speed_factor));
	new_direction = network_card.getAngle();
	if (new_direction != direction_angle)
		direction_angle = new_direction;

	// *** up *** -> Receiving packets
	received = false;
	if (!send)
	{
		// cout << "Host receiving" << endl;
		if (ipReceive()) received = true;
		else if (tcpReceive()) received = true;
		else if (udpReceive()) received = true;
	};
	// cout << "+++++" << endl;
	// cout << "end of execution." << endl;
};

