/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "l3_router_executenet.h"

L3_Router_ExecuteNet::L3_Router_ExecuteNet(void)
/** Konstuktor, ist leer */
{
};

L3_Router_ExecuteNet::~L3_Router_ExecuteNet(void)
/** Destruktor, l&ouml;scht alle Netzwerkkarten */
{
	size_t	i;

	for(i = 0; i < network_cards.size(); i++)
	{
		if (network_cards[i] != NULL) delete network_cards[i];
		network_cards[i] = NULL;
	};
	network_cards.empty();
};

void L3_Router_ExecuteNet::intoInterface(IPv4_Packet* ip_packet)
/** finde eine Route f&uuml;r das Packet 'ip_packet',
 *  d.h. es wird der Routentabelle &uuml;bergeben (solange sie kein Paket
 *  verwaltet, sonst wird es gel&ouml;scht), weitere Verarbeitung in doRoute()
 */
{
	// after Receiving find route for packet
	// cout << "L3_Router_ExecuteNet::intoInterface" << endl;
	if (ip_packet)
		// give packet to route table to find an interface
		if (!route_table.processPacket(ip_packet))
		{
			cout << "Can't process packet." << endl;
			cout << "Deleting: " << ip_packet << endl;
			delete ip_packet;
		};
};

void L3_Router_ExecuteNet::doRoute(double speed_factor)
/** fragt Routentabelle, ob es das Paket schon mit allen Routeneintr&auml;gen
 *  abgeglichen hat (Route_Table::foundRoute() liefert 'true'), nimmt dann
 *  den Namen der Karte aus der Route und &uuml;bergibt das Paket an diese
 *  Karte; oder es wird gel&ouml;scht, wenn keine Karte/Route passt; <br>
 *  wenn es nicht fertig ist, wird die Suche fortgesetzt mit
 *  Route_Table::executeNet() */
{
	size_t		i;
	string		interface_name;
	Route*		found_route;
	Interface*	found_network_card;
	IPv4_Packet*	ip_packet;

	// cout << "L3_Router_ExecuteNet::doRoute" << endl;
	interface_name = "";
	found_route = NULL;
	found_network_card = NULL;
	// all table entries matched against the packet ?
	if (route_table.foundRoute(ip_packet, found_route))
	{
		// get name of interface from found route
		if (found_route) interface_name = found_route->getInterface();
		cout << "Insert in interface: " << interface_name << endl;
		// cout << "Found route: " << found_route << endl;
		// find interface with this name
		if ((interface_name != "") && (found_route != NULL))
			for(i = 0; i < network_cards.size(); i++)
			{
				if (network_cards[i] == NULL) continue;
				if (network_cards[i]->getName() ==
						interface_name)
					found_network_card = network_cards[i];
			};
		// give packet to found interface
		if (found_network_card != NULL)
		{
			ip_packet->setRoute(found_route);
			found_network_card->down_in(ip_packet);
		}
		else
		{	// no route / no card for name in route -> delete
			cout << "No route for packet. Deleting " << ip_packet
				<< endl;
			delete ip_packet;
			ip_packet = NULL;
		};
	}
	// do next step in process of find route (match next route)
	else route_table.executeNet(speed_factor);
};

void L3_Router_ExecuteNet::importPacket(Packet* cable_packet)
/** importiert ein Paket in die Simulation, d.h. ruft intoInterface()
 *  und sucht Route dazu. */
{
	IPv4_Packet	*ip_packet;

	ip_packet = NULL;

	if (cable_packet != NULL)
	{
		if (cable_packet->isProtocol(IPv4))
			ip_packet = reinterpret_cast<IPv4_Packet*>
					(cable_packet);
	};
	if (ip_packet)
	{
		cout << "L3_Router: imported IPv4 packet." << endl;
		intoInterface(ip_packet);
	};
};

void L3_Router_ExecuteNet::executeNet(double speed_factor,
					double &direction_angle)
/** f&uuml;hrt einen Schritt in der Simulation des Netzwerkes aus,
 *  abh&auml;ngig von Geschwindigkeit 'speed_factor', der aktuelle
 *  Rotationswinkel einer der gerade verarbeiteten Karte wird
 *  zur&uuml;ckgegeben; (falls sie ein Paket erwartet/hat),
 *  sonst bleibt der Winkel unver&auml;ndert <br>
 *  zuerst in Routentabelle geschaut, ob ein Paket verwaltet wird
 *  (Route_Table::getPacketPos() liefert true), dann ruft es doRoute() und
 *  kehrt zur&uuml;ck <br>
 *  sonst wird nach Netzwerkkarte gesucht: die erste Karte, welche auf
 *  ein Paket wartet (gerade im Kabel) oder ein Paket hat, wird bevorzugt;
 *  wenn das auf keine zutrifft, kommen alle einmal dran; <br>
 *  f&uuml;r die gew&auml;hlte Karte wird Interface::executeNet() gerufen,
 *  ein eventuell empfangenes Paket (wenn es durch die Schlange durchgelaufen
 *  ist) wird entnommen, &uuml;bersetzt in NAT und
 *  wieder dem Routing zugef&uuml;hrt mit intoInterface()
 */
{
	bool		found;
	size_t		i, found_i;
	double		test_direction, new_direction;
	IPv4_Packet	*ip_packet;
	Point_3D	packet_pos;

	// routing has highest priority
	if (route_table.getPacketPos(packet_pos)) 
	{
		// cout << "L3_Router_ExecuteNet::executeNet"
		//	<< "Route table has packet." << endl;
		doRoute(speed_factor);
		return;
	};
	found = false;
	found_i = 0;
	// then find interface which has a packet / will receive one soon
	for(i = 0; i < network_cards.size(); i++)
	{
		if (network_cards[i] == NULL) continue;

		if ((network_cards[i]->numPacketsOnCable() > 0) ||
			(network_cards[i]->numPackets() > 0))
		{
			found_i = i;
			found = true;
			i = network_cards.size(); // exit for-loop
		};
	};
	// no interface found: find last choosed (same angle)
	for(i = 0; (i < network_cards.size()) && (!found); i++)
	{
		if (network_cards[i] == NULL) continue;

		test_direction = network_cards[i]->getAngle();
		if (test_direction == direction_angle) found_i = i;
	};
	// all interfaces should do one step if no packet is in network
	// round robin ?!
	i = found_i;
	if (!found) i = (i + 1) % network_cards.size();
	while((network_cards[i] == NULL) && (i != found_i))
	{
		i = (i + 1) % network_cards.size();
	};
	if (network_cards[i])
	{
		network_cards[i]->setAngle(direction_angle);

		// cout << "call Interface::executeNet" << endl;
		// one step for the interface
		network_cards[i]->executeNet(speed_factor);
		ip_packet = NULL;
		// cout << "call Interface::up_out" << endl;
		// has it received and animated (gone through the queue)
		// a packet ?
		network_cards[i]->up_out(ip_packet);
		if (ip_packet != NULL)
		{
			cout << "L3_Router: Received ip_packet " 
				<< ip_packet << endl;
			// do NAT and routing for it
			nat_packet_translator.translate(ip_packet);
			intoInterface(ip_packet);
		};

		// take angle of interface if it has a packet / will
		// receive one soon
		new_direction = direction_angle;
		if ((network_cards[i]->numPackets() > 0) ||
			(network_cards[i]->numPacketsOnCable() > 0))
				new_direction = network_cards[i]->getAngle();
		if (new_direction != direction_angle)
			direction_angle = new_direction;
		// cout << "direction angle: " << direction_angle << endl;
	};
	// cout << "+++++" << endl;
};

