/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <netinet/in.h>

#include "constants.h"
#include "ethernet_packet.h"

Ethernet_Packet::Ethernet_Packet(void)
/** Konstruktor, Initialisierung: ETHERTYPE_IP, setProtocol(),
 *  setSourceMacAddress(), setDestinationMacAddress() */
{
	setProtocol(1, ETHERNET);
	eth_header.ether_type = htons(ETHERTYPE_IP);
	setSourceMacAddress(MacAddress());
	setDestinationMacAddress(MacAddress());
};

Ethernet_Packet::~Ethernet_Packet(void)
/** Destruktor, ist leer */
{
	// cout << "destruktor ~Ethernet_Packet" << endl;
};

void Ethernet_Packet::copy(Packet* &copy_of_packet)
/** kopiert den eigenen Paketkopf in den von copy_of_packet;
 *  copy_of_packet wird neu angelegt, wenn es ein NULL-Zeiger ist;
 *  sollte als Objekt der Klasse Ethernet_Packet &uuml;bergeben werden, falls
 *  nicht NULL */
{
	Ethernet_Packet	*eth_packet;

	if (copy_of_packet == NULL) copy_of_packet = new Ethernet_Packet;
	eth_packet = reinterpret_cast<Ethernet_Packet*>(copy_of_packet);
	memcpy(&(eth_packet->eth_header), &eth_header, sizeof(eth_header));
	Packet::copy(copy_of_packet);
	cout << "Ethernet_Packet::copy" << endl;
};

void Ethernet_Packet::loadXML(xmlTextReaderPtr infile)
/** l&auml;dt Paketkopf aus XML-Datei 'infile', ruft Packet::loadXML() */
{
	int		state, depth, return_depth;
	MacAddress	address;

	return_depth = xmlTextReaderDepth(infile);
	do
	{
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		if (xmlStrcasecmp( xmlTextReaderConstName(infile),
			xml_syntax_source_mac) == 0)
		{
			address.loadXML(infile);
			setSourceMacAddress(address);
		}
		else if (xmlStrcasecmp( xmlTextReaderConstName(infile),
			xml_syntax_destination_mac) == 0)
		{
			address.loadXML(infile);
			setDestinationMacAddress(address);
		}
		else if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_packet) == 0) &&
				(xmlTextReaderNodeType(infile) == 1))
			Packet::loadXML(infile);
	} while ((state >= 0) && (depth > return_depth));
};

void Ethernet_Packet::saveXML(xmlTextWriterPtr outfile) const
/** speichert Paketkopf in XML-Datei 'outfile', ruft Packet::saveXML() */
{
	MacAddress	address;

	xmlTextWriterStartElement(outfile, xml_syntax_ethernet_packet);
	xmlTextWriterStartElement(outfile, xml_syntax_source_mac);
	getSourceMacAddress(address);
	address.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	xmlTextWriterStartElement(outfile, xml_syntax_destination_mac);
	getDestinationMacAddress(address);
	address.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	Packet::saveXML(outfile);
	xmlTextWriterEndElement(outfile);
};

void Ethernet_Packet::updatePacketText(void)
/** aktualisiert die dargestellte Paketbeschriftung im 3D-Fenster, aus
 *  den derzeitigen Daten im Paketkopf */
{
	MacAddress	address;

	getDestinationMacAddress(address);
	setTextLine(1, address.getAddress());
};

void Ethernet_Packet::importHeader(struct ether_header *new_eth_header)
/** kopiert den Paketkopf von Zeiger 'new_eth_header' in den intern
 *  gespeicherten Kopf, Beschriftung wird aktualisiert */
{
	if (new_eth_header)
	{
		memcpy(&eth_header, new_eth_header,
			sizeof(struct ether_header));
		updatePacketText();
	};
};

void Ethernet_Packet::exportHeader(
	struct ether_header *current_eth_header) const
/** kopiert den internen Kopf in den Speicherbereich, auf den
 *  'current_eth_header' zeigt */
{
	if (current_eth_header)
	{
		memcpy(current_eth_header, &eth_header,
			sizeof(struct ether_header));
	};
};

bool Ethernet_Packet::setDestinationMacAddress(const char mac_string[])
/** setzt die Ethernet-Zieladresse des Paketes aus dem Text 'mac_string' */
{
	bool			error;
	struct ether_addr	ethernet_address;
	MacAddress		new_destination;

	error = false;
	if (!new_destination.setAddress(mac_string)) error = true;
	else
	{
		new_destination.getAddress(ethernet_address);
		memcpy(eth_header.ether_dhost, 
			ethernet_address.ether_addr_octet, ETH_ALEN);
		updatePacketText();
	};
	return !error;
};

bool Ethernet_Packet::setSourceMacAddress(const char mac_string[])
/** setzt die Ethernet-Quelladresse des Paketes aus dem Text 'mac_string' */
{
	bool			error;
	struct ether_addr	ethernet_address;
	MacAddress		new_source;

	error = false;
	if (!new_source.setAddress(mac_string)) error = true;
	else
	{
		new_source.getAddress(ethernet_address);
		memcpy(eth_header.ether_shost, 
			ethernet_address.ether_addr_octet, ETH_ALEN);
		updatePacketText();
	};
	// cout << "setSourceMac: " << new_source << " " << mac_string << endl;
	return !error;
};

void Ethernet_Packet::setDestinationMacAddress(MacAddress address)
/** &uuml;bernimmt die Adresse aus dem Objekt 'address' als
 *  Ethernet-Zieladresse, ist der Methode mit mac_string
 *  vorzuziehen wenn m&ouml;glich */
{
	struct ether_addr	ethernet_address;
	
	address.getAddress(ethernet_address);
	memcpy(eth_header.ether_dhost,
		ethernet_address.ether_addr_octet, ETH_ALEN);
	updatePacketText();
};

void Ethernet_Packet::setSourceMacAddress(MacAddress address)
/** &uuml;bernimmt die Adresse aus dem Objekt 'address' als
 *  Ethernet-Quelladresse, ist der Methode mit mac_string
 *  vorzuziehen wenn m&ouml;glich */
{
	struct ether_addr	ethernet_address;
	
	address.getAddress(ethernet_address);
	memcpy(eth_header.ether_shost,
		ethernet_address.ether_addr_octet, ETH_ALEN);
	updatePacketText();
};

void Ethernet_Packet::getDestinationMacAddress(MacAddress &address) const
/** gibt die Ethernet-Zieladresse in 'address' zur&uuml;ck */
{
	struct ether_addr	ethernet_address;

	memcpy(ethernet_address.ether_addr_octet, eth_header.ether_dhost, 
		ETH_ALEN);
	address.setAddress(ethernet_address);
};

void Ethernet_Packet::getSourceMacAddress(MacAddress &address) const
/** gibt die Ethernet-Quelladresse in 'address' zur&uuml;ck */
{
	struct ether_addr	ethernet_address;

	memcpy(ethernet_address.ether_addr_octet, eth_header.ether_shost, 
		ETH_ALEN);
	address.setAddress(ethernet_address);
};

