/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "checksums.h"
#include "igmp_packet.h"

IGMP_Packet::IGMP_Packet(void)
/** Konstruktor, Initialisierung: setProtocol(), initPacketHeaders(),
 *  setGroupAddress() */
{
	setProtocol(3, IGMP);
	initPacketHeaders();
	setGroupAddress(IpAddress(INADDR_ANY));
};

IGMP_Packet::~IGMP_Packet(void)
/** Destruktor, ist leer */
{
};

void IGMP_Packet::copy(Packet* &copy_of_packet)
/** kopiert den eigenen Paketkopf in den von copy_of_packet;
 *  copy_of_packet wird neu angelegt, wenn es ein NULL-Zeiger ist;
 *  sollte als Objekt der Klasse IGMP_Packet &uuml;bergeben werden, falls
 *  nicht NULL */
{
	IGMP_Packet	*igmp_packet;

	if (copy_of_packet == NULL) copy_of_packet = new IGMP_Packet;
	igmp_packet = reinterpret_cast<IGMP_Packet*>(copy_of_packet);
	memcpy(&(igmp_packet->igmp_header), &igmp_header, sizeof(igmp_header));
	igmp_packet->updatePacketText();
	IPv4_Packet::copy(copy_of_packet);
	cout << "IGMP_Packet::copy" << endl;
};

void IGMP_Packet::initPacketHeaders(void)
/** setzt Felder im Paketkopf auf initiale Werte, <br>
 *  im IP-Kopf: Protokoll IGMP, Paketl&auml;nge, Pr&uuml;fsumme */
{
	ipv4_header.ip_p = IPPROTO_IGMP;
	ipv4_header.ip_len = htons(sizeof(ipv4_header) + sizeof(igmp_header));
	updateIpChecksum();
	igmp_header.igmp_type = 0;
	igmp_header.igmp_code = 0;
	updateIgmpChecksum();

};

void IGMP_Packet::updateIgmpChecksum(void)
/** berechnet die Pr&uuml;fsumme des Paketkopfes neu */
{
	Checksums::updateIgmpChecksum(&igmp_header);
};

void IGMP_Packet::loadXML(xmlTextReaderPtr infile)
/** ist zur Zeit leer, <br>
 *  l&auml;dt Paketkopf aus XML-Datei 'infile', sollte
 *  IPv4_Packet::loadXML() rufen */
{
};

void IGMP_Packet::saveXML(xmlTextWriterPtr outfile) const
/** ist zur Zeit leer, <br>
 *  speichert Paketkopf in XML-Datei 'outfile', sollte
 *  IPv4_Packet::saveXML() rufen */
{
};

void IGMP_Packet::updatePacketText(void)
/** ist zur Zeit leer, <br>
 *  aktualisiert die dargestellte Paketbeschriftung im 3D-Fenster, aus
 *  den derzeitigen Daten im Paketkopf */
{
};

void IGMP_Packet::importHeader(struct igmp *new_igmp_header)
/** kopiert den Paketkopf von Zeiger 'new_igmp_header' in den intern
 *  gespeicherten Kopf, Beschriftung wird aktualisiert */
{
	if (new_igmp_header)
	{
		memcpy(&igmp_header, new_igmp_header, sizeof(struct igmp));
		updatePacketText();
	};
};

void IGMP_Packet::exportHeader(struct igmp *current_igmp_header) const
/** kopiert den internen Kopf in den Speicherbereich, auf den
 *  'current_igmp_header' zeigt */
{
	if (current_igmp_header)
		memcpy(current_igmp_header, &igmp_header, sizeof(struct igmp));
};

u_int8_t IGMP_Packet::getIgmpType(void) const
/** gibt den Wert des Datenfeldes 'IGMP-Typ' im Paketkopf zur&uuml;ck */
{
	return igmp_header.igmp_type;
};

u_int8_t IGMP_Packet::getRoutingCode(void) const
/** gibt den Wert des Datenfeldes 'IGMP-Code' im Paketkopf zur&uuml;ck */
{
	return igmp_header.igmp_code;
};

void IGMP_Packet::getGroupAddress(IpAddress &group_address) const
/** gibt den Wert des Datenfeldes 'Gruppenadresse' im Paketkopf
 *  in 'group_address' zur&uuml;ck */
{
	group_address.setAddress(igmp_header.igmp_group);
};

void IGMP_Packet::setIgmpType(u_int8_t new_type)
/** setzt das Datenfeld 'IGMP-Typ' auf den Wert von 'new_type',
 *  neue Pr&uuml;fsumme, Beschriftung aktualisiert */
{
	igmp_header.igmp_type = new_type;
	updateIgmpChecksum();
	updatePacketText();
};

void IGMP_Packet::setRoutingCode(u_int8_t new_code)
/** setzt das Datenfeld 'IGMP-Code' auf den Wert von 'new_code',
 *  neue Pr&uuml;fsumme, Beschriftung aktualisiert */
{
	igmp_header.igmp_code = new_code;
	updateIgmpChecksum();
	updatePacketText();
};

void IGMP_Packet::setGroupAddress(IpAddress group_address)
/** setzt das Datenfeld 'Gruppenadresse' auf den Wert von 'group_address',
 *  neue Pr&uuml;fsumme, Beschriftung aktualisiert */
{
	group_address.getAddress(igmp_header.igmp_group);
	updateIgmpChecksum();
	updatePacketText();
};

