/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <iostream>

#include "constants.h"
#include "packet.h"
#include "glut.h"

/** Farbe des Paketes in der Darstellung */
const double packet_color[] = {0.0, 0.0, 1.0, 1.0};
/** Ausma&szlig;e des Paketes in der Darstellung (Breite, H&ouml;he, Tiefe) */
const double packet_size[] = {0.6, 0.25, 1.0};
/** der Unterschied im Farbwert der Schattierungen der dargestellten
 *  K&ouml;pfe */
const double packet_color_difference = 0.2;

/** Verschiebung der Beschriftung in der Breitenachse (X-Achse) vom linken
 *  Rand des Paketes */
const double	packet_text_width_shift = 0.05;
/** Breite eines Buchstaben der Beschriftung (Faktor) */
const double	packet_text_stroke_char_width = 0.03;
/** H&ouml;he eines Buchstaben der Beschriftung (Faktor) */
const double	packet_text_stroke_char_height = 0.045;
/** Breite eines Buchstaben der Beschriftung mit dem Text des vorigen Standes
 *  (Faktor) */
const double	packet_old_text_stroke_char_width = 0.025;
/** H&ouml;he eines Buchstaben der Beschriftung mit dem Text des
 *  vorigen Standes (Faktor) */
const double	packet_old_text_stroke_char_height = 0.0375;
// for Level of Detail distance to camera
/** maximaler Abstand zur Kamera, bei dem mit Level of Detail noch etwas
 *  dargestellt wird */
const double	packet_max_detail_distance = 3.0;

/** Schriftart von GLUT f&uuml;r die Beschriftung, derzeit nicht benutzt */
void*	packet_text_font = GLUT_BITMAP_HELVETICA_18;

using namespace std;

Packet::Packet(void): l1_type(NO_PROTOCOL), l2_type(NO_PROTOCOL),
			l3_type(NO_PROTOCOL), l4_type(NO_PROTOCOL),
			y_angle(0.0), moved_way(0.0),
			display_list_cuboids(0), display_list_text(0)
/** Standard-Konstruktor: Farbe auf 0, Speicher zum Zeichnen anlegen */
{
	setColor(packet_color[0], packet_color[1], packet_color[2],
			packet_color[3]);
	setSize(packet_size[0], packet_size[1], packet_size[2]);
	packet_text[0] = ""; old_packet_text[0] = "";	// l1: Ethernet
	packet_text[1] = ""; old_packet_text[1] = "";	// l2: IP/ARP
	packet_text[2] = ""; old_packet_text[2] = "";	// l3: TCP/UDP
	packet_text[3] = ""; old_packet_text[3] = "";	// l4: content
	// cout << "Packet::Packet()" << endl;
};

Packet::Packet(Protocol new_l3_protocol): 
		l1_type(NO_PROTOCOL), l2_type(NO_PROTOCOL),
		l3_type(new_l3_protocol), l4_type(NO_PROTOCOL),
		y_angle(0.0), moved_way(0.0), display_list_text(0)
/** Standard-Konstruktor: Farbe auf 0, Speicher zum Zeichnen anlegen,
 *  l3_type wird auf Wert von 'new_l3_protocol' gesetzt */
{
	setColor(packet_color[0], packet_color[1], packet_color[2],
			packet_color[3]);
	setSize(packet_size[0], packet_size[1], packet_size[2]);
	packet_text[0] = ""; old_packet_text[0] = "";
	packet_text[1] = ""; old_packet_text[1] = "";
	packet_text[2] = ""; old_packet_text[2] = "";
	packet_text[3] = ""; old_packet_text[3] = "";
	// cout << "Packet::Packet(new_l3)" << endl;
};

Packet::Packet(const Packet &forcopy)
/** Konstruktor, um vorhandenes Paket zu kopieren:
 * alle Werte werden bernommen */
{
	size_t	i;

	l1_type = forcopy.l1_type;
	l2_type = forcopy.l2_type;
	l3_type = forcopy.l3_type;
	l4_type = forcopy.l4_type;
	view = forcopy.view;
	first_pos = forcopy.first_pos;
	y_angle = forcopy.y_angle;
	moved_way = forcopy.moved_way;
	for(i = 0; i < 4; i++) 
	{
		packet_text[i] = forcopy.packet_text[i];
		old_packet_text[i] = forcopy.old_packet_text[i];
		stroke_text_output[i] = forcopy.stroke_text_output[i];
		stroke_old_text_output[i] = forcopy.stroke_old_text_output[i];
	};
	for(i = 0; i < 3; i++) headers[i] = forcopy.headers[i];
	content = forcopy.content;
	display_list_text = 0;
	updateDisplaylistCuboids();
	updateDisplaylistText();
};

Packet::~Packet(void)
/** Destruktor: Speicherbereich vom Zeichnen lschen */
{
	// cout << "~Packet destruktor begin" << endl;
	// cout << "~Packet end" << endl;
	if (glIsList(display_list_text)) glDeleteLists(display_list_text, 1);
};

void Packet::copy(Packet* &copy_of_packet)
/** alle Werte und Einstellungen dieses Paketes werden auf das hinter
 *  'copy_of_packet' &uuml;bertragen, wenn der Zeiger von 'copy_of_packet' NULL
 *  ist, wird ein neues Paket erstellt */
{
	size_t	i;

	if (copy_of_packet == NULL) copy_of_packet = new Packet;
	copy_of_packet->l1_type = l1_type;
	copy_of_packet->l2_type = l2_type;
	copy_of_packet->l3_type = l3_type;
	copy_of_packet->l4_type = l4_type;
	copy_of_packet->view = view;
	copy_of_packet->first_pos = first_pos;
	copy_of_packet->y_angle = y_angle;
	copy_of_packet->moved_way = moved_way;
	for(i = 0; i < 4; i++)
	{
		copy_of_packet->packet_text[i] = packet_text[i];
		copy_of_packet->old_packet_text[i] = old_packet_text[i];
		copy_of_packet->stroke_text_output[i] = stroke_text_output[i];
		copy_of_packet->stroke_old_text_output[i] = 
						stroke_old_text_output[i];
	};
	for(i = 0; i < 3; i++) copy_of_packet->headers[i] = headers[i];
	copy_of_packet->content = content;
	cout << "Packet::copy" << endl;
	updateDisplaylistCuboids();
	updateDisplaylistText();
};

void Packet::loadXMLPos(xmlTextReaderPtr infile, int return_depth,
			Point_3D &pos) const
/** laden eines Positionswertes aus der XML-Datei 'infile' in der derzeitigen
 *  XML-Baumtiefe 'return_depth' in die Positionsvariable 'pos' */
{
	int	state, depth;

	depth = xmlTextReaderDepth(infile);
	while ((xmlStrcasecmp( xmlTextReaderConstName(infile),
		xml_syntax_pos) != 0) &&
		(depth >= return_depth + 1))
	{	// read to <pos>
		state = xmlTextReaderRead(infile); 
		depth = xmlTextReaderDepth(infile);
	};
	pos.loadXML(infile);
};

void Packet::loadXML(xmlTextReaderPtr infile)
/** laden eines Paketes mit allen seinen Einstellungen aus der XML-Datei
 *  'infile' */
{
	unsigned char	level;
	int		state, depth, return_depth;
	char		*value_text, *end_ptr;
	xmlChar		*xml_content;
	Point_3D	header_pos, content_pos;

	return_depth = xmlTextReaderDepth(infile);
	if (xmlTextReaderHasAttributes(infile))
	{	// attributes y_angle and moved_way 
		xml_content = xmlTextReaderGetAttribute(infile,
						xml_syntax_packet_angle);
		value_text = NULL;
		if (xml_content)
			value_text = reinterpret_cast<char*>(xml_content);
		end_ptr = value_text;
		if (value_text)
			y_angle = strtod(value_text, &end_ptr);
		xml_content = xmlTextReaderGetAttribute(infile,
						xml_syntax_packet_moved_way);
		value_text = NULL;
		if (xml_content)
			value_text = reinterpret_cast<char*>(xml_content);
		end_ptr = value_text;
		if (value_text)
			moved_way = strtod(value_text, &end_ptr);

	};
	level = 1;
	do
	{	// Tags first_pos, header_pos, content_pos
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		if (xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_pos) == 0)
			view.loadXML(infile);
		else if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_packet_first_pos) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
				loadXMLPos(infile, return_depth, first_pos);
		else if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_packet_header_pos) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
			{
				loadXMLPos(infile, return_depth, header_pos);
				headers[level - 1].setPos(0, header_pos.getY(),
							header_pos.getZ());
				moveHeader(level, header_pos.getX());
				level++;
				cout << "Packet::loadXML: level: "
					<< static_cast<size_t>(level) << endl;
			}
		else if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_packet_content_pos) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
			{
				loadXMLPos(infile, return_depth, content_pos);
				content.setPos(0.0, content_pos.getY(),
						content_pos.getZ());
				moveHeader(4, content_pos.getX());
			};
	} while((state >= 0) && (depth > return_depth));
	updateDisplaylistCuboids();
	updateDisplaylistText();
};

void Packet::saveXML(xmlTextWriterPtr outfile) const
/** speichern eines Paketes mit allen seinen Einstellungen in der XML-Datei
 *  'outfile' */
{
	unsigned char	level;
	double		pos_x, pos_y, pos_z;
	Point_3D	header_pos, content_pos;

	xmlTextWriterStartElement(outfile, xml_syntax_packet);
	xmlTextWriterWriteFormatAttribute(outfile, xml_syntax_packet_angle,
						"%f", y_angle);
	xmlTextWriterWriteFormatAttribute(outfile, xml_syntax_packet_moved_way,
						"%f", moved_way);
	view.saveXML(outfile);
	xmlTextWriterStartElement(outfile, xml_syntax_packet_first_pos);
	first_pos.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	for(level = 1; level <= 3; level++)
	{
		headers[level - 1].getPos(pos_x, pos_y, pos_z);
		header_pos.setPos(pos_x, pos_y, pos_z);
		xmlTextWriterStartElement(outfile,
						xml_syntax_packet_header_pos);
		header_pos.saveXML(outfile);
		xmlTextWriterEndElement(outfile);
	};
	content.getPos(pos_x, pos_y, pos_z);
	content_pos.setPos(pos_x, pos_y, pos_z);
	xmlTextWriterStartElement(outfile, xml_syntax_packet_content_pos);
	content_pos.saveXML(outfile);
	xmlTextWriterEndElement(outfile);
	xmlTextWriterEndElement(outfile);
};

bool Packet::isProtocol(Protocol test_protocol)
/** ist dieses Paket vom Protokoll 'test_protocol' ? true/false <br>
 *  Werte von l1_type, l2_type, l3_type, l4_type, wurden vorher durch
 *  setProtocol() eingestellt */
{
	bool	test;

	test = (l1_type == test_protocol) || (l2_type == test_protocol);
	test = test || (l3_type == test_protocol) || (l4_type == test_protocol);
	// cout << "This: " << this << " isProtocol: Protocol l4 " << l4_type;
	// cout << " l3 " << l3_type << " l2 " << l2_type << " ";
	// cout << "l1 " << l1_type << endl;
	return test;
};

void Packet::setProtocol(unsigned char level, Protocol new_protocol)
/** setzt den Protokolltyp 'new_protocol' auf Ebene/Schicht 'level'
 * (1 = Subnetz, 2 = Internet, 3 = Transport, 4 = Service) */
{
	// cout << "Setting level " << int(level) 
	// 	<< " to " << new_protocol << endl;
	switch (level)
	{
		case 1: l1_type = new_protocol;	// typical ethernet
			break;
		case 2: l2_type = new_protocol; // typical ipv4
			break;
		case 3: l3_type = new_protocol;	// typical tcp/udp
			break;
		case 4: l4_type = new_protocol; // typical service_packet
			break;
		default: break;
	};
	setColor(view.getR(), view.getG(), view.getB(), view.getA());
};

void Packet::clearOldTextLines(void)
/** l&ouml;scht die gespeicherten alten Beschriftungstexte */
{
	size_t	i;

	for(i = 0; i < 4; i++) old_packet_text[i] = "";
};

string Packet::getTextLine(unsigned char level) const
/** gibt den Text der Beschriftung im angegebenen 'level' zur&uuml;ck,
 *  (1 = Subnetz, 2 = Internet, 3 = Transport, 4 = Service) */
{
	string	return_text;

	return_text = "";
	if ((level >= 1) && (level <= 4)) return_text = packet_text[level - 1];
	return return_text;
};

void Packet::setTextLine(unsigned char level, string new_text)
/** setzt den Text der Beschriftung im angegebenen 'level' auf 'new_text',
 *  alter Text wird gesichert,
 *  (level 1 = Subnetz, 2 = Internet, 3 = Transport, 4 = Service) */
{
	if ((level >= 1) && (level <= 4))
		if (new_text != packet_text[level - 1])
		{
			old_packet_text[level - 1] = packet_text[level - 1];
			packet_text[level - 1] = new_text;
		};
	updateDisplaylistText();
};

void Packet::setPos(double new_centerx, double new_centery, 
		                        double new_centerz)
/** Position des Paketes setzen (Mittelpunkt x, y, z), moved_way wird
 *  entsprechend erh&ouml;ht */
{
	Point_3D	new_pos;

	new_pos.setPos(new_centerx, new_centery, new_centerz);
	moved_way += view.getDistance(new_pos, false);
	view.setPos(new_centerx, new_centery, new_centerz);
};

void Packet::getPos(double &my_centerx, double &my_centery,
					double &my_centerz) const
/** Position des Paketes zurckgeben */
{
	my_centerx = view.getX();
	my_centery = view.getY();
	my_centerz = view.getZ();
};

void Packet::setFirstPos(double new_firstx, double new_firsty,
				double new_firstz)
/** setzt die Position des Paketes, an der es in ein Kabel eingetreten ist,
 *  bestimmt Transportrichtung */
{
	first_pos.setPos(new_firstx, new_firsty, new_firstz);
	moved_way = 0.0;
};

void Packet::getFirstPos(double &my_firstx, double &my_firsty,
				double &my_firstz) const
/** gibt die Position zur&uuml;ck, an der es in ein Kabel eingetreten ist,
 *  bestimmt Transportrichtung */
{
	my_firstx = first_pos.getX();
	my_firsty = first_pos.getY();
	my_firstz = first_pos.getZ();
};

void Packet::setAngle(double new_angle)
/** setzt den Drehwinkel, mit dem das Paket dargestellt wird, 0.0 - 360.0 Grad
 */
{
	y_angle = new_angle;
};

double Packet::getAngle(void) const
/** gibt den Drehwinkel des Paketes zur&uuml;ck */
{
	return y_angle;
};

void Packet::setSize(double new_width, double new_height, double new_depth)
/** Gre des Pakets beim Zeichnen setzen
 *  (Breite new_width, H&ouml;he 'new_height', Tiefe 'new_depth') */
{
	size_t	i;
	double	header_length, text_xpos;

	// cout << "new packet size: " << new_width << endl;
	header_length = new_depth / 4.0;
	headers[0].setSize(new_width, new_height, header_length);
	headers[1].setSize(new_width, new_height, header_length);
	headers[2].setSize(new_width, new_height, header_length);
	content.setSize(new_width, new_height, header_length);
	headers[0].setPos(0.0, 0.0, -1.5 * header_length);
	headers[1].setPos(0.0, 0.0, -0.5 * header_length);
	headers[2].setPos(0.0, 0.0,  0.5 * header_length);
	content.setPos(   0.0, 0.0,  1.5 * header_length);
	// calculate text positions and text sizes with new packet size
	text_xpos = -new_width / 2.0 + packet_text_width_shift;
	for(i = 0; i < 4; i++)
	{
		stroke_text_output[i].setSize(packet_text_stroke_char_width,
					packet_text_stroke_char_height);
		stroke_old_text_output[i].setSize(
					packet_old_text_stroke_char_width,
					packet_old_text_stroke_char_height);
		stroke_text_output[i].setPos(text_xpos, new_height / 1.8,
						(-1.0 + i) * header_length);
		stroke_old_text_output[i].setPos(text_xpos, new_height / 1.8,
						(-1.5 + i) * header_length);
	};
	updateDisplaylistCuboids();
	updateDisplaylistText();
};

void Packet::getSize(double &current_width, double &current_height,
			double &current_depth) const
/** gibt die derzeitige Gr&ouml;&szlig;e des Pakets zur&uuml;ck
 *  (current_width = Breite, current_height = H&ouml;he, current_depth = Tiefe)
 */
{
	current_width = content.getWidth();
	current_height = content.getHeight();
	current_depth = 4 * content.getDepth();
};

void Packet::setColor(double new_red, double new_green, double new_blue,
		                        double new_alpha)
/** Farbe des Paketes setzen, Wertebereich 0.0 - 1.0,
 *  (rot = new_red, gr&uuml;n = new_green, blau = new_blue,
 *  Transparenz = new_alpha alpha = 1.0 ist undurchsichtig),
 *  Quader werden abgestuft schattiert, die Beschriftung erfolgt in der
 *  jeweiligen Konstrastfarbe zum Quader */
{
	size_t	i;
	double	color_difference[4];

	view.setColor(new_red, new_green, new_blue, new_alpha);
	color_difference[0] = 0.0;
	color_difference[1] = color_difference[0];
	if (l4_type != NO_PROTOCOL) color_difference[1] +=
						packet_color_difference;
	color_difference[2] = color_difference[1];
	if (l3_type != NO_PROTOCOL) color_difference[2] +=
						packet_color_difference;
	color_difference[3] = color_difference[2];
	if (l2_type != NO_PROTOCOL) color_difference[3] +=
						packet_color_difference;
	headers[0].setColor(new_red - color_difference[3],
				new_green - color_difference[3],
				new_blue - color_difference[3], new_alpha);
	headers[1].setColor(new_red - color_difference[2],
				new_green - color_difference[2],
				new_blue - color_difference[2], new_alpha);
	headers[2].setColor(new_red - color_difference[1],
				new_green - color_difference[1],
				new_blue - color_difference[1], new_alpha);
	content.setColor(new_red - color_difference[0],
				new_green - color_difference[0],
				new_blue - color_difference[0], new_alpha);
	for(i = 0; i < 4; i++)
	{
		stroke_text_output[i].setColor(
				1.0 - new_red + color_difference[3 - i],
				1.0 - new_green + color_difference[3 - i],
				1.0 - new_blue + color_difference[3 - i],
				new_alpha);
		stroke_old_text_output[i].setColor(
				1.0 - new_red + color_difference[3 - i],
				new_green,
				1.0 - new_blue + color_difference[3 - i],
				new_alpha);
	};
	updateDisplaylistCuboids();
	updateDisplaylistText();
};

void Packet::moveHeader(unsigned char level, double move_x)
/** verschiebt den Quader (Kopf) der Ebene 'level' um 'move_x' nach rechts,
 *  Beschriftung wird mit verschoben,
 *  (level 1 = Subnetz, 2 = Internet, 3 = Transport, 4 = Service) */
{
	double	x_pos, y_pos, z_pos;
	double	text_xpos, text_ypos, header_length;
	Cuboid	*current_cube;

	if ((level >= 1) && (level <= 4))
	{
		if (level <= 3) current_cube = &(headers[level - 1]);
		else current_cube = &content;
		current_cube->getPos(x_pos, y_pos, z_pos);
		x_pos += move_x;
		current_cube->setPos(x_pos, y_pos, z_pos);
		text_xpos = -current_cube->getWidth() / 2.0 
				+ packet_text_width_shift + x_pos;
		text_ypos = current_cube->getHeight() / 1.8;
		header_length = current_cube->getDepth();
		level--;
		stroke_text_output[level].setPos(text_xpos, text_ypos,
					(-1.0 + level) * header_length);
		stroke_old_text_output[level].setPos(text_xpos, text_ypos,
					(-1.5 + level) * header_length);
	};
	updateDisplaylistCuboids();
	updateDisplaylistText();
};

double Packet::getHeaderXPos(unsigned char level) const
/** gibt die Position des Quaders (Kopfes) 'level' auf der X-Achse zur&uuml;ck,
 *  (level 1 = Subnetz, 2 = Internet, 3 = Transport, 4 = Service) */
{
	double	x_pos, y_pos, z_pos;

	x_pos = 0.0;
	if ((level >= 1) && (level <= 4))
	{
		if (level <= 3) headers[level - 1].getPos(x_pos, y_pos, z_pos);
		else content.getPos(x_pos, y_pos, z_pos);
	};
	return x_pos;
};

void Packet::setMovedWay(double new_moved_way)
/** setzt den Wert des zur&uuml;ckgelegten Weges im Kabel/in der Schlange */
{
	moved_way = new_moved_way;
};

double Packet::getMovedWay(void) const
/** gibt den Wert des zur&uuml;ckgelegten Weges im Kabel/in der Schlange
 *  zur&uuml;ck, siehe setMovedWay() und setPos() */
{
	return moved_way;
};

void Packet::updateDisplaylistCuboids(void)
/** zeichnet die Quader f&uuml;r die Displayliste Quader */
{
	size_t		i;
	const Cuboid*	current_cube;

	if (!glIsList(display_list_cuboids)) 
		display_list_cuboids = glGenLists(1);
	if (glIsList(display_list_cuboids))
	{
		glNewList(display_list_cuboids, GL_COMPILE);
		for(i = 1; i <= 4; i++)
		{
			// cout << "cube: " << endl;
			view.colorForDraw();
			if (i == 4) current_cube = &content;
			else current_cube = &(headers[i - 1]);
			current_cube->draw();
		};
		glEndList();
	};
};

void Packet::updateDisplaylistText(void)
/** zeichnet die Beschriftung f&uuml;r die Displayliste Text,
 *  getrennt von den Quadern wegen Level of Detail */
{
	size_t		i;

	if (!glIsList(display_list_text)) 
		display_list_text = glGenLists(1);
	if (glIsList(display_list_text))
	{
		glNewList(display_list_text, GL_COMPILE);
		for(i = 0; i < 4; i++)
		{
			// cout << "text: " << endl;
			stroke_text_output[i].print(packet_text[i].c_str());
			stroke_old_text_output[i].print(
						old_packet_text[i].c_str());
		};
		glEndList();
	};
};

void Packet::draw(bool level_of_details_on, Point_3D camera_pos) const
/** zeichnet Paket, d.h. ruft die Displaylisten entsprechend 'camera_pos'
 *  (Abstand zur Kamera) und 'level_of_details_on', also ob LOD an ist */
{

	// Position, Farbe setzen, Quader zeichnen, Position zurcksetzen
	view.translate(1);
	glRotated(y_angle, 0.0, 1.0, 0.0);
	// cout << "Packet::draw" << endl;
	glCallList(display_list_cuboids);
	if ((!level_of_details_on) ||
		(camera_pos.getDistance(view) <= packet_max_detail_distance))
			glCallList(display_list_text);
	glRotated(-y_angle, 0.0, 1.0, 0.0);
	view.translate(-1);
};

