/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <iostream>

#include "constants.h"
#include "service_packet.h"

using namespace std;

Service_Packet::Service_Packet(void) : service_name(""),
					data_size(0), data(NULL)
/** Konstruktor, Initialisierung: service_name, data_size, data,
 *  ruft setProtocol() */
{
	setProtocol(4, SERVICE_PACKET);
};

Service_Packet::~Service_Packet(void)
/** Destruktor, ist leer */
{
};

void Service_Packet::copy(Packet* &copy_of_packet)
/** kopiert die eigenen Einstellungen (data, data_size, service_name) nach
 *  copy_of_packet;
 *  copy_of_packet wird neu angelegt, wenn es ein NULL-Zeiger ist;
 *  sollte als Objekt der Klasse Service_Packet &uuml;bergeben werden, falls
 *  nicht NULL */
{
	Service_Packet	*service_packet;

	if (copy_of_packet == NULL) copy_of_packet = new Service_Packet;
	service_packet = reinterpret_cast<Service_Packet*>(copy_of_packet);
	service_packet->service_name = service_name;
	service_packet->data_size = data_size;
	if ((data_size > 0) && (data != NULL))
	{
		service_packet->data = new char [data_size];
		memcpy(service_packet->data, data, data_size);
	};
	Packet::copy(copy_of_packet);
	cout << "Service_Packet::copy" << endl;
};

void Service_Packet::loadXML(xmlTextReaderPtr infile)
/** l&auml;dt Paketeinstellungen (nur service_name) aus XML-Datei 'infile',
 *  ruft Packet::loadXML() */
{
	int	state, depth, return_depth;
	xmlChar	*content;

	return_depth = xmlTextReaderDepth(infile);
	if (xmlTextReaderHasAttributes(infile))
	{
		content = xmlTextReaderGetAttribute(infile,
				xml_syntax_service_packet_service_name);
		if (content) service_name = reinterpret_cast<char*>(content);
	};
	do
	{
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_packet) == 0) &&
				(xmlTextReaderNodeType(infile) == 1))
			Packet::loadXML(infile);
	} while ((state >= 0) && (depth > return_depth));
};

void Service_Packet::saveXML(xmlTextWriterPtr outfile) const
/** speichert Paketeinstellungen (nur service_name) in XML-Datei 'outfile',
 *  ruft Packet::saveXML() */
{
	xmlTextWriterStartElement(outfile, xml_syntax_service_packet);
	xmlTextWriterWriteFormatAttribute(outfile,
					xml_syntax_service_packet_service_name,
					"%s", service_name.c_str());
	Packet::saveXML(outfile);
	xmlTextWriterEndElement(outfile);
};

void Service_Packet::setServiceName(const char new_service_name[])
/** setzt den internen Dienstnamen auf 'new_service_name',
 *  Beschriftung wird aktualisiert */
{
	service_name = new_service_name;
	// cout << "Service_Packet: new service name: " << this << " "
	//	<< service_name << endl;
	setTextLine(4, new_service_name);
};

void Service_Packet::print(void)
/** Testmethode, gibt Adresse des Objektes aus (cout << this). */
{
	cout << "Packet: " << this << " ";
};

string Service_Packet::getServiceName(void)
/** gibt den derzeit gesetzten Dienstnamen zur&uuml;ck */
{
	// cout << "Service_Packet::getServiceName: " << service_name << endl;
	return service_name;
};

