/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include "constants.h"
#include "tcp_service_packet.h"

TCP_Service_Packet::TCP_Service_Packet(void) : TCP_Packet(), Service_Packet()
/** Konstruktor, Initialisierung: TCP_Packet(), Service_Packet(),
 *  setProtocol() */
{
	TCP_Packet::setProtocol(4, SERVICE_PACKET);
	Service_Packet::setProtocol(3, TCP);
};

TCP_Service_Packet::~TCP_Service_Packet(void)
/** Destruktor, ist leer */
{
};

void TCP_Service_Packet::copy(Packet* &copy_of_packet)
/** kopiert die Einstellungen des Paketes (service_name, data_size, data) nach
 *  copy_of_packet; copy_of_packet wird neu angelegt, wenn es ein NULL-Zeiger
 *  ist; sollte als Objekt der Klasse TCP_Service_Packet &uuml;bergeben werden,
 *  falls nicht NULL; <br> ruft TCP_Packet::copy() */
{
	TCP_Service_Packet*	new_packet;

	if (copy_of_packet == NULL)
	{
		new_packet = new TCP_Service_Packet;
		new_packet->service_name = service_name;
		new_packet->data_size = data_size;
		if ((data_size > 0) && (data != NULL))
		{
			new_packet->data = new char [data_size];
			memcpy(new_packet->data, data, data_size);
		};
		copy_of_packet = reinterpret_cast<TCP_Packet*>(new_packet);
	};
	cout << "Call TCP_Packet::copy " << copy_of_packet << endl;
	TCP_Packet::copy(copy_of_packet);
	// call of Service_Packet::copy(copy_of_packet) don't work;
	cout << "TCP_Service_Packet::copy" << endl;
};

void TCP_Service_Packet::loadXML(xmlTextReaderPtr infile)
/** l&auml;dt die Paketeinstellungen (nur service_name) aus der XML-Datei
 *  'infile', ruft TCP_Packet::loadXML() */
{
	int	state, depth, return_depth;
	xmlChar	*content;

	return_depth = xmlTextReaderDepth(infile);
	if (xmlTextReaderHasAttributes(infile))
	{
		content = xmlTextReaderGetAttribute(infile,
				xml_syntax_service_packet_service_name);
		if (content) setServiceName(reinterpret_cast<char*>(content));
	};
	do
	{
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_tcp_packet) == 0) &&
				(xmlTextReaderNodeType(infile) == 1))
			TCP_Packet::loadXML(infile);
	} while ((state >= 0) && (depth > return_depth));
};

void TCP_Service_Packet::saveXML(xmlTextWriterPtr outfile) const
/** speichert Paketeinstellungen (nur service_name) in XML-Datei 'outfile',
 *  ruft TCP_Packet::saveXML() */
{
	xmlTextWriterStartElement(outfile, xml_syntax_tcp_service_packet);
	xmlTextWriterWriteFormatAttribute(outfile,
					xml_syntax_service_packet_service_name,
					"%s", service_name.c_str());
	TCP_Packet::saveXML(outfile);
	xmlTextWriterEndElement(outfile);
};

void TCP_Service_Packet::print(void)
/** Testmethode, gibt Adresse des Objektes aus (cout << this). */
{
	cout << "This: " << this << endl;
	Service_Packet::print();
};

void TCP_Service_Packet::setServiceName(const char new_service_name[])
/** setzt den internen Dienstnamen auf 'new_service_name',
 *  Beschriftung wird aktualisiert */
{
	// cout << "TCP_Service_Packet::setServiceName: "
	//	<< new_service_name << endl;
	Service_Packet::setServiceName(new_service_name);
	TCP_Packet::setTextLine(4, new_service_name);
};

string TCP_Service_Packet::getServiceName(void)
/** gibt den derzeit gesetzten Dienstnamen zur&uuml;ck */
{
	// cout << "TCP_Service_Packet::getServiceName." << endl;
	return Service_Packet::getServiceName();
};

