/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include "constants.h"
#include "ipqueue.h"

/** Variablenname der Quelladresse f&uuml;r den Baum ans Dialogfenster */
const char	ipqueue_varname_address[] = "ip address";
/** Farbe der IPv4-Schlange, RGBA 0.0 - 1.0 */
const double	ipqueue_color[4] = {0.0, 0.8, 1.0, 1.0};
/** Text der Beschriftung der Schlange im 3D-Fenster */
const char	ipqueue_label_text[] = "IPv4";

IpQueue::IpQueue(void)
/** Konstruktor, Beschriftungstext gesetzt, ruft setColor() */
{
	label_text = ipqueue_label_text;
	setColor(ipqueue_color[0], ipqueue_color[1], ipqueue_color[2],
			ipqueue_color[3]);
};

IpQueue::~IpQueue(void)
/** Destruktor, l&ouml;scht alle &uuml;brigen Pakete der Sende- und
 *  Empfangsschlangen */
{
	list<IPv4_Packet*>::iterator	all_packets;

	all_packets = down_queue.begin();
	while (all_packets != down_queue.end())
	{	
		// cout << "Deleting: " << *all_packets << endl;
		if (*all_packets != NULL) delete *all_packets;
		*all_packets = NULL;
		all_packets++;
	};
	down_queue.erase(down_queue.begin(), down_queue.end());
	all_packets = up_queue.begin();
	while (all_packets != up_queue.end())
	{	
		if (*all_packets != NULL) delete *all_packets;
		*all_packets = NULL;
		all_packets++;
	};
	up_queue.erase(up_queue.begin(), up_queue.end());
};

void IpQueue::drawPackets(bool level_of_details_on,
				Point_3D camera_pos) const
/** zeichnet jeweils das erste Paket der Sende- und der Empfangsschlange,
 *  ruft nur Net_Queue::drawPackets() */
{
	IPv4_Packet	*type_sample = NULL;

	Net_Queue::drawPackets(type_sample, down_queue, up_queue,
				level_of_details_on, camera_pos);
};

void IpQueue::init(void)
/** Initialisierung, ruft nur Net_Queue::init() */
{
	Net_Queue::init();
};

void IpQueue::setSize(double new_width, double new_height, double new_depth)
/** setzt die Gr&ouml;&szlig;e der gezeichneten Schlange: Breite 'new_width',
 *  H&ouml;he 'new_height' und Tiefe 'new_depth' als Bounding-Box,
 *  ruft Cuboid::setSize() und Net_Queue::updatePos() */
{
	Cuboid::setSize(new_width, new_height, new_depth);
	Net_Queue::updatePos();
};

bool IpQueue::setAddress(const char ip_string[])
/** setzt die neue Quell-IPv4-Adresse auf die in 'ip_string', z.B.
 *  von IpAddress::getAddress(),
 *  'true' zur&uuml;ck wenn &uuml;bernommen, bei Fehler 'false' */
{
	return address.setAddress(ip_string);
};

char* IpQueue::getAddress(void) const
/** gibt die Quell-IPv4-Adresse als Textfeld zur&uuml;ck,
 *  ruft IpAddress::getAddress() */
{
	return address.getAddress();
};

void IpQueue::getAddress(IpAddress &current_address) const
/** gibt die Quell-IPv4-Adresse &uuml;ber 'current_address' zur&uuml;ck  */
{
	current_address = address;
};

void IpQueue::getInfos(Variable_Tree* &infos)
/** stellt den Baum f&uuml;r das Dialogfenster zusammen, alter Baum wird
 *  hier gel&ouml;scht, R&uuml;ckgabe des Zeigers auf den Baum mit 'infos' */
{
	Numeric_Variable<IpAddress>	*var_address;

	current_infos.clear();
	var_address = new Numeric_Variable<IpAddress>;
	if (var_address)
	{
		var_address->setName(ipqueue_varname_address);
		var_address->setLength(1, 0);
		var_address->setMaxmin(INADDR_NONE, INADDR_ANY);
		var_address->setValue(address);
		current_infos.addVar(var_address);
	};
	infos = &current_infos;
};

void IpQueue::setInfos(Variable_Tree* infos)
/** liest die aktuellen Werte aus dem Baum vom Dialogfenster aus,
 *  'infos' sollte darauf zeigen, es wird aber nur die interne Struktur
 *  benutzt, Wert von 'infos' ist noch egal */
{
	string		value_text;
	Variable	*current_var;

	if (current_infos.findVar(ipqueue_varname_address, current_var))
	{
		value_text = current_var->getValue();
		value_text[ value_text.size() ] = 0;
		address.setAddress(value_text.c_str());
	};
};

void IpQueue::loadXML(xmlTextReaderPtr infile)
/** l&auml;dt Pakete und Einstellungen aus der XML-Datei 'infile' */
{
	int     	state, depth, return_depth;
                      
	address.loadXML(infile);
	return_depth = xmlTextReaderDepth(infile);
	do
	{
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_down_queue) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
		{	// nodeType: start of node, not end
			loadXMLPackets<IPv4_Packet>(infile, down_queue);
		}
		else if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_up_queue) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
		{	// nodeType: start of node, not end
			loadXMLPackets<IPv4_Packet>(infile, up_queue);
		};
	} while ((state >= 0) && (depth > return_depth));
};

void IpQueue::saveXML(xmlTextWriterPtr outfile) const
/** speichert Pakete und Einstellungen in die XML-Datei 'outfile' */
{
	list<IPv4_Packet*>::const_iterator	all_packets;

	xmlTextWriterStartElement(outfile, xml_syntax_ip_queue);
	address.saveXML(outfile);
	xmlTextWriterStartElement(outfile, xml_syntax_down_queue);
	all_packets = down_queue.begin();
	while (all_packets != down_queue.end())
	{	
		if (*all_packets != NULL) (*all_packets)->saveXML(outfile);
		all_packets++;
	};
	xmlTextWriterEndElement(outfile);
	xmlTextWriterStartElement(outfile, xml_syntax_up_queue);
	all_packets = up_queue.begin();
	while (all_packets != up_queue.end())
	{	
		if (*all_packets != NULL) (*all_packets)->saveXML(outfile);
		all_packets++;
	};
	xmlTextWriterEndElement(outfile);
	xmlTextWriterEndElement(outfile);
};

size_t IpQueue::down_size(void) const
/** gibt die L&auml;nge der Sendeschlange zur&uuml;ck */
{
	return down_queue.size();
};

void IpQueue::down_in(IPv4_Packet* incoming_packet)
/** neues Paket 'incoming_packet' zum Senden, kommt ans Ende der Schlange,
 *  ruft u.a. Net_Queue::down_in() */
{
	Net_Queue::down_in(incoming_packet, 2);
	if (incoming_packet != NULL)
	{
		down_queue.insert(down_queue.end(), incoming_packet);
	};
};

void IpQueue::down_out(IPv4_Packet* &outgoing_packet)
/** gibt erstes Paket 'outgoing_packet' der Sendeschlange zur&uuml;ck, wenn es
 *  in der Animation am Ende angekommen ist, sonst NULL,
 *  ruft u.a. Net_Queue::down_movePackets() und Net_Queue::down_out() */
{
	if (down_queue.size() > 0)
	{
		outgoing_packet = *down_queue.begin();
		down_movePackets<IPv4_Packet,
			list<IPv4_Packet*>::iterator>(2, down_queue);
		if (Net_Queue::down_out(outgoing_packet))
			down_queue.erase(down_queue.begin());
		else outgoing_packet = NULL;
	}
	else outgoing_packet = NULL;
};

size_t IpQueue::up_size(void) const
/** gibt die L&auml;nge der Empfangsschlange zur&uuml;ck */
{
	return up_queue.size();
};

void IpQueue::up_in(IPv4_Packet* incoming_packet)
/** neues Paket 'incoming_packet' zum Empfangen, kommt ans Ende der Schlange,
 *  ruft u.a. Net_Queue::up_in() */
{
	Net_Queue::up_in(incoming_packet);
	if (incoming_packet != NULL)
		up_queue.insert(up_queue.end(), incoming_packet);
};

void IpQueue::up_out(IPv4_Packet* &outgoing_packet)
/** gibt erstes Paket 'outgoing_packet' der Empfangsschlange zur&uuml;ck, wenn
 *  es in der Animation am Ende angekommen ist, sonst NULL,
 *  ruft u.a. Net_Queue::up_movePackets() und Net_Queue::up_out() */
{
	if (up_queue.size() > 0)
	{
		outgoing_packet = *up_queue.begin();
		up_movePackets<IPv4_Packet,
			list<IPv4_Packet*>::iterator>(2, up_queue);
		if (Net_Queue::up_out(outgoing_packet))
			up_queue.erase(up_queue.begin());
		else outgoing_packet = NULL;
	}
	else outgoing_packet = NULL;
};

bool IpQueue::getFirstPacketPos(Point_3D &packet_pos) const
/** gibt Position 'packet_pos' des ersten Paketes der Sende- bzw.
 *  Empfangsschlange (falls keines zum Senden) zur&uuml;ck, false wenn
 *  kein Paket da, <br> ruft Net_Queue::getFirstPacketPos() */
{
	IPv4_Packet	*type_sample = NULL;

	return Net_Queue::getFirstPacketPos(type_sample, packet_pos,
						down_queue, up_queue);
};

