/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

template <class PacketClass>
void Net_Queue::internalLoadXMLPacket(xmlTextReaderPtr infile,
					const xmlChar xml_tag_name[],
					Packet* &new_packet)
/** Laden eines Datenpaketes der Klasse 'PacketClass' aus der XML-Datei
 *  'infile', dessen Beschreibung beginnt mit Tag 'xml_tag_name';<br>
 *  R&uuml;ckgabe in Zeiger 'new_packet' wenn ordentlich geladen, sonst
 *  bleibt Zeiger unver&auml;ndert, <br> ruft PacketClass::loadXML() */
{
	PacketClass	*new_class_packet;

	new_class_packet = NULL;
	if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
				xml_tag_name) == 0) &&
		(xmlTextReaderNodeType(infile) == 1))
	{	// nodeType: start of node, not end
		new_class_packet = new PacketClass;
		if (new_class_packet)
		{
			new_class_packet->loadXML(infile);
			new_packet =
				reinterpret_cast<Packet*>(new_class_packet);
		};
	};
};

template <class PacketClass>
void Net_Queue::loadXMLPackets(xmlTextReaderPtr infile,
				list<PacketClass*> &up_down_queue)
/** laden aller Pakete unterhalb des aktuellen Tags aus XML-Datei 'infile'
 *  in die Liste 'up_down_queue' des Typs 'PacketClass' (eingef&uuml;gt am
 *  Ende), ruft loadPacketClasses() */
{
	int     	state, depth, return_depth;
	Packet		*new_packet;
	PacketClass	*net_queue_packet;

	return_depth = xmlTextReaderDepth(infile);
	do
	{
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		new_packet = NULL;
		loadPacketClasses(infile, new_packet);
		net_queue_packet = NULL;
		if (new_packet)
			net_queue_packet =
				reinterpret_cast<PacketClass*>(new_packet);
		if (net_queue_packet)
			up_down_queue.insert(up_down_queue.end(),
						net_queue_packet);
	} while ((state >= 0) && (depth > return_depth));
};

template <class PacketClass>
bool Net_Queue::getFirstPacketPos(PacketClass *type_sample,
					Point_3D &packet_pos,
					const list<PacketClass*> &down_queue,
					const list<PacketClass*> &up_queue)
/** gibt Position 'packet_pos' des ersten Paketes der Sendeschlange
 *  'down_queue' des Typs 'PacketClass' zur&uuml;ck; <br> wenn diese leer: die
 *  Position des ersten Paketes der Empfangsschlange 'up_queue'; <br>
 *  wenn beide leer: R&uuml;ckgabe 'false' und 'packet_pos' bleibt
 *  unver&auml;ndert', <br>
 *  'type_sample' ist nur eine Hilfsvariable, mit der Windows die
 *  'PacketClass' bestimmen kann (kann keine richtigen Funktionstemplates) */
{
	bool			has_packet;
	double			posx, posy, posz;
	const PacketClass	*first_packet;

	first_packet = NULL;
	has_packet = false;
	if (down_queue.size() > 0) first_packet = *down_queue.begin();
	else if (up_queue.size() > 0) first_packet = *up_queue.begin();
	if (first_packet)
	{
		first_packet->getPos(posx, posy, posz);
		packet_pos.setPos(posx, posy, posz);
		has_packet = true;
	};
	return has_packet;
};

template <class PacketClass>
void Net_Queue::drawPackets(PacketClass *type_sample,
				const list<PacketClass*> &down_queue,
				const list<PacketClass*> &up_queue,
				bool level_of_details_on,
				Point_3D camera_pos) const
/** zeichnet jeweils das erste Paket der Sendeschlange 'down_queue' und das
 *  erste der Empfangsschlange 'up_queue', d.h. ruft deren draw-Methode mit
 *  &uuml;bergebenen 'level_of_details_on' und 'camera_pos', <br>
 *  'type_sample' ist nur eine Hilfsvariable, mit der Windows die
 *  'PacketClass' bestimmen kann (kann keine richtigen Funktionstemplates) */
{
	if (down_queue.size() > 0)
		if (*down_queue.begin() != NULL)
		{
			(*down_queue.begin())->draw(level_of_details_on,
							camera_pos);
		};
	if (up_queue.size() > 0)
		if (*up_queue.begin() != NULL)
		{
			(*up_queue.begin())->draw(level_of_details_on,
							camera_pos);
		};
};

template <class PacketClass, typename IteratorType>
void Net_Queue::down_movePackets(unsigned char level,
					list<PacketClass*> &down_queue)
/** verschiebt alle Pakete der Sendeschlange 'down_queue', d. h. ruft
 *  jeweils movePacket() mit direction '-1' und &uuml;bergebenem 'level',
 *  'IteratorType' ist der Typ des Iterators der Schlange, meist
 *  list<PacketClass*>::iterator,
 *  gerufen von down_out der abgeleiteten Klassen */
{
	IteratorType	all_packets;

	all_packets = down_queue.begin();
	while (all_packets != down_queue.end())
	{	
		if (*all_packets != NULL) movePacket(*all_packets, -1, level);
		all_packets++;
	};
};

template <class PacketClass, typename IteratorType>
void Net_Queue::up_movePackets(unsigned char level,
				list<PacketClass*> &up_queue)
/** verschiebt alle Pakete der Empfangsschlange 'up_queue', d. h. ruft
 *  jeweils movePacket() mit direction '1' und &uuml;bergebenem 'level',
 *  'IteratorType' ist der Typ des Iterators der Schlange, meist
 *  list<PacketClass*>::iterator,
 *  gerufen von up_out der abgeleiteten Klassen */
{
	IteratorType	all_packets;

	all_packets = up_queue.begin();
	while (all_packets != up_queue.end())
	{	
		if (*all_packets != NULL) movePacket(*all_packets, 1, level);
		all_packets++;
	};
};

