/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "windows32.h"

#include <netdb.h>

#include "constants.h"
#include "udp_service_packet.h"
#include "udpqueue.h"

/** Farbe der UDP-Schlange, RGBA 0.0 - 1.0 */
const double	udpqueue_color[4] = {0.0, 1.0, 1.0, 1.0};
/** Text der Beschriftung der Schlange im 3D-Fenster */
const char	udpqueue_label_text[] = "   /UDP";

UdpQueue::UdpQueue(void)
/** Konstruktor, Beschriftungstext gesetzt, ruft setColor() */
{
	label_text = udpqueue_label_text;
	setColor(udpqueue_color[0], udpqueue_color[1], udpqueue_color[2],
			udpqueue_color[3]);
};

UdpQueue::~UdpQueue(void)
/** Destruktor, l&ouml;scht alle &uuml;brigen Pakete der Sende- und
 *  Empfangsschlangen */
{
	list<UDP_Packet*>::iterator	all_packets;

	all_packets = down_queue.begin();
	while (all_packets != down_queue.end())
	{	
		if (*all_packets != NULL) delete *all_packets;
		*all_packets = NULL;
		all_packets++;
	};
	down_queue.erase(down_queue.begin(), down_queue.end());
	all_packets = up_queue.begin();
	while (all_packets != up_queue.end())
	{	
		if (*all_packets != NULL) delete *all_packets;
		*all_packets = NULL;
		all_packets++;
	};
	up_queue.erase(up_queue.begin(), up_queue.end());
};

void UdpQueue::drawPackets(bool level_of_details_on, Point_3D camera_pos) const
/** zeichnet jeweils das erste Paket der Sende- und der Empfangsschlange,
 *  ruft nur Net_Queue::drawPackets() */
{
	UDP_Packet	*type_sample = NULL;

	Net_Queue::drawPackets(type_sample, down_queue, up_queue,
				level_of_details_on, camera_pos);
};

void UdpQueue::init(void)
/** Initialisierung, ruft nur Net_Queue::init() */
{
	Net_Queue::init();
};

void UdpQueue::setSize(double new_width, double new_height, double new_depth)
/** setzt die Gr&ouml;&szlig;e der gezeichneten Schlange: Breite 'new_width',
 *  H&ouml;he 'new_height' und Tiefe 'new_depth' als Bounding-Box,
 *  ruft Cuboid::setSize() und Net_Queue::updatePos() */
{
	Cuboid::setSize(new_width, new_height, new_depth);
	Net_Queue::updatePos();
};

void UdpQueue::getInfos(Variable_Tree* &infos)
/** zur Zeit leer, <br> stellt den Baum f&uuml;r das Dialogfenster zusammen,
 *  alter Baum wird hier gel&ouml;scht, R&uuml;ckgabe des Zeigers auf den
 *  Baum mit 'infos' */
{
};

void UdpQueue::setInfos(Variable_Tree* infos)
/** zur Zeit leer, <br>
 * liest die aktuellen Werte aus dem Baum vom Dialogfenster aus,
 *  'infos' sollte darauf zeigen, es wird aber nur die interne Struktur
 *  benutzt, Wert von 'infos' ist noch egal */
{
};

void UdpQueue::loadXML(xmlTextReaderPtr infile)
/** l&auml;dt Pakete und Einstellungen aus der XML-Datei 'infile' */
{
	int     	state, depth, return_depth;
                      
	return_depth = xmlTextReaderDepth(infile);
	do
	{
		state = xmlTextReaderRead(infile);
		depth = xmlTextReaderDepth(infile);
		if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_down_queue) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
		{	// nodeType: start of node, not end
			loadXMLPackets<UDP_Packet>(infile, down_queue);
		}
		else if ((xmlStrcasecmp( xmlTextReaderConstName(infile),
					xml_syntax_up_queue) == 0) &&
			(xmlTextReaderNodeType(infile) == 1))
		{	// nodeType: start of node, not end
			loadXMLPackets<UDP_Packet>(infile, up_queue);
		};
	} while ((state >= 0) && (depth > return_depth));
};

void UdpQueue::saveXML(xmlTextWriterPtr outfile) const
/** speichert Pakete und Einstellungen in die XML-Datei 'outfile' */
{
	list<UDP_Packet*>::const_iterator	all_packets;

	xmlTextWriterStartElement(outfile, xml_syntax_udp_queue);
	xmlTextWriterStartElement(outfile, xml_syntax_down_queue);
	all_packets = down_queue.begin();
	while (all_packets != down_queue.end())
	{	
		if (*all_packets != NULL) (*all_packets)->saveXML(outfile);
		all_packets++;
	};
	xmlTextWriterEndElement(outfile);
	xmlTextWriterStartElement(outfile, xml_syntax_up_queue);
	all_packets = up_queue.begin();
	while (all_packets != up_queue.end())
	{	
		if (*all_packets != NULL) (*all_packets)->saveXML(outfile);
		all_packets++;
	};
	xmlTextWriterEndElement(outfile);
	xmlTextWriterEndElement(outfile);
};

unsigned short UdpQueue::findServicePort(string service_name)
/** sucht den Port zum Dienst 'service_name' durch getservbyname(),
 *  R&uuml;ckgabe '0' oder gefundener Port */
{
	unsigned short	service_port;
	struct servent	*service_info;

	service_port = 0;
	service_info = getservbyname(service_name.c_str(), "udp");
	if (service_info)
		service_port = ntohs(service_info->s_port);
	// cout << service_info->s_name << " " << service_info->s_proto << endl;
	// cout << "service name: " << service_name << " ";
	// cout << "service port: " << service_port << endl;
	return service_port;
};

size_t UdpQueue::down_size(void) const
/** gibt die L&auml;nge der Sendeschlange zur&uuml;ck */
{
	return down_queue.size();
};

void UdpQueue::down_in(UDP_Packet* incoming_packet)
/** neues Paket 'incoming_packet' zum Senden, kommt ans Ende der Schlange;
 *  setzt die Portnummer durch findServicePort(), falls sie auf '0' war,
 *  ruft u.a. Net_Queue::down_in() */
{
	UDP_Service_Packet	*data_packet;
	unsigned short		service_port;
	string			service_name;

	Net_Queue::down_in(incoming_packet, 3);
	if (incoming_packet != NULL)
	{
		if (incoming_packet->isProtocol(SERVICE_PACKET))
		{
			service_name = "";
			data_packet = reinterpret_cast<UDP_Service_Packet*>
					(incoming_packet);
			if (data_packet) 
				service_name = data_packet->getServiceName();
			service_port = findServicePort(service_name);
			if (incoming_packet->getSourcePort() == 0)
				incoming_packet->setSourcePort(service_port);
		};
		down_queue.insert(down_queue.end(), incoming_packet);
	};
};

void UdpQueue::down_out(UDP_Packet* &outgoing_packet)
/** gibt erstes Paket 'outgoing_packet' der Sendeschlange zur&uuml;ck, wenn es
 *  in der Animation am Ende angekommen ist, sonst NULL,
 *  ruft u.a. Net_Queue::down_movePackets() und Net_Queue::down_out() */
{
	if (down_queue.size() > 0)
	{
		outgoing_packet = *down_queue.begin();
		down_movePackets<UDP_Packet,
			list<UDP_Packet*>::iterator>(3, down_queue);
		if (Net_Queue::down_out(outgoing_packet))
			down_queue.erase(down_queue.begin());
		else outgoing_packet = NULL;
	}
	else outgoing_packet = NULL;
};

size_t UdpQueue::up_size(void) const
/** gibt die L&auml;nge der Empfangsschlange zur&uuml;ck */
{
	return up_queue.size();
};

void UdpQueue::up_in(UDP_Packet* incoming_packet)
/** neues Paket 'incoming_packet' zum Empfangen, kommt ans Ende der Schlange,
 *  ruft u.a. Net_Queue::up_in() */
{
	Net_Queue::up_in(incoming_packet);
	if (incoming_packet != NULL)
		up_queue.insert(up_queue.end(), incoming_packet);
};

void UdpQueue::up_out(UDP_Packet* &outgoing_packet)
/** gibt erstes Paket 'outgoing_packet' der Empfangsschlange zur&uuml;ck, wenn
 *  es in der Animation am Ende angekommen ist, sonst NULL,
 *  ruft u.a. Net_Queue::up_movePackets() und Net_Queue::up_out() */
{
	if (up_queue.size() > 0)
	{
		outgoing_packet = *up_queue.begin();
		up_movePackets<UDP_Packet,
			list<UDP_Packet*>::iterator>(3, up_queue);
		if (Net_Queue::up_out(outgoing_packet))
			up_queue.erase(up_queue.begin());
		else outgoing_packet = NULL;
	}
	else outgoing_packet = NULL;
};

bool UdpQueue::getFirstPacketPos(Point_3D &packet_pos) const
/** gibt Position 'packet_pos' des ersten Paketes der Sende- bzw.
 *  Empfangsschlange (falls keines zum Senden) zur&uuml;ck, false wenn
 *  kein Paket da, <br> ruft Net_Queue::getFirstPacketPos() */
{
	UDP_Packet	*type_sample = NULL;

	return Net_Queue::getFirstPacketPos(type_sample, packet_pos,
						down_queue, up_queue);
};

