/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/
#include "variable.h"

Variable::Variable(void) : constant(false)
/** Konstruktor, Initialisierung von Variable::constant auf 'false' */
{
};

Variable::~Variable(void)
/** Destruktor, ist leer */
{
};

/**************************************************************************/

String_Variable::String_Variable(void) : 
	min_length(0), max_length(255), value("")
/** Konstruktor, Initialisierung: minimale L&auml;nge '0',
 *  maximale L&auml;nge '255' und Leerstring "" als Wert */
{
};

String_Variable::~String_Variable(void)
/** Destruktor, ist leer */
{
};

bool String_Variable::isConstant(void)
/** gibt 'true' zur&uuml;ck, wenn der Wert (Inhalt) der Variablen konstant
 *  ist, also unver&auml;nderbar, sonst 'false' */
{
	return constant;
};

string String_Variable::getName(void)
/** gibt den Namen der Variable zur&uuml;ck */
{
	return name;
};

string String_Variable::getValue(void)
/** gibt den Wert (den Textinhalt) der Variablen als String zur&uuml;ck */
{
	return value;
};

void String_Variable::setConstant(bool is_constant)
/**
 *  - mit 'is_constant' auf 'true' wird der Wert (Inhalt) der Variablen als
 *    konstant angesehen; solange, bis es &uuml;ber diese Methode wieder
 *    ge&auml;ndert wird
 *  - mit 'is_constant' auf 'false' ist der Wert (Inhalt) innerhalb der
 *    L&auml;ngenbeschr&auml;nkungen beliebig anpassbar */
{
	constant = is_constant;
};

void String_Variable::setName(string new_name)
/** setzt den Namen der Variable auf 'new_name' */
{
	name = new_name;
}

bool String_Variable::setLength(size_t new_max, size_t new_min)
/** setzt die neue maximale 'new_max' und minimale 'new_min' L&auml;nge des
 *  Textfeldes, die es annehmen kann; <br>
 *  der aktuelle Inhalt wird nicht gepr&uuml;ft, nur die &Auml;nderungen
 *  &uuml;ber die Methode setValue(); <br>
 *  funktioniert auch, wenn der Inhalt als konstant markiert ist;<br>
 *  gibt 'true' zur&uuml;ck, wenn die neuen L&auml;ngen &uuml;bernommen wurden,
 *  sonst 'false' (Fehler, zur Zeit nur, wenn 'new_min > 'new_max') */
// min and max length of string
{
	if (new_max >= new_min)	{
		min_length = new_min;
		max_length = new_max;
		return true;
	}
	else	return false;
};

bool String_Variable::setValue(string new_value)
/** setzt den Wert (Textinhalt) der Variable auf 'new_value', <br>
 *  gibt 'true' zur&uuml;ck, wenn er &uuml;bernommen wurde,
 *  sonst 'false' (wegen L&auml;ngenbeschr&auml;nkung, zu kurz / zu lang
 *  oder String_Variable ist konstant) */
{
	bool	test_minmax;

	// not changeable if constant==true
	test_minmax = !constant;
	// not changeable if length is wrong
	if ((new_value.length() < min_length) || 
		(new_value.length() > max_length))
			test_minmax = false;
	if (test_minmax) value = new_value;
	return test_minmax;
}

ostream& operator << (ostream& output, String_Variable &var)
/** schreibt den Wert (Inhalt) der Variablen 'var' auf den Stream 'output',
 *  <br> Referenz auf den Stream wird zur&uuml;ckgegeben */
{
	output << var.getValue();
	return output;
};

