/******************************************************************************
* This file is part of netview 
* (an program to animate TCP/IP networks in 3D with OpenGL).
*
*    Copyright (C) 2004,2005 Stefan Ziegler.
*
*    netview is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    netview is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with netview; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#include <direct.h>
#include <math.h>

#include "win_net.h"

/** @file win_funcs.cpp Datei stellt Funktionen bereit, die in
 *  Unix/Linux-Umgebungen
 *  verf&uuml;gbar sind, aber nicht in Windows. Sie werden mit Funktionen
 *  nachgebildet, die Windows bietet */

int inet_aton(const char *cp, struct in_addr * inp)
/** Umwandeln eines Adress-Strings (cp) in eine numerische Adresse (inp),
 *  R&uuml;ckgabewert immer 1,
 *  nutzt Funktion inet_addr(const char*) <br>
 *  nutze Headerdatei <win_net.h> */
{
	inp->S_un.S_addr = inet_addr(cp);
	return 1;
};

double rint(double x)
/** Runden einer Gleitkommazahl (x) auf n&auml;chste ganze Zahl (Return-Wert), 
 *  nutzt double modf(double, double*) <br>
 *  nutze Headerdatei <windows32.h> */
{
	double x_int, x_frac;

	// Aufteilen in ganzzahligen und Nachkomma-Anteil
	x_frac = modf(x, &x_int);
	// bei Bedarf entsprechend auf-/abrunden
	if (x_frac >= 0.5) x_int += 1.0;
	if (x_frac <= -0.5) x_int -= 1.0;
	return x_int;
};

void usleep(unsigned long time)
/** Prozess soviele Mikrosekunden schlafen legen, wird gerundet auf 
 *  Millisekunden, nutzt Sleep(long time_in_Millisekunden), <br>
 *  nutze Headerdatei <windows32.h> */
{
	Sleep(time / 1000);
};

void mkdir(const char* name, int rights)
/** Anlegen des Verzeichnisses 'name', nutzt mkdir(name), <br>
 *  nutze Headerdatei <windows32.h> */
{
	mkdir(name);
};

int close(int socket)
/** schlie&szlig;t den Socket 'socket' mit closesocket(), <br>
 *  nutze Headerdatei <win_net.h> */
{
	return closesocket(socket);
};

int fcntl(int socket, int operation, int parameter)
/** setzt nicht-blockierende Ein-/Ausgabe auf dem Socket 'socket', wenn
 *  'operation' == 'F_SETFL' und 'parameter' == 'O_NONBLOCK', <br>
 *  es werden keine anderen Operationen von fcntl unterst&uuml;tzt, <br>
 *  nutze Headerdatei <win_net.h> */
{
	int	error;
	unsigned long	param;

	error = -1;
	if ((operation == F_SETFL) && (parameter == O_NONBLOCK))
	{
		param = 1;
		error = ioctlsocket(socket, FIONBIO, &param);
	};
	return error;
};

int init_socket(void)
/** initialisiert die Socket-Umgebung unter MS Windows, gibt Wert von
 *  WSAStartup zur&uuml;ck; <br>
 *  nutze Headerdatei <win_net.h> */
{
	WORD	requestedVersionNumber;
	WSADATA	wsaStartupData;

	// initialize socket environment in MS Windows
	// WSACleanup () needed ?
	requestedVersionNumber = MAKEWORD(2, 0);
	return WSAStartup(requestedVersionNumber, &wsaStartupData);
};

